@@ -12,14 +12,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_build.c,v 1.3 2003/06/19 22:11:35 ken Exp $
+ * RCSID $Id: pfkey_v2_build.c,v 1.2 2003/09/12 19:23:31 ken Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_parser.c.
  */
 
-char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.3 2003/06/19 22:11:35 ken Exp $";
+char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.2 2003/09/12 19:23:31 ken Exp $";
 
 /*
  * Some ugly stuff to allow consistent debugging code for use in the
@@ -59,19 +59,18 @@ char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.3 2003/06/19 22:11:
 # include <string.h> /* memset */
 
 # include <freeswan.h>
-# include "programs/pluto/constants.h" /* XXX this is crap */
-# include "programs/pluto/defs.h"      /* for PRINTF_LIKE */
-
-extern unsigned int debugging;  /* bits selecting what to report */
 unsigned int pfkey_lib_debug = 0;
 
+void (*pfkey_debug_func)(const char *message, ...) PRINTF_LIKE(1);
+
 /* #define PLUTO */
 
-# ifdef PLUTO
-#  define DEBUGGING(args...)  { DBG_log("pfkey_lib_debug:" args);  }
-# else
-#  define DEBUGGING(args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" args); } else { ; }
-# endif
+#define DEBUGGING(args...)  if(pfkey_lib_debug) { \
+                              if(pfkey_debug_func != NULL) { \
+                                (*pfkey_debug_func)("pfkey_lib_debug:" args); \
+                              } else { \
+                                printf("pfkey_lib_debug:" args); \
+                              } }
 # define MALLOC(size) malloc(size)
 # define FREE(obj) free(obj)
 #endif /* __KERNEL__ */
@@ -87,7 +86,6 @@ unsigned int pfkey_lib_debug = 0;
 
 # define DEBUGGING(args...) \
          KLIPS_PRINT(debug_pfkey, "klips_debug:" args)
-/*         ((debug_pfkey) ? printk(KERN_INFO "klips_debug:" format , ## args) : 0) */
 #endif /* __KERNEL__ */
 
 #include "freeswan/ipsec_sa.h"  /* IPSEC_SAREF_NULL, IPSEC_SA_REF_TABLE_IDX_WIDTH */
@@ -454,9 +452,6 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 	case SADB_X_EXT_ADDRESS_DST_FLOW:
 	case SADB_X_EXT_ADDRESS_SRC_MASK:
 	case SADB_X_EXT_ADDRESS_DST_MASK:
-#ifdef NAT_TRAVERSAL
-	case SADB_X_EXT_NAT_T_OA:
-#endif
 		break;
 	default:
 		DEBUGGING( 
@@ -1140,99 +1135,6 @@ errlab:
 	return error;
 }
 
-#ifdef NAT_TRAVERSAL
-int
-pfkey_x_nat_t_type_build(struct sadb_ext**	pfkey_ext,
-		    uint8_t         type)
-{
-	int error = 0;
-	int i;
-	struct sadb_x_nat_t_type *pfkey_x_nat_t_type = (struct sadb_x_nat_t_type *)*pfkey_ext;
-
-	DEBUGGING(
-		"pfkey_x_nat_t_type_build:\n");
-	/* sanity checks... */
-	if(pfkey_x_nat_t_type) {
-		DEBUGGING(
-			"pfkey_x_nat_t_type_build: "
-			"why is pfkey_x_nat_t_type already pointing to something?\n");
-		SENDERR(EINVAL);
-	}
-	
-	DEBUGGING(
-		"pfkey_x_nat_t_type_build: "
-		"type=%d\n", type);
-
-	if(!(*pfkey_ext = (struct sadb_ext*)pfkey_x_nat_t_type = (struct sadb_x_nat_t_type*)
-	     MALLOC(sizeof(struct sadb_x_nat_t_type)))) {
-		DEBUGGING(
-			"pfkey_x_nat_t_type_build: "
-			"memory allocation failed\n");
-		SENDERR(ENOMEM);
-	}
-	
-	pfkey_x_nat_t_type->sadb_x_nat_t_type_len = sizeof(struct sadb_x_nat_t_type) / IPSEC_PFKEYv2_ALIGN;
-	pfkey_x_nat_t_type->sadb_x_nat_t_type_exttype = SADB_X_EXT_NAT_T_TYPE;
-	pfkey_x_nat_t_type->sadb_x_nat_t_type_type = type;
-	for(i=0; i<3; i++) {
-		pfkey_x_nat_t_type->sadb_x_nat_t_type_reserved[i] = 0;
-	}
-
-errlab:
-	return error;
-}
-int
-pfkey_x_nat_t_port_build(struct sadb_ext**	pfkey_ext,
-		    uint16_t         exttype,
-		    uint16_t         port)
-{
-	int error = 0;
-	struct sadb_x_nat_t_port *pfkey_x_nat_t_port = (struct sadb_x_nat_t_port *)*pfkey_ext;
-
-	DEBUGGING(
-		"pfkey_x_nat_t_port_build:\n");
-	/* sanity checks... */
-	if(pfkey_x_nat_t_port) {
-		DEBUGGING(
-			"pfkey_x_nat_t_port_build: "
-			"why is pfkey_x_nat_t_port already pointing to something?\n");
-		SENDERR(EINVAL);
-	}
-	
-	switch(exttype) {	
-	case SADB_X_EXT_NAT_T_SPORT:
-	case SADB_X_EXT_NAT_T_DPORT:
-		break;
-	default:
-		DEBUGGING( 
-			"pfkey_nat_t_port_build: "
-			"unrecognised ext_type=%d.\n", 
-			exttype); 
-		SENDERR(EINVAL); 
-	}
-
-	DEBUGGING(
-		"pfkey_x_nat_t_port_build: "
-		"ext=%d, port=%d\n", exttype, port);
-
-	if(!(*pfkey_ext = (struct sadb_ext*)pfkey_x_nat_t_port = (struct sadb_x_nat_t_port*)
-	     MALLOC(sizeof(struct sadb_x_nat_t_port)))) {
-		DEBUGGING(
-			"pfkey_x_nat_t_port_build: "
-			"memory allocation failed\n");
-		SENDERR(ENOMEM);
-	}
-	
-	pfkey_x_nat_t_port->sadb_x_nat_t_port_len = sizeof(struct sadb_x_nat_t_port) / IPSEC_PFKEYv2_ALIGN;
-	pfkey_x_nat_t_port->sadb_x_nat_t_port_exttype = exttype;
-	pfkey_x_nat_t_port->sadb_x_nat_t_port_port = port;
-	pfkey_x_nat_t_port->sadb_x_nat_t_port_reserved = 0;
-
-errlab:
-	return error;
-}
-#endif
-
 int pfkey_x_protocol_build(struct sadb_ext **pfkey_ext,
 			   uint8_t protocol)
 {
@@ -1403,14 +1305,15 @@ errlab:
 
 /*
  * $Log: pfkey_v2_build.c,v $
- * Revision 1.3  2003/06/19 22:11:35  ken
- * Added NAT-T 0.6 (mlafon)
+ * Revision 1.2  2003/09/12 19:23:31  ken
+ * Import X.509 1.4.6
  *
- * Revision 1.2  2003/05/05 11:50:15  ken
- * Integrated X.509 1.3.1
+ * Revision 1.1.1.1  2003/09/12 19:02:33  ken
+ * Stock 2.02
  *
- * Revision 1.1.1.1  2003/05/05 11:46:13  ken
- * Initial Import of v2.00
+ * Revision 1.43  2003/05/07 17:29:17  mcr
+ * 	new function pfkey_debug_func added for us in debugging from
+ * 	pfkey library.
  *
  * Revision 1.42  2003/01/30 02:32:09  rgb
  *
