@@ -12,14 +12,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_parse.c,v 1.4 2003/06/19 22:11:35 ken Exp $
+ * RCSID $Id: pfkey_v2_parse.c,v 1.2 2003/09/12 19:23:31 ken Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_parser.c.
  */
 
-char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.4 2003/06/19 22:11:35 ken Exp $";
+char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.2 2003/09/12 19:23:31 ken Exp $";
 
 /*
  * Some ugly stuff to allow consistent debugging code for use in the
@@ -388,9 +388,6 @@ pfkey_address_parse(struct sadb_ext *pfkey_ext)
 	case SADB_X_EXT_ADDRESS_DST_FLOW:
 	case SADB_X_EXT_ADDRESS_SRC_MASK:
 	case SADB_X_EXT_ADDRESS_DST_MASK:
-#ifdef NAT_TRAVERSAL
-	case SADB_X_EXT_NAT_T_OA:
-#endif
 		break;
 	default:
 		DEBUGGING(PF_KEY_DEBUG_PARSE_PROBLEM, 
@@ -1142,7 +1139,7 @@ pfkey_x_ext_protocol_parse(struct sadb_ext *pfkey_ext)
 	if (p->sadb_protocol_len != sizeof(*p)/IPSEC_PFKEYv2_ALIGN) {
 		DEBUGGING(PF_KEY_DEBUG_PARSE_PROBLEM,
 			  "pfkey_x_protocol_parse: size wrong ext_len=%d, key_ext_len=%d.\n",
-			  p->sadb_protocol_len, (int)sizeof(*p));
+			  p->sadb_protocol_len, sizeof(*p));
 		SENDERR(EINVAL);
 	}
 	
@@ -1157,19 +1154,6 @@ pfkey_x_ext_protocol_parse(struct sadb_ext *pfkey_ext)
 	return error;
 }
 
-#ifdef NAT_TRAVERSAL
-DEBUG_NO_STATIC int
-pfkey_x_ext_nat_t_type_parse(struct sadb_ext *pfkey_ext)
-{
-	return 0;
-}
-DEBUG_NO_STATIC int
-pfkey_x_ext_nat_t_port_parse(struct sadb_ext *pfkey_ext)
-{
-	return 0;
-}
-#endif
-
 #define DEFINEPARSER(NAME) static struct pf_key_ext_parsers_def NAME##_def={NAME, #NAME};
 
 DEFINEPARSER(pfkey_sa_parse);
@@ -1185,10 +1169,6 @@ DEFINEPARSER(pfkey_x_kmprivate_parse);
 DEFINEPARSER(pfkey_x_satype_parse);
 DEFINEPARSER(pfkey_x_ext_debug_parse);
 DEFINEPARSER(pfkey_x_ext_protocol_parse);
-#ifdef NAT_TRAVERSAL
-DEFINEPARSER(pfkey_x_ext_nat_t_type_parse);
-DEFINEPARSER(pfkey_x_ext_nat_t_port_parse);
-#endif
 
 struct pf_key_ext_parsers_def *ext_default_parsers[]=
 {
@@ -1219,13 +1199,6 @@ struct pf_key_ext_parsers_def *ext_default_parsers[]=
 	&pfkey_address_parse_def,
 	&pfkey_x_ext_debug_parse_def,
 	&pfkey_x_ext_protocol_parse_def
-#ifdef NAT_TRAVERSAL
-	,
-	&pfkey_x_ext_nat_t_type_parse_def,
-	&pfkey_x_ext_nat_t_port_parse_def,
-	&pfkey_x_ext_nat_t_port_parse_def,
-	&pfkey_address_parse_def
-#endif
 };
 
 int
@@ -1609,17 +1582,11 @@ errlab:
 
 /*
  * $Log: pfkey_v2_parse.c,v $
- * Revision 1.4  2003/06/19 22:11:35  ken
- * Added NAT-T 0.6 (mlafon)
+ * Revision 1.2  2003/09/12 19:23:31  ken
+ * Import X.509 1.4.6
  *
- * Revision 1.3  2003/05/26 23:02:31  jjo
- * . 64bits cleanups
- *
- * Revision 1.2  2003/05/05 11:50:15  ken
- * Integrated X.509 1.3.1
- *
- * Revision 1.1.1.1  2003/05/05 11:46:13  ken
- * Initial Import of v2.00
+ * Revision 1.1.1.1  2003/09/12 19:02:34  ken
+ * Stock 2.02
  *
  * Revision 1.53  2003/01/30 02:32:09  rgb
  *
