@@ -18,7 +18,7 @@
  * Split out from ipsec_init.c version 1.70.
  */
 
-char ipsec_proc_c_version[] = "RCSID $Id: ipsec_proc.c,v 1.2 2003/06/05 18:59:26 ken Exp $";
+char ipsec_proc_c_version[] = "RCSID $Id: ipsec_proc.c,v 1.1.1.1 2003/09/12 19:02:53 ken Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -821,9 +821,7 @@ ipsec_proc_init()
 	proc_symlink("ipsec_spigrp", proc_net, "ipsec/spigrp/all");
 	proc_symlink("ipsec_tncfg",  proc_net, "ipsec/tncfg");
 	proc_symlink("ipsec_version",proc_net, "ipsec/version");
-#ifdef CONFIG_IPSEC_DEBUG
-        proc_symlink("ipsec_klipsdebug",proc_net,"ipsec/klipsdebug");
-#endif /* CONFIG_IPSEC_DEBUG */
+	proc_symlink("ipsec_klipsdebug",proc_net,"ipsec/klipsdebug");
 
 #endif /* !PROC_FS_2325 */
 
@@ -893,11 +891,8 @@ ipsec_proc_cleanup()
 
 /*
  * $Log: ipsec_proc.c,v $
- * Revision 1.2  2003/06/05 18:59:26  ken
- * Fix ipsec_klipsdebug symlink gets created but never deleted
- *
- * Revision 1.1.1.1  2003/05/05 11:46:14  ken
- * Initial Import of v2.00
+ * Revision 1.1.1.1  2003/09/12 19:02:53  ken
+ * Stock 2.02
  *
  * Revision 1.22  2002/09/20 15:40:57  rgb
  * Renamed saref macros for consistency and brevity.
