@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.3 2003/06/19 22:11:36 ken Exp $";
+char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.2 2003/09/15 12:57:30 jjo Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -82,10 +82,6 @@ char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.3 2003/06/19 22:11:36 k
 #include "freeswan/ipsec_proto.h"
 #include "freeswan/ipsec_alg.h"
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-#include <linux/udp.h>
-#endif
-
 #ifdef CONFIG_IPSEC_DEBUG
 int debug_ah = 0;
 int debug_esp = 0;
@@ -204,13 +200,6 @@ ipsec_updatereplaywindow(struct ipsec_sa*ipsp, __u32 seq)
 	return 1;			/* out of order but good */
 }
 
-struct auth_alg {
-	void (*init)(void *ctx);
-	void (*update)(void *ctx, unsigned char *bytes, __u32 len);
-	void (*final)(unsigned char *hash, void *ctx);
-	int hashlen;
-};
-
 #ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
 struct auth_alg ipsec_rcv_md5[]={
 	{MD5Init, MD5Update, MD5Final, AHMD596_ALEN}
@@ -270,21 +259,15 @@ struct ipsec_rcv_state {
 	int octx_len;
 	union {
 		struct {
-			struct esp *espp;
+			struct esphdr *espp;
 		} espstuff;
 		struct {
-			struct ah *ahp;
+			struct ahhdr *ahp;
 		} ahstuff;
 		struct {
 			struct ipcomphdr *compp;
 		} ipcompstuff;
 	} protostuff;
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	__u16 natt_len;
-	__u16 natt_sport;
-	__u16 natt_dport;
-	__u8 natt_type;
-#endif
 };
 
 struct xform_functions {
@@ -323,7 +306,7 @@ ipsec_rcv_esp_checks(struct ipsec_rcv_state *irs,
 		return IPSEC_RCV_BADLEN;
 	}
 
-	if(skb->len < (irs->hard_header_len + sizeof(struct iphdr) + sizeof(struct esp))) {
+	if(skb->len < (irs->hard_header_len + sizeof(struct iphdr) + sizeof(struct esphdr))) {
 		KLIPS_PRINT(debug_rcv & DB_RX_INAU,
 			    "klips_debug:ipsec_rcv: "
 			    "runt esp packet of skb->len=%d received from %s, dropped.\n",
@@ -335,7 +318,7 @@ ipsec_rcv_esp_checks(struct ipsec_rcv_state *irs,
 		return IPSEC_RCV_BADLEN;
 	}
 
-	irs->protostuff.espstuff.espp = (struct esp *)(skb->data + irs->iphlen);
+	irs->protostuff.espstuff.espp = (struct esphdr *)(skb->data + irs->iphlen);
 	irs->said.spi = irs->protostuff.espstuff.espp->esp_spi;
 
 	return IPSEC_RCV_OK;
@@ -347,7 +330,7 @@ ipsec_rcv_esp_decrypt_setup(struct ipsec_rcv_state *irs,
 			    __u32          *replay,
 			    unsigned char **authenticator)
 {
-	struct esp *espp = irs->protostuff.espstuff.espp;
+	struct esphdr *espp = irs->protostuff.espstuff.espp;
 
 	KLIPS_PRINT(debug_rcv,
 		    "klips_debug:ipsec_rcv: "
@@ -371,7 +354,7 @@ ipsec_rcv_esp_authcalc(struct ipsec_rcv_state *irs,
 		       struct sk_buff *skb)
 {
 	struct auth_alg *aa;
-	struct esp *espp = irs->protostuff.espstuff.espp;
+	struct esphdr *espp = irs->protostuff.espstuff.espp;
 	union {
 		MD5_CTX		md5;
 		SHA1_CTX	sha1;
@@ -414,7 +397,7 @@ enum ipsec_rcv_value
 ipsec_rcv_esp_decrypt(struct ipsec_rcv_state *irs)
 {
 	struct ipsec_sa *ipsp = irs->ipsp;
-	struct esp *espp = irs->protostuff.espstuff.espp;
+	struct esphdr *espp = irs->protostuff.espstuff.espp;
 	int esphlen = 0;
 	__u8 *idat;	/* pointer to content to be decrypted/authenticated */
 #ifdef CONFIG_IPSEC_ENC_3DES
@@ -427,7 +410,6 @@ ipsec_rcv_esp_decrypt(struct ipsec_rcv_state *irs)
 #ifdef CONFIG_IPSEC_ALG
 	struct ipsec_alg_enc *ixt_e=NULL;
 #endif /* CONFIG_IPSEC_ALG */
-  
 
 	skb=irs->skb;
 
@@ -447,7 +429,7 @@ ipsec_rcv_esp_decrypt(struct ipsec_rcv_state *irs)
 	case ESP_3DES:
 		iv[0] = *((__u32 *)(espp->esp_iv)    );
 		iv[1] = *((__u32 *)(espp->esp_iv) + 1);
-		esphlen = sizeof(struct esp);
+		esphlen = sizeof(struct esphdr);
 		break;
 #endif /* !CONFIG_IPSEC_ENC_3DES */
 	default:
@@ -622,9 +604,9 @@ ipsec_rcv_ah_checks(struct ipsec_rcv_state *irs,
 	ahminlen = irs->hard_header_len + sizeof(struct iphdr);
 
 	/* take care not to deref this pointer until we check the minlen though */
-	irs->protostuff.ahstuff.ahp = (struct ah *) (skb->data + irs->iphlen);
+	irs->protostuff.ahstuff.ahp = (struct ahhdr *) (skb->data + irs->iphlen);
 
-	if((skb->len < ahminlen+sizeof(struct ah)) ||
+	if((skb->len < ahminlen+sizeof(struct ahhdr)) ||
 	   (skb->len < ahminlen+(irs->protostuff.ahstuff.ahp->ah_hl << 2))) {
 		KLIPS_PRINT(debug_rcv & DB_RX_INAU,
 			    "klips_debug:ipsec_rcv: "
@@ -645,7 +627,7 @@ ipsec_rcv_ah_checks(struct ipsec_rcv_state *irs,
 			    "klips_debug:ipsec_rcv: "
 			    "bad authenticator length %ld, expected %lu from %s.\n",
 			    (long)(irs->protostuff.ahstuff.ahp->ah_hl << 2),
-			    (unsigned long) sizeof(struct ah),
+			    (unsigned long) sizeof(struct ahhdr),
 			    irs->ipsaddr_txt);
 		if(irs->stats) {
 			irs->stats->rx_errors++;
@@ -663,7 +645,7 @@ ipsec_rcv_ah_setup_auth(struct ipsec_rcv_state *irs,
 			__u32          *replay,
 			unsigned char **authenticator)
 {
-	struct ah *ahp = irs->protostuff.ahstuff.ahp;
+	struct ahhdr *ahp = irs->protostuff.ahstuff.ahp;
 
 	*replay = ntohl(ahp->ah_rpl);
 	*authenticator = ahp->ah_data;
@@ -676,7 +658,7 @@ ipsec_rcv_ah_authcalc(struct ipsec_rcv_state *irs,
 		      struct sk_buff *skb)
 {
 	struct auth_alg *aa;
-	struct ah *ahp = irs->protostuff.ahstuff.ahp;
+	struct ahhdr *ahp = irs->protostuff.ahstuff.ahp;
 	union {
 		MD5_CTX		md5;
 		SHA1_CTX	sha1;
@@ -726,7 +708,7 @@ ipsec_rcv_ah_authcalc(struct ipsec_rcv_state *irs,
 enum ipsec_rcv_value
 ipsec_rcv_ah_decap(struct ipsec_rcv_state *irs)
 {
-	struct ah *ahp = irs->protostuff.ahstuff.ahp;
+	struct ahhdr *ahp = irs->protostuff.ahstuff.ahp;
 	struct sk_buff *skb;
 	int ahhlen;
 
@@ -921,13 +903,6 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 	struct in_addr ipdaddr;
 	int replay = 0;	/* replay value in AH or ESP packet */
 	struct ipsec_sa* ipsnext = NULL;	/* next SA towards inside of packet */
-#ifdef INBOUND_POLICY_CHECK_eroute
-	struct sockaddr_encap matcher;	/* eroute search key */
-	struct eroute *er;
-	struct sa_id policy_said;
-	struct sockaddr_encap policy_eaddr;
-	struct sockaddr_encap policy_emask;
-#endif /* INBOUND_POLICY_CHECK_eroute */
 	struct xform_functions *proto_funcs;
 	struct ipsec_sa *newipsp;
 	struct iphdr *ipp;
@@ -1042,40 +1017,6 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		return IPSEC_RCV_SAIDNOTLIVE;
 	}
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	if ((irs->natt_type) &&
-		( (irs->ipp->saddr != (((struct sockaddr_in*)(newipsp->ips_addr_s))->sin_addr.s_addr)) ||
-		  (irs->natt_sport != newipsp->ips_natt_sport)
-		)) {
-		struct sockaddr sipaddr;
-		/** Advertise NAT-T addr change to pluto **/
-		sipaddr.sa_family = AF_INET;
-		((struct sockaddr_in*)&sipaddr)->sin_addr.s_addr = irs->ipp->saddr;
-		((struct sockaddr_in*)&sipaddr)->sin_port = htons(irs->natt_sport);
-		pfkey_nat_t_new_mapping(newipsp, &sipaddr, irs->natt_sport);
-		/**
-		 * Then allow or block packet depending on
-		 * sysctl_ipsec_inbound_policy_check.
-		 *
-		 * In all cases, pluto will update SA if new mapping is
-		 * accepted.
-		 */
-		if (sysctl_ipsec_inbound_policy_check) {
-			KLIPS_PRINT(debug_rcv,
-				"klips_debug:ipsec_rcv: "
-				"SA:%s, src=%s:%u of pkt does not agree with expected "
-				"SA source address policy (pluto has been informed).\n",
-				irs->sa_len ? irs->sa : " (error)",
-				irs->ipsaddr_txt, irs->natt_sport);
-			if(irs->stats) {
-				irs->stats->rx_dropped++;
-			}
-			ipsec_sa_put(newipsp);
-			return IPSEC_RCV_FAILEDINBOUND;
-		}
-	}
-#endif
-
 	if(sysctl_ipsec_inbound_policy_check) {
 		if(irs->ipp->saddr != ((struct sockaddr_in*)(newipsp->ips_addr_s))->sin_addr.s_addr) {
 			KLIPS_PRINT(debug_rcv,
@@ -1128,24 +1069,6 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		 * we just found matched the back pointer. But, we won't do this check anymore,
 		 * because we want to be able to nest SAs
 		 */
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-		KLIPS_PRINT(debug_rcv,
-			"klips_debug:ipsec_rcv: "
-			"natt_type=%u tdbp->ips_natt_type=%u : %s\n",
-			irs->natt_type, newipsp->ips_natt_type,
-			(irs->natt_type==newipsp->ips_natt_type)?"ok":"bad");
-		if (irs->natt_type != newipsp->ips_natt_type) {
-			KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: "
-				    "SA:%s does not agree with expected NAT-T policy.\n",
-				    irs->sa_len ? irs->sa : " (error)");
-			if(irs->stats) {
-				irs->stats->rx_dropped++;
-			}
-			ipsec_sa_put(newipsp);
-			return IPSEC_RCV_FAILEDINBOUND;
-		}
-#endif
 	}
 
 	/* okay, SA checks out, so free any previous SA, and record a new one */
@@ -1500,13 +1423,6 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	struct in_addr ipdaddr;
 
 	struct ipsec_sa* ipsnext = NULL;	/* next SA towards inside of packet */
-#ifdef INBOUND_POLICY_CHECK_eroute
-	struct sockaddr_encap matcher;	/* eroute search key */
-	struct eroute *er;
-	struct sa_id policy_said;
-	struct sockaddr_encap policy_eaddr;
-	struct sockaddr_encap policy_emask;
-#endif /* INBOUND_POLICY_CHECK_eroute */
 	struct ipsec_rcv_state irs;
 
 	/* Don't unlink in the middle of a turnaround */
@@ -1528,76 +1444,6 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		goto rcvleave;
 	}
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	if (skb->sk && skb->nh.iph && skb->nh.iph->protocol==IPPROTO_UDP) {
-		/**
-		 * Packet comes from udp_queue_rcv_skb so it is already defrag,
-		 * checksum verified, ... (ie safe to use)
-		 *
-		 * If the packet is not for us, return -1 and udp_queue_rcv_skb
-		 * will continue to handle it (do not kfree skb !!).
-		 */
-		struct udp_opt *tp =  &(skb->sk->tp_pinfo.af_udp);
-		struct iphdr *ip = (struct iphdr *)skb->nh.iph;
-		struct udphdr *udp = (struct udphdr *)((__u32 *)ip+ip->ihl);
-		__u8 *udpdata = (__u8 *)udp + sizeof(struct udphdr);
-		__u32 *udpdata32 = (__u32 *)udpdata;
-
-		irs.natt_sport = ntohs(udp->source);
-		irs.natt_dport = ntohs(udp->dest);
-
-		KLIPS_PRINT(debug_rcv,
-		    "klips_debug:ipsec_rcv: "
-		    "suspected ESPinUDP packet (NAT-Traversal) [%d].\n",
-			tp->esp_in_udp);
-		KLIPS_IP_PRINT(debug_rcv, ip);
-
-		if (udpdata < skb->tail) {
-			unsigned int len = skb->tail - udpdata;
-			if ((len==1) && (udpdata[0]==0xff)) {
-				KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: "
-					/* not IPv6 compliant message */
-				    "NAT-keepalive from %d.%d.%d.%d.\n", NIPQUAD(ip->saddr));
-				goto rcvleave;
-			}
-			else if ( (tp->esp_in_udp == ESPINUDP_WITH_NON_IKE) &&
-				(len > (2*sizeof(__u32) + sizeof(struct esp))) &&
-				(udpdata32[0]==0) && (udpdata32[1]==0) ) {
-				/* ESP Packet with Non-IKE header */
-				KLIPS_PRINT(debug_rcv, 
-					"klips_debug:ipsec_rcv: "
-					"ESPinUDP pkt with Non-IKE - spi=0x%x\n",
-					udpdata32[2]);
-				irs.natt_type = ESPINUDP_WITH_NON_IKE;
-				irs.natt_len = sizeof(struct udphdr)+(2*sizeof(__u32));
-			}
-			else if ( (tp->esp_in_udp == ESPINUDP_WITH_NON_ESP) &&
-				(len > sizeof(struct esp)) &&
-				(udpdata32[0]!=0) ) {
-				/* ESP Packet without Non-ESP header */
-				irs.natt_type = ESPINUDP_WITH_NON_ESP;
-				irs.natt_len = sizeof(struct udphdr);
-				KLIPS_PRINT(debug_rcv, 
-					"klips_debug:ipsec_rcv: "
-					"ESPinUDP pkt without Non-ESP - spi=0x%x\n",
-					udpdata32[0]);
-			}
-			else {
-				KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: "
-					"IKE packet - not handled here\n");
-				MOD_DEC_USE_COUNT;
-				return -1;
-			}
-		}
-		else {
-			MOD_DEC_USE_COUNT;
-			return -1;
-		}
-	}
-#endif
-
 #ifdef IPH_is_SKB_PULLED
 	/* In Linux 2.4.4, the IP header has been skb_pull()ed before the
 	   packet is passed to us. So we'll skb_push() to get back to it. */
@@ -1658,50 +1504,6 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	}
 #endif /* IP_FRAGMENT_LINEARIZE */
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	if (irs.natt_len) {
-		/**
-		 * Now, we are sure packet is ESPinUDP. Remove natt_len bytes from
-		 * packet and modify protocol to ESP.
-		 */
-		if (((unsigned char *)skb->data > (unsigned char *)skb->nh.iph) &&
-			((unsigned char *)skb->nh.iph > (unsigned char *)skb->head)) {
-			unsigned int _len = (unsigned char *)skb->data -
-				(unsigned char *)skb->nh.iph;
-			KLIPS_PRINT(debug_rcv,
-				"klips_debug:ipsec_rcv: adjusting skb: skb_push(%u)\n",
-				_len);
-			skb_push(skb, _len);
-		}
-		KLIPS_PRINT(debug_rcv,
-		    "klips_debug:ipsec_rcv: "
-			"removing %d bytes from ESPinUDP packet\n", irs.natt_len);
-		ipp = (struct iphdr *)skb->data;
-		irs.iphlen = ipp->ihl << 2;
-		ipp->tot_len = htons(ntohs(ipp->tot_len) - irs.natt_len);
-		if (skb->len < irs.iphlen + irs.natt_len) {
-			printk(KERN_WARNING
-		       "klips_error:ipsec_rcv: "
-		       "ESPinUDP packet is too small (%d < %d+%d). "
-			   "This should never happen, please report.\n",
-		       (int)(skb->len), irs.iphlen, irs.natt_len);
-			goto rcvleave;
-		}
-		memmove(skb->data + irs.natt_len, skb->data, irs.iphlen);
-		skb_pull(skb, irs.natt_len);
-
-		/* update nh.iph */
-		ipp = skb->nh.iph = (struct iphdr *)skb->data;
-
-		/* modify protocol */
-		ipp->protocol = IPPROTO_ESP;
-
-		skb->sk = NULL;
-
-		KLIPS_IP_PRINT(debug_rcv, skb->nh.iph);
-	}
-#endif
-
 	ipp = skb->nh.iph;
 	ipsaddr.s_addr = ipp->saddr;
 	addrtoa(ipsaddr, 0, irs.ipsaddr_txt, sizeof(irs.ipsaddr_txt));
@@ -1867,91 +1669,6 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	}
 #endif /* CONFIG_IPSEC_IPCOMP */
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	if ((irs.natt_type) && (ipp->protocol != IPPROTO_IPIP)) {
-		/**
-		 * NAT-Traversal and Transport Mode:
-		 *   we need to correct TCP/UDP checksum
-		 *
-		 * If we've got NAT-OA, we can fix checksum without recalculation.
-		 */
-		__u32 natt_oa = ipsp->ips_natt_oa ?
-			((struct sockaddr_in*)(ipsp->ips_natt_oa))->sin_addr.s_addr : 0;
-		__u16 pkt_len = skb->tail - (unsigned char *)ipp;
-		__u16 data_len = pkt_len - (ipp->ihl << 2);
-
-		switch (ipp->protocol) {
-			case IPPROTO_TCP:
-				if (data_len >= sizeof(struct tcphdr)) {
-					struct tcphdr *tcp = (struct tcphdr *)((__u32 *)ipp+ipp->ihl);
-					if (natt_oa) {
-						__u32 buff[2] = { ~natt_oa, ipp->saddr };
-						KLIPS_PRINT(debug_rcv,
-				    		"klips_debug:ipsec_rcv: "
-							"NAT-T & TRANSPORT: "
-							"fix TCP checksum using NAT-OA\n");
-						tcp->check = csum_fold(
-							csum_partial((unsigned char *)buff, sizeof(buff),
-							tcp->check^0xffff));
-					}
-					else {
-						KLIPS_PRINT(debug_rcv,
-			    			"klips_debug:ipsec_rcv: "
-							"NAT-T & TRANSPORT: recalc TCP checksum\n");
-						if (pkt_len > (ntohs(ipp->tot_len)))
-							data_len -= (pkt_len - ntohs(ipp->tot_len));
-						tcp->check = 0;
-						tcp->check = csum_tcpudp_magic(ipp->saddr, ipp->daddr,
-							data_len, IPPROTO_TCP,
-							csum_partial((unsigned char *)tcp, data_len, 0));
-					}
-				}
-				else {
-					KLIPS_PRINT(debug_rcv,
-			    		"klips_debug:ipsec_rcv: "
-						"NAT-T & TRANSPORT: can't fix TCP checksum\n");
-				}
-				break;
-			case IPPROTO_UDP:
-				if (data_len >= sizeof(struct udphdr)) {
-					struct udphdr *udp = (struct udphdr *)((__u32 *)ipp+ipp->ihl);
-					if (udp->check == 0) {
-						KLIPS_PRINT(debug_rcv,
-				    		"klips_debug:ipsec_rcv: "
-							"NAT-T & TRANSPORT: UDP checksum already 0\n");
-					}
-					else if (natt_oa) {
-						__u32 buff[2] = { ~natt_oa, ipp->saddr };
-						KLIPS_PRINT(debug_rcv,
-				    		"klips_debug:ipsec_rcv: "
-							"NAT-T & TRANSPORT: "
-							"fix UDP checksum using NAT-OA\n");
-						udp->check = csum_fold(
-							csum_partial((unsigned char *)buff, sizeof(buff),
-							udp->check^0xffff));
-					}
-					else {
-						KLIPS_PRINT(debug_rcv,
-				    		"klips_debug:ipsec_rcv: "
-							"NAT-T & TRANSPORT: zero UDP checksum\n");
-						udp->check = 0;
-					}
-				}
-				else {
-					KLIPS_PRINT(debug_rcv,
-			    		"klips_debug:ipsec_rcv: "
-						"NAT-T & TRANSPORT: can't fix UDP checksum\n");
-				}
-				break;
-			default:
-				KLIPS_PRINT(debug_rcv,
-			    	"klips_debug:ipsec_rcv: "
-					"NAT-T & TRANSPORT: non TCP/UDP packet -- do nothing\n");
-				break;
-		}
-	}
-#endif
-
 	/*
 	 * XXX this needs to be locked from when it was first looked
 	 * up in the decapsulation loop.  Perhaps it is better to put
@@ -2093,97 +1810,6 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 #endif /* CONFIG_NETFILTER */
 	}
 
-#ifdef INBOUND_POLICY_CHECK_eroute
-	/*
-	  Do *not* enable this without thoroughly checking spinlock issues
-	  first.  In particular, nesting an eroute spinlock within a tdb
-	  spinlock could result in a deadlock.  (Well, only on a SMP machine
-	  under 2.4?)
-	*/
-
-	/*
-	 * First things first -- look us up in the erouting tables.
-	 */
-	matcher.sen_len = sizeof (struct sockaddr_encap);
-	matcher.sen_family = AF_ENCAP;
-	matcher.sen_type = SENT_IP4;
-	if(ipp->protocol == IPPROTO_IPIP) {
-		struct iphdr *ipp2;
-
-		ipp2 = (struct iphdr*) (((char*)ipp) + (ipp->ihl << 2));
-		matcher.sen_ip_src.s_addr = ipp2->saddr;
-		matcher.sen_ip_dst.s_addr = ipp2->daddr;
-	} else {
-		matcher.sen_ip_src.s_addr = ipp->saddr;
-		matcher.sen_ip_dst.s_addr = ipp->daddr;
-	}
-
-	/*
-	 * The spinlock is to prevent any other process from accessing or
-	 * deleting the eroute while we are using and updating it.
-	 */
-	spin_lock(&eroute_lock);
-
-	er = ipsec_findroute(&matcher);
-	if(er) {
-		policy_said = er->er_said;
-		policy_eaddr = er->er_eaddr;
-		policy_emask = er->er_emask;
-		er->er_count++;
-		er->er_lasttime = jiffies/HZ;
-	}
-
-	spin_unlock(&eroute_lock);
-
-	if(er) {
-		struct ipsec_sa* policy_ips = NULL, *ipsq = NULL;
-		/*
-		 * The spinlock is to prevent any other process from
-		 * accessing or deleting the ipsec_sa while we are using and
-		 * updating it.
-		 */
-		spin_lock(&tdb_lock);
-
-		ipsq = ipsec_sa_getbyid(&policy_said);
-		policy_ips = ipsq;
-		if (policy_ips == NULL) {
-			ipsec_sa_put(ipsq);
-			spin_unlock(&tdb_lock);
-			KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: "
-				    "no ipsec_sa for SA%s: incoming packet with no policy SA, dropped.\n",
-				    sa_len ? sa : " (error)");
-			goto rcvleave;
-		}
-
-		sa_len = satoa(policy_said, 0, sa, SATOA_BUF);
-
-		KLIPS_PRINT(debug_rcv,
-			    "klips_debug:ipsec_rcv: "
-			    "found policy ipsec_sa -- SA:%s\n",
-			    sa_len ? sa : " (error)");
-		while(1) {
-			if(policy_ips->ips_inext) {
-				policy_ips = policy_ips->ips_inext;
-			} else {
-				break;
-			}
-		}
-		if(policy_ips != ipsp) {
-			ipsec_sa_put(ipsq);
-			spin_unlock(&tdb_lock);
-			KLIPS_PRINT(debug_rcv,
-				    "klips_debug:ipsec_rcv: "
-				    "ipsec_sa for SA%s: incoming packet with different policy SA, dropped.\n",
-				    sa_len ? sa : " (error)");
-			goto rcvleave;
-		}
-
-		ipsec_sa_put(ipsq);
-		/* spin_unlock(&tdb_lock); */
-	}
-#endif /* INBOUND_POLICY_CHECK_eroute */
-
 	spin_unlock(&tdb_lock);
 
 #ifdef NET_21
@@ -2305,14 +1931,17 @@ struct inet_protocol comp_protocol =
 
 /*
  * $Log: ipsec_rcv.c,v $
- * Revision 1.3  2003/06/19 22:11:36  ken
- * Added NAT-T 0.6 (mlafon)
+ * Revision 1.2  2003/09/15 12:57:30  jjo
+ * . alg-0.8.1_rc10 merged into sfs-2.02+x509-1.4.6
  *
- * Revision 1.2  2003/05/09 17:41:47  jjo
- * . Merged alg-0.8.1rc7
+ * Revision 1.1.1.1  2003/09/12 19:02:56  ken
+ * Stock 2.02
  *
- * Revision 1.1.1.1  2003/05/05 11:46:14  ken
- * Initial Import of v2.00
+ * Revision 1.129  2003/02/06 02:21:34  rgb
+ *
+ * Moved "struct auth_alg" from ipsec_rcv.c to ipsec_ah.h .
+ * Changed "struct ah" to "struct ahhdr" and "struct esp" to "struct esphdr".
+ * Removed "#ifdef INBOUND_POLICY_CHECK_eroute" dead code.
  *
  * Revision 1.128  2002/12/13 20:58:03  rgb
  * Relegated MCR's recent "_dmp" routine to debug_verbose.
