@@ -14,7 +14,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_sa.c,v 1.3 2003/06/19 22:11:36 ken Exp $
+ * RCSID $Id: ipsec_sa.c,v 1.2 2003/09/15 12:57:30 jjo Exp $
  *
  * This is the file formerly known as "ipsec_xform.h"
  *
@@ -282,7 +282,7 @@ ipsec_sadb_init(void)
 	int error = 0;
 	unsigned i;
 
-	for(i = 1; i < SADB_HASHMOD; i++) {
+	for(i = 0; i < SADB_HASHMOD; i++) {
 		ipsec_sadb_hash[i] = NULL;
 	}
 	/* parts above are for the old style SADB hash table */
@@ -967,14 +967,6 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 	}
 	ips->ips_addr_p = NULL;
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-	if(ips->ips_natt_oa) {
-		memset((caddr_t)(ips->ips_natt_oa), 0, ips->ips_natt_oa_size);
-		kfree(ips->ips_natt_oa);
-	}
-	ips->ips_natt_oa = NULL;
-#endif
-
 	if(ips->ips_key_a != NULL) {
 		memset((caddr_t)(ips->ips_key_a), 0, ips->ips_key_a_size);
 		kfree(ips->ips_key_a);
@@ -1033,14 +1025,14 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 
 /*
  * $Log: ipsec_sa.c,v $
- * Revision 1.3  2003/06/19 22:11:36  ken
- * Added NAT-T 0.6 (mlafon)
+ * Revision 1.2  2003/09/15 12:57:30  jjo
+ * . alg-0.8.1_rc10 merged into sfs-2.02+x509-1.4.6
  *
- * Revision 1.2  2003/05/09 17:41:48  jjo
- * . Merged alg-0.8.1rc7
+ * Revision 1.1.1.1  2003/09/12 19:02:56  ken
+ * Stock 2.02
  *
- * Revision 1.1.1.1  2003/05/05 11:46:14  ken
- * Initial Import of v2.00
+ * Revision 1.20  2003/02/06 01:50:34  rgb
+ * Fixed initialisation bug for first sadb hash bucket that would only manifest itself on platforms where NULL != 0.
  *
  * Revision 1.19  2003/01/30 02:32:22  rgb
  *
