@@ -12,14 +12,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_ext_process.c,v 1.4 2003/06/19 22:11:37 ken Exp $
+ * RCSID $Id: pfkey_v2_ext_process.c,v 1.3 2003/09/15 12:57:31 jjo Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_netlink.c.
  */
 
-char pfkey_v2_ext_process_c_version[] = "$Id: pfkey_v2_ext_process.c,v 1.4 2003/06/19 22:11:37 ken Exp $";
+char pfkey_v2_ext_process_c_version[] = "$Id: pfkey_v2_ext_process.c,v 1.3 2003/09/15 12:57:31 jjo Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -77,7 +77,7 @@ char pfkey_v2_ext_process_c_version[] = "$Id: pfkey_v2_ext_process.c,v 1.4 2003/
 #include "freeswan/ipsec_xform.h"
 #include "freeswan/ipsec_ah.h"
 #include "freeswan/ipsec_esp.h"
-#include "freeswan/ipsec_tunnel.h"	/* ..., ipsec_tunnel_start_xmit() */
+#include "freeswan/ipsec_tunnel.h"
 #include "freeswan/ipsec_rcv.h"
 #include "freeswan/ipcomp.h"
 
@@ -358,15 +358,6 @@ pfkey_address_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* e
 		sap = (unsigned char **)&(extr->eroute->er_emask.sen_ip_dst);
 		portp = &(extr->eroute->er_emask.sen_dport);
 		break;
-#ifdef NAT_TRAVERSAL
-	case SADB_X_EXT_NAT_T_OA:
-		KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_address_process: "
-			    "found NAT-OA address.\n");
-		sap = (unsigned char **)&(extr->ips->ips_natt_oa);
-		extr->ips->ips_natt_oa_size = saddr_len;
-		break;
-#endif
 	default:
 		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_address_process: "
@@ -380,9 +371,6 @@ pfkey_address_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* e
 	case SADB_EXT_ADDRESS_DST:
 	case SADB_EXT_ADDRESS_PROXY:
 	case SADB_X_EXT_ADDRESS_DST2:
-#ifdef NAT_TRAVERSAL
-	case SADB_X_EXT_NAT_T_OA:
-#endif
 		KLIPS_PRINT(debug_pfkey,
 			    "klips_debug:pfkey_address_process: "
 			    "allocating %d bytes for saddr.\n",
@@ -697,93 +685,6 @@ errlab:
 	return error;
 }
 
-#ifdef CONFIG_IPSEC_NAT_TRAVERSAL
-int
-pfkey_x_nat_t_type_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* extr)
-{
-	int error = 0;
-	struct sadb_x_nat_t_type *pfkey_x_nat_t_type = (struct sadb_x_nat_t_type *)pfkey_ext;
-
-	if(!pfkey_x_nat_t_type) {
-		printk("klips_debug:pfkey_x_nat_t_type_process: "
-		       "null pointer passed in\n");
-		SENDERR(EINVAL);
-	}
-
-	KLIPS_PRINT(debug_pfkey,
-		    "klips_debug:pfkey_x_nat_t_type_process: %d.\n",
-			pfkey_x_nat_t_type->sadb_x_nat_t_type_type);
-
-	if(!extr || !extr->ips) {
-		KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_nat_t_type_process: "
-			    "extr or extr->ips is NULL, fatal\n");
-		SENDERR(EINVAL);
-	}
-
-	switch(pfkey_x_nat_t_type->sadb_x_nat_t_type_type) {
-		case ESPINUDP_WITH_NON_IKE: /* with Non-IKE */
-		case ESPINUDP_WITH_NON_ESP: /* with Non-ESP */
-			extr->ips->ips_natt_type = pfkey_x_nat_t_type->sadb_x_nat_t_type_type;
-			break;
-		default:
-			KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_x_nat_t_type_process: "
-			    "unknown type %d.\n",
-			    pfkey_x_nat_t_type->sadb_x_nat_t_type_type);
-			SENDERR(EINVAL);
-			break;
-	}
-
-errlab:
-	return error;
-}
-
-int
-pfkey_x_nat_t_port_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* extr)
-{
-	int error = 0;
-	struct sadb_x_nat_t_port *pfkey_x_nat_t_port = (struct sadb_x_nat_t_port *)pfkey_ext;
-
-	if(!pfkey_x_nat_t_port) {
-		printk("klips_debug:pfkey_x_nat_t_port_process: "
-		       "null pointer passed in\n");
-		SENDERR(EINVAL);
-	}
-
-	KLIPS_PRINT(debug_pfkey,
-		    "klips_debug:pfkey_x_nat_t_port_process: %d/%d.\n",
-			pfkey_x_nat_t_port->sadb_x_nat_t_port_exttype,
-			pfkey_x_nat_t_port->sadb_x_nat_t_port_port);
-
-	if(!extr || !extr->ips) {
-		KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_nat_t_type_process: "
-			    "extr or extr->ips is NULL, fatal\n");
-		SENDERR(EINVAL);
-	}
-
-	switch(pfkey_x_nat_t_port->sadb_x_nat_t_port_exttype) {
-		case SADB_X_EXT_NAT_T_SPORT:
-			extr->ips->ips_natt_sport = pfkey_x_nat_t_port->sadb_x_nat_t_port_port;
-			break;
-		case SADB_X_EXT_NAT_T_DPORT:
-			extr->ips->ips_natt_dport = pfkey_x_nat_t_port->sadb_x_nat_t_port_port;
-			break;
-		default:
-			KLIPS_PRINT(debug_pfkey,
-			    "klips_debug:pfkey_x_nat_t_port_process: "
-			    "unknown exttype %d.\n",
-			    pfkey_x_nat_t_port->sadb_x_nat_t_port_exttype);
-			SENDERR(EINVAL);
-			break;
-	}
-
-errlab:
-	return error;
-}
-#endif
-
 int
 pfkey_x_debug_process(struct sadb_ext *pfkey_ext, struct pfkey_extracted_data* extr)
 {
@@ -852,17 +753,17 @@ errlab:
 
 /*
  * $Log: pfkey_v2_ext_process.c,v $
- * Revision 1.4  2003/06/19 22:11:37  ken
- * Added NAT-T 0.6 (mlafon)
+ * Revision 1.3  2003/09/15 12:57:31  jjo
+ * . alg-0.8.1_rc10 merged into sfs-2.02+x509-1.4.6
  *
- * Revision 1.3  2003/05/09 17:41:49  jjo
- * . Merged alg-0.8.1rc7
+ * Revision 1.2  2003/09/12 19:23:31  ken
+ * Import X.509 1.4.6
  *
- * Revision 1.2  2003/05/05 11:50:16  ken
- * Integrated X.509 1.3.1
+ * Revision 1.1.1.1  2003/09/12 19:03:01  ken
+ * Stock 2.02
  *
- * Revision 1.1.1.1  2003/05/05 11:46:14  ken
- * Initial Import of v2.00
+ * Revision 1.10  2003/02/06 01:51:41  rgb
+ * Removed no longer relevant comment
  *
  * Revision 1.9  2003/01/30 02:32:44  rgb
  *
