@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: constants.c,v 1.9 2003/07/21 02:17:05 ken Exp $
+ * RCSID $Id: constants.c,v 1.3 2003/09/15 12:57:38 jjo Exp $
  */
 
 /*
@@ -25,6 +25,7 @@
 #include <netinet/in.h>
 
 #include <freeswan.h>
+#include <freeswan/ipsec_policy.h>
 
 #include "constants.h"
 #include "defs.h"
@@ -37,14 +38,21 @@ const char compile_time_interop_options[] = ""
 #ifdef X509
 	" " X509
 #endif
-#ifdef LDAP_URL
-	" LDAP"
+#ifdef LDAP_VER
+#if LDAP_VER == 2
+	" LDAP_V2"
+#else
+	" LDAP_V3"
+#endif
 #endif
 #ifdef SMARTCARD
 	" SMARTCARD"
 #endif
 #ifdef PLUTO_SENDS_VENDORID
 	" PLUTO_SENDS_VENDORID"
+#endif
+#ifdef USE_KEYRR
+	" PLUTO_USES_KEYRR"
 #endif
     ;
 
@@ -75,21 +83,12 @@ static const char *const timer_event_name[] = {
 	"EVENT_SO_DISCARD",
 	"EVENT_RETRANSMIT",
 	"EVENT_SA_REPLACE",
+	"EVENT_SA_REPLACE_IF_USED",
 	"EVENT_SA_EXPIRE",
-        "EVENT_DPD",
-        "EVENT_DPD_TIMEOUT",
-#ifdef NAT_TRAVERSAL
-	"EVENT_NAT_T_KEEPALIVE",
-#endif
     };
 
 enum_names timer_event_names =
-#ifdef NAT_TRAVERSAL
-    { EVENT_NULL, EVENT_NAT_T_KEEPALIVE, timer_event_name, NULL };
-#else
-    { EVENT_NULL, EVENT_DPD_TIMEOUT, timer_event_name, NULL };
-#endif
-
+    { EVENT_NULL, EVENT_SA_EXPIRE, timer_event_name, NULL };
 
 /* Domain of Interpretation */
 
@@ -100,7 +99,7 @@ static const char *const doi_name[] = {
 
 enum_names doi_names = { ISAKMP_DOI_ISAKMP, ISAKMP_DOI_IPSEC, doi_name, NULL };
 
-/* debugging settings: a set selections for reporting
+/* debugging settings: a set of selections for reporting
  * These would be more naturally situated in log.h,
  * but they are shared with whack.
  * It turns out that "debug-" is clutter in all contexts this is used,
@@ -117,6 +116,8 @@ const char *const debug_bit_names[] = {
 	"klips",
 	"dns",
 	"oppo",
+	"controlmore",
+
 	"private",
 
 	"impair-delay-adns-key-answer",
@@ -178,7 +179,7 @@ const char *const state_story[] = {
 /* kind of struct connection */
 
 static const char *const connection_kind_name[] = {
-    "CK_GROUP",		/* food group: instantiates to template */
+    "CK_GROUP",		/* policy group: instantiates to template */
     "CK_TEMPLATE",	/* abstract connection, with wildcard */
     "CK_PERMANENT",	/* normal connection */
     "CK_INSTANCE",	/* instance of template, created for a particular attempt */
@@ -198,6 +199,7 @@ static const char *const routing_story_strings[] = {
     "erouted HOLD",	/* RT_ROUTED_HOLD: routed, and HOLD shunt installed */
     "fail erouted",	/* RT_ROUTED_FAILURE: routed, and failure-context shunt eroute installed */
     "erouted",	/* RT_ROUTED_TUNNEL: routed, and erouted to an IPSEC SA group */
+    "keyed, unrouted",  /* RT_UNROUTED_KEYED: was routed+keyed, but it got turned into an outer policy */
     };
 
 enum_names routing_story =
@@ -220,25 +222,11 @@ const char *const payload_name[] = {
 	"ISAKMP_NEXT_N",
 	"ISAKMP_NEXT_D",
 	"ISAKMP_NEXT_VID",
-#ifdef NAT_TRAVERSAL
-	"ISAKMP_NEXT_14",
-	"ISAKMP_NEXT_NAT-D",
-	"ISAKMP_NEXT_NAT-OA",
-#endif
 	NULL
     };
 
-#ifdef NAT_TRAVERSAL
-const char *const payload_name_nat_d[] = { "ISAKMP_NEXT_NAT-D",
-	"ISAKMP_NEXT_NAT-OA", NULL };
-static enum_names payload_names_nat_d =
-	{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
-enum_names payload_names =
-    { ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_nat_d };
-#else
 enum_names payload_names =
     { ISAKMP_NEXT_NONE, ISAKMP_NEXT_VID, payload_name, NULL };
-#endif
 
 
 /* Exchange types (note: two discontinuous ranges) */
@@ -569,28 +557,10 @@ enum_names sa_lifetime_names =
 static const char *const enc_mode_name[] = {
 	"ENCAPSULATION_MODE_TUNNEL",
 	"ENCAPSULATION_MODE_TRANSPORT",
-#ifdef NAT_TRAVERSAL
-	"ENCAPSULATION_MODE_UDP_TUNNEL",
-	"ENCAPSULATION_MODE_UDP_TRANSPORT",
-#endif
     };
 
-#ifdef NAT_TRAVERSAL
-static const char *const enc_udp_mode_name[] = {
-	"ENCAPSULATION_MODE_UDP_TUNNEL",
-	"ENCAPSULATION_MODE_UDP_TRANSPORT",
-    };
-#endif
-
-#ifdef NAT_TRAVERSAL
-static enum_names enc_udp_mode_names =
-    { ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS, ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS, enc_udp_mode_name, NULL };
-enum_names enc_mode_names =
-    { ENCAPSULATION_MODE_TUNNEL, ENCAPSULATION_MODE_UDP_TRANSPORT_RFC, enc_mode_name, &enc_udp_mode_names };
-#else
 enum_names enc_mode_names =
     { ENCAPSULATION_MODE_TUNNEL, ENCAPSULATION_MODE_TRANSPORT, enc_mode_name, NULL };
-#endif
 
 /* Auth Algorithm attribute */
 
@@ -808,17 +778,9 @@ static const char *const ipsec_notification_name[] = {
 	"IPSEC_INITIAL_CONTACT",
     };
 
-static const char *const notification_dpd_name[] = {
-        "R_U_THERE",
-        "R_U_THERE_ACK",
-};
-
-enum_names notification_dpd_names =
-    { R_U_THERE, R_U_THERE_ACK, notification_dpd_name, NULL };
-
 enum_names notification_names =
     { INVALID_PAYLOAD_TYPE, UNEQUAL_PAYLOAD_LENGTHS,
-       notification_name, &notification_dpd_names };  
+	notification_name, NULL };
 
 enum_names notification_status_names =
     { CONNECTED, CONNECTED,
@@ -956,19 +918,6 @@ enum_names rr_class_names = { C_IN, C_IN, rr_class_name, NULL };
 
 
 
-/* food groups (policies) */
-
-static const char *const fg_policy_name[] = {
-    "clear",
-    "clear_or_private",
-    "private_or_clear",
-    "private_or_block",
-    "block",
-    NULL
-};
-
-enum_names fg_policy_names = { FG_CLEAR, FG_BLOCK, fg_policy_name, NULL };
-
 /* look up enum names in an enum_names */
 
 const char *
@@ -1034,8 +983,10 @@ bitnamesof(const char *const table[], lset_t val)
     if (val == 0)
 	return "none";
 
-    for (tp = table, bit = 01; val != 0; bit <<= 1) {
-	if (val & bit) {
+    for (tp = table, bit = 01; val != 0; bit <<= 1)
+    {
+	if (val & bit)
+	{
 	    const char *n = *tp;
 	    size_t nl;
 
@@ -1053,7 +1004,8 @@ bitnamesof(const char *const table[], lset_t val)
 	    if (p != bitnamesbuf && p < bitnamesbuf+sizeof(bitnamesbuf) - 1)
 		*p++ = '+';
 
-	    if (bitnamesbuf+sizeof(bitnamesbuf) - p > (ptrdiff_t)nl) {
+	    if (bitnamesbuf+sizeof(bitnamesbuf) - p > (ptrdiff_t)nl)
+	    {
 		strcpy(p, n);
 		p += nl;
 	    }
@@ -1109,7 +1061,8 @@ testset(const char *const table[], lset_t val)
     lset_t bit;
     const char *const *tp;
 
-    for (tp = table, bit = 01; val != 0; bit <<= 1, tp++) {
+    for (tp = table, bit = 01; val != 0; bit <<= 1, tp++)
+    {
 	const char *n = *tp;
 
 	if (n == NULL || ((val & bit) && *n == '\0'))
