@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: constants.h,v 1.9 2003/06/19 22:09:37 ken Exp $
+ * RCSID $Id: constants.h,v 1.3 2003/09/15 12:57:38 jjo Exp $
  */
 
 extern const char compile_time_interop_options[];
@@ -67,10 +67,7 @@ typedef unsigned long long lset_t;
 
 #define CTL_SUFFIX ".ctl"	/* for UNIX domain socket pathname */
 #define LOCK_SUFFIX ".pid"	/* for pluto's lock */
-
-#define DPD_ACTION_CLEAR      0
-#define DPD_ACTION_HOLD       1
-
+#define INFO_SUFFIX ".info"     /* for UNIX domain socket for apps */
 
 /* Routines to check and display values.
  *
@@ -317,12 +314,9 @@ enum event_type {
     EVENT_SO_DISCARD,	/* discard unfinished state object */
     EVENT_RETRANSMIT,	/* Retransmit packet */
     EVENT_SA_REPLACE,	/* SA replacement event */
+    EVENT_SA_REPLACE_IF_USED,	/* SA replacement event */
     EVENT_SA_EXPIRE,	/* SA expiration event */
-    EVENT_DPD,          /* dead peer detection */
-    EVENT_DPD_TIMEOUT   /* dead peer detection timeout */
-#ifdef NAT_TRAVERSAL
-	, EVENT_NAT_T_KEEPALIVE
-#endif
+    EVENT_LOG_DAILY     /* reset certain log events/stats */
 };
 
 #define EVENT_REINIT_SECRET_DELAY		3600 /* 1 hour */
@@ -366,11 +360,14 @@ extern enum_names doi_names;
 #define IPSEC_DOI_SPI_MIN          0x100
 #define IPSEC_DOI_SPI_OUR_MIN      0x1000
 
-/* debugging settings: a set selections for reporting
+/* debugging settings: a set of selections for reporting
  * These would be more naturally situated in log.h,
  * but they are shared with whack.
  * IMPAIR_* actually change behaviour, usually badly,
  * to aid in testing.  Naturally, these are not included in ALL.
+ *
+ * NOTE: changes here must be done in concert with changes to DBGOPT_*
+ * in whack.c.  A change to WHACK_MAGIC in whack.h will be required too.
  */
 #ifdef DEBUG
 extern const char *const debug_bit_names[];
@@ -383,17 +380,20 @@ extern const char *const debug_bit_names[];
 #define DBG_LIFECYCLE	LELEM(5)	/* SA lifecycle */
 #define DBG_KLIPS	LELEM(6)	/* messages to KLIPS */
 #define DBG_DNS		LELEM(7)	/* DNS activity */
-#define DBG_NATT	LELEM(8)	/* NAT-T */
-#define DBG_OPPO	LELEM(9)	/* opportunism */
+#define DBG_OPPO	LELEM(8)	/* opportunism */
+#define DBG_CONTROLMORE LELEM(9)	/* more detailed debugging */
+
 #define DBG_PRIVATE	LELEM(10)	/* private information: DANGER! */
 
-#define IMPAIR_DELAY_ADNS_KEY_ANSWER	LELEM(11)	/* sleep before answering */
-#define IMPAIR_DELAY_ADNS_TXT_ANSWER	LELEM(12)	/* sleep before answering */
-#define IMPAIR_BUST_MI2	LELEM(13)	/* make MI2 really large */
-#define IMPAIR_BUST_MR2	LELEM(14)	/* make MI2 really large */
+#define IMPAIR0	11	/* first bit for IMPAIR_* */
+
+#define IMPAIR_DELAY_ADNS_KEY_ANSWER	LELEM(IMPAIR0+0)	/* sleep before answering */
+#define IMPAIR_DELAY_ADNS_TXT_ANSWER	LELEM(IMPAIR0+1)	/* sleep before answering */
+#define IMPAIR_BUST_MI2	LELEM(IMPAIR0+2)	/* make MI2 really large */
+#define IMPAIR_BUST_MR2	LELEM(IMPAIR0+3)	/* make MI2 really large */
 
 #define DBG_NONE	0	/* no options on, including impairments */
-#define DBG_ALL		LRANGES(DBG_RAW, DBG_OPPO)	/* all logging options on EXCEPT DBG_PRIVATE */
+#define DBG_ALL		LRANGES(DBG_RAW, DBG_CONTROLMORE)  /* all logging options on EXCEPT DBG_PRIVATE */
 #endif
 
 /* State of exchanges
@@ -475,7 +475,7 @@ enum state_kind {
 extern enum_names connection_kind_names;
 
 enum connection_kind {
-    CK_GROUP,		/* food group: instantiates to template */
+    CK_GROUP,		/* policy group: instantiates to template */
     CK_TEMPLATE,	/* abstract connection, with wildcard */
     CK_PERMANENT,	/* normal connection */
     CK_INSTANCE,	/* instance of template, created for a particular attempt */
@@ -491,6 +491,7 @@ enum connection_kind {
 
 extern enum_names routing_story;
 
+/* note that this is assumed to be ordered! */
 enum routing_t {
     RT_UNROUTED,	/* unrouted */
     RT_UNROUTED_HOLD,	/* unrouted, but HOLD shunt installed */
@@ -498,7 +499,8 @@ enum routing_t {
     RT_ROUTED_PROSPECTIVE,	/* routed, and prospective shunt installed */
     RT_ROUTED_HOLD,	/* routed, and HOLD shunt installed */
     RT_ROUTED_FAILURE,	/* routed, and failure-context shunt installed */
-    RT_ROUTED_TUNNEL	/* routed, and erouted to an IPSEC SA group */
+    RT_ROUTED_TUNNEL,	/* routed, and erouted to an IPSEC SA group */
+    RT_UNROUTED_KEYED   /* keyed, but not routed, on purpose */
 };
 
 #define routed(rs) ((rs) > RT_UNROUTED_HOLD)
@@ -531,15 +533,7 @@ extern const char *const payload_name[];
 #define ISAKMP_NEXT_D          12	/* Delete */
 #define ISAKMP_NEXT_VID        13	/* Vendor ID */
 
-#ifdef NAT_TRAVERSAL
-#define ISAKMP_NEXT_NATD_RFC   15   /* NAT-Traversal: NAT-D (rfc) */
-#define ISAKMP_NEXT_NATOA_RFC  16   /* NAT-Traversal: NAT-OA (rfc) */
-#define ISAKMP_NEXT_ROOF       17	/* roof on payload types */
-#define ISAKMP_NEXT_NATD_DRAFTS   130  /* NAT-Traversal: NAT-D (drafts) */
-#define ISAKMP_NEXT_NATOA_DRAFTS  131  /* NAT-Traversal: NAT-OA (drafts) */
-#else
 #define ISAKMP_NEXT_ROOF       14	/* roof on payload types */
-#endif
 
 /* Exchange types
  * RFC2408 "Internet Security Association and Key Management Protocol (ISAKMP)"
@@ -603,60 +597,16 @@ extern enum_names protocol_names;
     : (p)==PROTO_IPCOMP ? enum_show(&ipcomp_transformid_names, (t)) \
     : "??")
 
-/* IPsec ISAKMP transform values
- * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.2
- */
+/* many transform values are moved to freeswan/ipsec_policy.h */
 
 extern enum_names isakmp_transformid_names;
 
 #define KEY_IKE               1
 
-/* IPsec AH transform values
- * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.3
- * and in http://www.isi.edu/in-notes/iana/assignments/isakmp-registry
- */
-
 extern enum_names ah_transformid_names;
-
-#define AH_MD5                   2
-#define AH_SHA                   3
-#define AH_DES                   4
-#define AH_SHA2_256              5
-#define AH_SHA2_384              6
-#define AH_SHA2_512              7
-
-/* IPsec ESP transform values
- * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.4
- * and from http://www.isi.edu/in-notes/iana/assignments/isakmp-registry
- */
-
 extern enum_names esp_transformid_names;
-
-#define ESP_reserved             0
-#define ESP_DES_IV64             1
-#define ESP_DES                  2
-#define ESP_3DES                 3
-#define ESP_RC5                  4
-#define ESP_IDEA                 5
-#define ESP_CAST                 6
-#define ESP_BLOWFISH             7
-#define ESP_3IDEA                8
-#define ESP_DES_IV32             9
-#define ESP_RC4                 10
-#define ESP_NULL                11
-#define ESP_AES                 12
-
-/* IPCOMP transform values
- * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.5
- */
-
 extern enum_names ipcomp_transformid_names;
 
-#define IPCOMP_OUI               1
-#define IPCOMP_DEFLATE           2
-#define IPCOMP_LZS               3
-#define IPCOMP_V42BIS            4
-
 /* the following are from RFC 2393/draft-shacham-ippcp-rfc2393bis-05.txt 3.3 */
 typedef u_int16_t cpi_t;
 #define IPCOMP_CPI_SIZE          2
@@ -668,39 +618,8 @@ typedef u_int16_t cpi_t;
  */
 
 extern enum_names ident_names;
-
-#define ID_NONE                     0	/* private to Pluto */
-#define ID_IPV4_ADDR                1
-#define ID_FQDN                     2
-#define ID_USER_FQDN                3
-#define ID_IPV4_ADDR_SUBNET         4
-#define ID_IPV6_ADDR                5
-#define ID_IPV6_ADDR_SUBNET         6
-#define ID_IPV4_ADDR_RANGE          7
-#define ID_IPV6_ADDR_RANGE          8
-#define ID_DER_ASN1_DN              9
-#define ID_DER_ASN1_GN              10
-#define ID_KEY_ID                   11
-
-/* Certificate type values
- * RFC 2408 ISAKMP, chapter 3.9
- */
-
 extern enum_names cert_type_names;
 
-#define CERT_NONE			0
-#define CERT_PKCS7_WRAPPED_X509		1
-#define CERT_PGP			2
-#define CERT_DNS_SIGNED_KEY		3
-#define CERT_X509_SIGNATURE		4
-#define CERT_X509_KEY_EXCHANGE		5
-#define CERT_KERBEROS_TOKENS		6
-#define CERT_CRL			7
-#define CERT_ARL			8
-#define CERT_SPKI			9
-#define CERT_X509_ATTRIBUTE		10
-
-
 /* Policies for establishing an SA
  *
  * These are used to specify attributes (eg. encryption) and techniques
@@ -855,13 +774,6 @@ extern enum_names enc_mode_names;
 #define ENCAPSULATION_MODE_TUNNEL      1
 #define ENCAPSULATION_MODE_TRANSPORT   2
 
-#ifdef NAT_TRAVERSAL
-#define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
-#define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
-#endif
-
 /* Auth Algorithm attribute */
 
 extern enum_names auth_alg_names, extended_auth_alg_names;
@@ -890,7 +802,7 @@ extern enum_names oakley_lifetime_names;
 #define OAKLEY_LIFE_KILOBYTES 2
 
 #define OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT 3600    /* one hour */
-#define OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM 86400   /* 24 hours */
+#define OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM 28800   /* eight hours */
 
 /* Oakley PRF attribute (none defined)
  * draft-ietf-ipsec-ike-01.txt appendix A
@@ -1047,9 +959,7 @@ typedef enum {
      */
     IPSEC_RESPONDER_LIFETIME = 24576,
     IPSEC_REPLAY_STATUS =      24577,
-    IPSEC_INITIAL_CONTACT =    24578,
-    R_U_THERE =                36136,
-    R_U_THERE_ACK =            36137
+    IPSEC_INITIAL_CONTACT =    24578
     } notification_t;
 
 
@@ -1128,15 +1038,12 @@ enum dns_auth_level {
     DAL_LOCAL	/* locally provided (pretty good) */
 };
 
+/*
+ * define a macro for use in error messages
+ */
 
-/* food groups (policies) */
-
-enum fg_policy {
-    FG_CLEAR,
-    FG_CLEAR_OR_PRIVATE,
-    FG_PRIVATE_OR_CLEAR,
-    FG_PRIVATE_OR_BLOCK,
-    FG_BLOCK
-};
-
-extern enum_names fg_policy_names;
+#ifdef USE_KEYRR
+#define RRNAME "TXT or KEY"
+#else
+#define RRNAME "TXT"
+#endif
