@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: demux.c,v 1.10 2003/07/21 02:17:05 ken Exp $
+ * RCSID $Id: demux.c,v 1.3 2003/09/15 12:57:38 jjo Exp $
  */
 
 /* Ordering Constraints on Payloads
@@ -146,11 +146,6 @@
 #include "whack.h"	/* requires connections.h */
 #include "server.h"
 
-#ifdef NAT_TRAVERSAL
-#include "nat_traversal.h"
-#endif
-#include "vendor.h"
-
 /* This file does basic header checking and demux of
  * incoming packets.
  */
@@ -276,11 +271,7 @@ static const struct state_microcode state_microcode_table[] = {
      */
     { STATE_MAIN_R1, STATE_MAIN_R2
     , SMF_PSK_AUTH | SMF_DS_AUTH | SMF_REPLY
-#ifdef NAT_TRAVERSAL
-    , P(KE) | P(NONCE), P(VID) | P(CR) | P(NATD_RFC), PT(KE)
-#else
     , P(KE) | P(NONCE), P(VID) | P(CR), PT(KE)
-#endif
     , EVENT_RETRANSMIT, main_inI2_outR2 },
 
     { STATE_MAIN_R1, STATE_UNDEFINED
@@ -305,11 +296,7 @@ static const struct state_microcode state_microcode_table[] = {
      */
     { STATE_MAIN_I2, STATE_MAIN_I3
     , SMF_PSK_AUTH | SMF_DS_AUTH | SMF_INITIATOR | SMF_OUTPUT_ENCRYPTED | SMF_REPLY
-#ifdef NAT_TRAVERSAL
-    , P(KE) | P(NONCE), P(VID) | P(CR) | P(NATD_RFC), PT(ID)
-#else
     , P(KE) | P(NONCE), P(VID) | P(CR), PT(ID)
-#endif
     , EVENT_RETRANSMIT, main_inR2_outI3 },
 
     { STATE_MAIN_I2, STATE_UNDEFINED
@@ -400,11 +387,7 @@ static const struct state_microcode state_microcode_table[] = {
      */
     { STATE_QUICK_R0, STATE_QUICK_R1
     , SMF_ALL_AUTH | SMF_ENCRYPTED | SMF_REPLY
-#ifdef NAT_TRAVERSAL
-    , P(HASH) | P(SA) | P(NONCE), /* P(SA) | */ P(KE) | P(ID) | P(NATOA_RFC), PT(NONE)
-#else
     , P(HASH) | P(SA) | P(NONCE), /* P(SA) | */ P(KE) | P(ID), PT(NONE)
-#endif
     , EVENT_RETRANSMIT, quick_inI1_outR1 },
 
     /* STATE_QUICK_I1:
@@ -415,11 +398,7 @@ static const struct state_microcode state_microcode_table[] = {
      */
     { STATE_QUICK_I1, STATE_QUICK_I2
     , SMF_ALL_AUTH | SMF_INITIATOR | SMF_ENCRYPTED | SMF_REPLY
-#ifdef NAT_TRAVERSAL
-    , P(HASH) | P(SA) | P(NONCE), /* P(SA) | */ P(KE) | P(ID) | P(NATOA_RFC), PT(HASH)
-#else
     , P(HASH) | P(SA) | P(NONCE), /* P(SA) | */ P(KE) | P(ID), PT(HASH)
-#endif
     , EVENT_SA_REPLACE, quick_inR1_outI2 },
 
     /* STATE_QUICK_R1: HDR*, HASH(3) --> done
@@ -718,18 +697,6 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 		    /* note dirty trick to suppress ~ at start of format
 		     * if we know what state to blame.
 		     */
-#ifdef NAT_TRAVERSAL
-		    if ((packet_len == 1) && (buffer[0] = 0xff)
-#ifdef DEBUG
-			&& ((cur_debugging & DBG_NATT) == 0)
-#endif
-			) {
-			/* don't log NAT-T keepalive related errors unless NATT debug is
-			 * enabled
-			 */
-		    }
-		    else
-#endif
 		    plog((sender != NULL) + "~"
 			"ERROR: asynchronous network error report on %s"
 			"%s"
@@ -767,48 +734,20 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 #endif /* defined(IP_RECVERR) && defined(MSG_ERRQUEUE) */
 
 bool
-#ifdef NAT_TRAVERSAL
-_send_packet(struct state *st, const char *where, bool verbose)
-#else
 send_packet(struct state *st, const char *where)
-#endif
 {
     struct connection *c = st->st_connection;
     int port_buf;
     bool err;
 
-#ifdef NAT_TRAVERSAL
-    u_int8_t ike_pkt[MAX_OUTPUT_UDP_SIZE];
-    u_int8_t *ptr;
-    unsigned long len;
-
-    if ((c->interface->ike_float == TRUE) && (st->st_tpacket.len != 1)) {
-	if ((unsigned long) st->st_tpacket.len >
-	    (MAX_OUTPUT_UDP_SIZE-sizeof(u_int32_t))) {
-	    DBG_log("send_packet(): really too big");
-	    return FALSE;
-	}
-	ptr = ike_pkt;
-	/** Add Non-ESP marker **/
-	memset(ike_pkt, 0, sizeof(u_int32_t));
-	memcpy(ike_pkt + sizeof(u_int32_t), st->st_tpacket.ptr,
-	    (unsigned long)st->st_tpacket.len);
-	len = (unsigned long) st->st_tpacket.len + sizeof(u_int32_t);
-    }
-    else {
-	ptr = st->st_tpacket.ptr;
-	len = (unsigned long) st->st_tpacket.len;
-    }
-#endif
-
     DBG(DBG_RAW,
 	{
 	    DBG_log("sending %lu bytes for %s through %s to %s:%u:"
 		, (unsigned long) st->st_tpacket.len
 		, where
 		, c->interface->rname
-		, ip_str(&c->that.host_addr)
-		, (unsigned)c->that.host_port);
+ 		, ip_str(&c->spd.that.host_addr)
+		, (unsigned)c->spd.that.host_port);
 	    DBG_dump_chunk(NULL, st->st_tpacket);
 	});
 
@@ -817,39 +756,27 @@ send_packet(struct state *st, const char *where)
      * and restore it afterwards.
      */
 
-    port_buf = portof(&c->that.host_addr);
-    setportof(htons(c->that.host_port), &c->that.host_addr);
+    port_buf = portof(&c->spd.that.host_addr);
+    setportof(htons(c->spd.that.host_port), &c->spd.that.host_addr);
 
 #if defined(IP_RECVERR) && defined(MSG_ERRQUEUE)
     (void) check_msg_errqueue(c->interface, POLLOUT);
 #endif /* defined(IP_RECVERR) && defined(MSG_ERRQUEUE) */
 
-#ifdef NAT_TRAVERSAL
     err = sendto(c->interface->fd
-        , ptr, len, 0
-        , sockaddrof(&c->that.host_addr)
-        , sockaddrlenof(&c->that.host_addr)) != (ssize_t)len;
-#else
-    err = sendto(c->interface->fd
-        , st->st_tpacket.ptr, st->st_tpacket.len, 0
-        , sockaddrof(&c->that.host_addr)
-        , sockaddrlenof(&c->that.host_addr)) != (ssize_t)st->st_tpacket.len;
-#endif
-
+    , st->st_tpacket.ptr, st->st_tpacket.len, 0
+    , sockaddrof(&c->spd.that.host_addr)
+    , sockaddrlenof(&c->spd.that.host_addr)) != (ssize_t)st->st_tpacket.len;
+    
     /* restore port */
-    setportof(port_buf, &c->that.host_addr);
+    setportof(port_buf, &c->spd.that.host_addr);
 
     if (err)
     {
-#ifdef NAT_TRAVERSAL
-	/* do not log NAT-T Keep Alive packets */
-	if (!verbose)
-	    return FALSE;
-#endif
- 	log_errno((e, "sendto on %s to %s:%u failed in %s"
+	log_errno((e, "sendto on %s to %s:%u failed in %s"
 	    , c->interface->rname
-	    , ip_str(&c->that.host_addr)
-	    , (unsigned)c->that.host_port
+	    , ip_str(&c->spd.that.host_addr)
+	    , (unsigned)c->spd.that.host_port
 	    , where));
 	return FALSE;
     }
@@ -870,38 +797,7 @@ unexpected(struct msg_digest *md)
 static stf_status
 informational(struct msg_digest *md UNUSED)
 {
-    struct payload_digest *const n_pld = md->chain[ISAKMP_NEXT_N];
- 
-    /* If the Notification Payload is not null... */
-    if (n_pld != NULL)
-    {
-	pb_stream *const n_pbs = &n_pld->pbs;
-	struct isakmp_notification *const n = &n_pld->payload.notification;
-	int disp_len;
-	char disp_buf[200];
-
-	/* Switch on Notification Type (enum) */
-	switch (n->isan_type)
-	{
-	case R_U_THERE:
-	    return dpd_inI_outR(md->st, n, n_pbs);
-
-	case R_U_THERE_ACK:
-	    return dpd_inR(md->st, n, n_pbs);
-
-	default:
-	    if (pbs_left(n_pbs) >= sizeof(disp_buf)-1)
-		disp_len = sizeof(disp_buf)-1;
-	    else
-		disp_len = pbs_left(n_pbs);
-	    memcpy(disp_buf, n_pbs->cur, disp_len);
-	    disp_buf[disp_len] = '\0';
-            return STF_IGNORE;
-	}
-    } else {
-	return STF_IGNORE;
-    }
-
+    return STF_IGNORE;
 }
 
 /* message digest allocation and deallocation */
@@ -1021,9 +917,6 @@ read_packet(struct msg_digest *md)
     int packet_len;
     /* ??? this buffer seems *way* too big */
     u_int8_t bigbuffer[MAX_INPUT_UDP_SIZE];
-#ifdef NAT_TRAVERSAL
-    u_int8_t *_buffer = bigbuffer;
-#endif
     union
     {
 	struct sockaddr sa;
@@ -1119,34 +1012,11 @@ read_packet(struct msg_digest *md)
     cur_from = &md->sender;
     cur_from_port = md->sender_port;
 
-#ifdef NAT_TRAVERSAL
-    if (ifp->ike_float == TRUE) {
-	u_int32_t non_esp;
-	if (packet_len < (int)sizeof(u_int32_t)) {
-	    plog("recvfrom %s:%u too small packet (%d)"
-		, ip_str(cur_from), (unsigned) cur_from_port, packet_len);
-	    return FALSE;
-	}
-	memcpy(&non_esp, _buffer, sizeof(u_int32_t));
-	if (non_esp != 0) {
-	    plog("recvfrom %s:%u has no Non-ESP marker"
-		, ip_str(cur_from), (unsigned) cur_from_port);
-	    return FALSE;
-	}
-	_buffer += sizeof(u_int32_t);
-	packet_len -= sizeof(u_int32_t);
-    }
-#endif
-
     /* Clone actual message contents
      * and set up md->packet_pbs to describe it.
      */
     init_pbs(&md->packet_pbs
-#ifdef NAT_TRAVERSAL
-	, clone_bytes(_buffer, packet_len, "message buffer in comm_handle()")
-#else
 	, clone_bytes(bigbuffer, packet_len, "message buffer in comm_handle()")
-#endif
 	, packet_len, "packet");
 
     DBG(DBG_RAW | DBG_CRYPT | DBG_PARSING | DBG_CONTROL,
@@ -1161,22 +1031,6 @@ read_packet(struct msg_digest *md)
     DBG(DBG_RAW,
 	DBG_dump("", md->packet_pbs.start, pbs_room(&md->packet_pbs)));
 
-#ifdef NAT_TRAVERSAL
-	if ((pbs_room(&md->packet_pbs)==1) && (md->packet_pbs.start[0]==0xff)) {
-		/**
-		 * NAT-T Keep-alive packets should be discared by kernel ESPinUDP
-		 * layer. But boggus keep-alive packets (sent with a non-esp marker)
-		 * can reach this point. Complain and discard them.
-		 */
-		DBG(DBG_NATT,
-			DBG_log("NAT-T keep-alive (boggus ?) should not reach this point. "
-				"Ignored. Sender: %s:%u", ip_str(cur_from),
-				(unsigned) cur_from_port);
-			);
-		return FALSE;
-	}
-#endif
-
     return TRUE;
 }
 
@@ -1537,9 +1391,9 @@ process_packet(struct msg_digest **mdp)
 	/* Mark as encrypted */
 	md->encrypted = TRUE;
 
-	DBG(DBG_CRYPT, DBG_log("decrypting %u bytes using algorithm %s",
-	    (unsigned) pbs_left(&md->message_pbs),
-	    enum_show(&oakley_enc_names, st->st_oakley.encrypt)));
+	DBG(DBG_CRYPT, DBG_log("decrypting %u bytes using algorithm %s"
+	    , (unsigned) pbs_left(&md->message_pbs)
+	    , enum_show(&oakley_enc_names, st->st_oakley.encrypt)));
 
 	/* do the specified decryption
 	 *
@@ -1584,8 +1438,8 @@ process_packet(struct msg_digest **mdp)
 			    pbs_left(&md->message_pbs) , st);
 	}
 
-	DBG_cond_dump(DBG_CRYPT, "decrypted:\n", md->message_pbs.cur,
-	    md->message_pbs.roof - md->message_pbs.cur);
+	DBG_cond_dump(DBG_CRYPT, "decrypted:\n", md->message_pbs.cur
+	    , md->message_pbs.roof - md->message_pbs.cur);
 
 	DBG_cond_dump(DBG_CRYPT, "next IV:"
 	    , st->st_new_iv, st->st_new_iv_len);
@@ -1627,22 +1481,6 @@ process_packet(struct msg_digest **mdp)
 		return;
 	    }
 
-#ifdef NAT_TRAVERSAL
-	    switch (np)
-	    {
-		case ISAKMP_NEXT_NATD_RFC:
-		case ISAKMP_NEXT_NATOA_RFC:
-		    if ((!st) || (!(st->nat_traversal & NAT_T_WITH_RFC_VALUES))) {
-			/*
-			 * don't accept NAT-D/NAT-OA reloc directly in message, unless
-			 * we're using NAT-T RFC
-			 */
-			sd = NULL;
-		    }
-		    break;
-	    }
-#endif
-
 	    if (sd == NULL)
 	    {
 		/* payload type is out of range or requires special handling */
@@ -1652,16 +1490,6 @@ process_packet(struct msg_digest **mdp)
 		    sd = IS_PHASE1(from_state)
 			? &isakmp_identification_desc : &isakmp_ipsec_identification_desc;
 		    break;
-#ifdef NAT_TRAVERSAL
-		case ISAKMP_NEXT_NATD_DRAFTS:
-		    np = ISAKMP_NEXT_NATD_RFC;  /* NAT-D relocated */
-		    sd = payload_descs[np];
-		    break;
-		case ISAKMP_NEXT_NATOA_DRAFTS:
-		    np = ISAKMP_NEXT_NATOA_RFC;  /* NAT-OA relocated */
-		    sd = payload_descs[np];
-		    break;
-#endif
 		default:
 		    loglog(RC_LOG_SERIOUS, "%smessage ignored because it contains an unknown or"
 			" unexpected payload type (%s) at the outermost level"
@@ -1676,8 +1504,8 @@ process_packet(struct msg_digest **mdp)
 		if (LDISJOINT(s
 		, needed | smc->opt_payloads| LELEM(ISAKMP_NEXT_N) | LELEM(ISAKMP_NEXT_D)))
 		{
-		    loglog(RC_LOG_SERIOUS, "%smessage ignored because it contains a"
-			" payload type (%s) unexpected in this message"
+		    loglog(RC_LOG_SERIOUS, "%smessage ignored because it "
+			   "contains an unexpected payload type (%s)"
 			, excuse, enum_show(&payload_names, np));
 		    return;
 		}
@@ -1820,12 +1648,6 @@ process_packet(struct msg_digest **mdp)
 
 	for (p = md->chain[ISAKMP_NEXT_N]; p != NULL; p = p->next)
 	{
-		/* XXX This should use defines instead of raw numbers,
-		 * but it's faster than digging through structures to
-		 * find the right number -kravietz
-		 */
-		if(p->payload.notification.isan_type != 36136
-		&& p->payload.notification.isan_type != 36137)
 	    loglog(RC_LOG_SERIOUS, "ignoring informational payload, type %s"
 		, enum_show(&ipsec_notification_names, p->payload.notification.isan_type));
 	    DBG_cond_dump(DBG_PARSING, "info:", p->pbs.cur, pbs_left(&p->pbs));
@@ -1839,11 +1661,10 @@ process_packet(struct msg_digest **mdp)
 
 	for (p = md->chain[ISAKMP_NEXT_VID]; p != NULL; p = p->next)
 	{
-#if 0
 	    char vid_string[48];
-	    int vid_len = sizeof(vid_string) - 1 < pbs_left(&p->pbs)
+	    size_t vid_len = sizeof(vid_string) - 1 < pbs_left(&p->pbs)
 		? sizeof(vid_string) - 1 : pbs_left(&p->pbs);
-	    int i;
+	    size_t i;
 
 	    /* make it printable by forcing bits; truncate if long */
 	    for (i = 0; i < vid_len; i++)
@@ -1853,8 +1674,6 @@ process_packet(struct msg_digest **mdp)
 	    loglog(RC_LOG, "received Vendor ID Payload; ASCII hash: %s"
 		, vid_string);
 	    DBG_cond_dump(DBG_PARSING, "VID:", p->pbs.cur, pbs_left(&p->pbs));
-#endif
-		handle_vendorid(md, p->pbs.cur, pbs_left(&p->pbs));
 	}
     }
     md->from_state = from_state;
@@ -1881,9 +1700,6 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 
     cur_state = st = md->st;	/* might have changed */
 
-    if( st && md->dpd)
-	    st->st_dpd = md->dpd;
-
     switch (result)
     {
 	case STF_IGNORE:
@@ -1931,12 +1747,6 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		clonetochunk(st->st_tpacket, md->reply.start
 		    , pbs_offset(&md->reply), "reply packet");
 
-#ifdef NAT_TRAVERSAL
-		if (nat_traversal_enabled) {
-		    nat_traversal_change_port_lookup(md, md->st);
-		}
-#endif
-
 		/* actually send the packet
 		 * Note: this is a great place to implement "impairments"
 		 * for testing purposes.  Suppress or duplicate the
@@ -1950,6 +1760,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		time_t delay;
 		enum event_type kind = smc->timeout_event;
 		bool agreed_time = FALSE;
+		struct connection *c = st->st_connection;
 
 		switch (kind)
 		{
@@ -1968,8 +1779,8 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 			 * rekeying.  The negative consequences seem
 			 * minor.
 			 */
-			delay = st->st_connection->sa_ike_life_seconds;
-			if ((st->st_connection->policy & POLICY_DONT_REKEY)
+			delay = c->sa_ike_life_seconds;
+			if ((c->policy & POLICY_DONT_REKEY)
 			|| delay >= st->st_oakley.life_seconds)
 			{
 			    agreed_time = TRUE;
@@ -1981,7 +1792,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 			/* Delay is min of up to four things:
 			 * each can limit the lifetime.
 			 */
-			delay = st->st_connection->sa_ipsec_life_seconds;
+			delay = c->sa_ipsec_life_seconds;
 			if (st->st_ah.present
 			&& delay >= st->st_ah.attrs.life_seconds)
 			{
@@ -2002,13 +1813,18 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 			}
 		    }
 
-		    /* If we have enough time, save some for
-		     * replacement.  Otherwise, don't attempt.
-		     * In fact, we should always have time.
-		     * Whack enforces this restriction on our
-		     * own lifetime.  If a smaller liftime comes
-		     * from the other IKE, we won't have
-		     * EVENT_SA_REPLACE.
+		    /* By default, we plan to rekey.
+		     *
+		     * If there isn't enough time to rekey, plan to
+		     * expire.
+		     *
+		     * If we are --dontrekey, a lot more rules apply.
+		     * If we are the Initiator, use REPLACE_IF_USED.
+		     * If we are the Responder, and the dictated time
+		     * was unacceptable (too large), plan to REPLACE
+		     * (the only way to ratchet down the time).
+		     * If we are the Responder, and the dictated time
+		     * is acceptable, plan to EXPIRE.
 		     *
 		     * Important policy lies buried here.
 		     * For example, we favour the initiator over the
@@ -2016,23 +1832,23 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		     * sooner.  Also, fuzz is only added to the
 		     * initiator's margin.
 		     *
-		     * If POLICY_DONT_REKEY, and we are willing to use
-		     * the agreed time, we'll just let the SA expire.
 		     * Note: for ISAKMP SA, we let the negotiated
-		     * time stand.
+		     * time stand (implemented by earlier logic).
 		     */
 		    if (agreed_time
-		    && (st->st_connection->policy & POLICY_DONT_REKEY))
+		    && (c->policy & POLICY_DONT_REKEY))
 		    {
-			kind = EVENT_SA_EXPIRE;
+			kind = (smc->flags & SMF_INITIATOR)
+			    ? EVENT_SA_REPLACE_IF_USED
+			    : EVENT_SA_EXPIRE;
 		    }
-		    else
+		    if (kind != EVENT_SA_EXPIRE)
 		    {
-			unsigned long marg = st->st_connection->sa_rekey_margin;
+			unsigned long marg = c->sa_rekey_margin;
 
 			if (smc->flags & SMF_INITIATOR)
 			    marg += marg
-				* st->st_connection->sa_rekey_fuzz / 100.E0
+				* c->sa_rekey_fuzz / 100.E0
 				* (rand() / (RAND_MAX + 1.E0));
 			else
 			    marg /= 2;
@@ -2105,20 +1921,6 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		    , enum_name(&state_names, from_state)));
 	    break;
 
-#ifdef DODGE_DH_MISSING_ZERO_BUG
-	case STF_REPLACE_DOOMED_EXCHANGE:
-	    /* we've got a distateful DH shared secret --
-	     * let's renegotiate.
-	     */
-	    loglog(RC_LOG_SERIOUS, "dropping and reinitiating exchange"
-		" to avoid Pluto 1.0 bug handling DH shared secret"
-		" with leading zero byte");
-	    ipsecdoi_replace(st, st->st_try);
-	    delete_state(st);
-	    st = NULL;
-	    break;
-#endif
-
 	default:	/* a shortcut to STF_FAIL, setting md->note */
 	    passert(result > STF_FAIL);
 	    md->note = result - STF_FAIL;
