@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: dnskey.c,v 1.4 2003/07/21 02:17:06 ken Exp $
+ * RCSID $Id: dnskey.c,v 1.2 2003/09/12 19:23:32 ken Exp $
  */
 
 #include <stdlib.h>
@@ -29,6 +29,7 @@
 #include <netdb.h>	/* ??? for h_errno */
 
 #include <freeswan.h>
+#include <freeswan/ipsec_policy.h>
 
 #include "constants.h"
 #include "adns.h"	/* needs <resolv.h> */
@@ -233,7 +234,7 @@ decode_iii(u_char **pp, struct id *gw_id)
     if (*p == '@')
     {
 	/* gateway specification in this record is @FQDN */
-	err_t ugh = atoid(p, gw_id);
+	err_t ugh = atoid(p, gw_id, FALSE);
 
 	if (ugh != NULL)
 	    return builddiag("malformed FQDN in TXT " our_TXT_attr_string ": %s"
@@ -319,7 +320,7 @@ process_txt_rr_body(u_char *str
 
     zero(&gi);	/* before first use */
 
-    TRY(decode_iii(&p, &gi.gw_id));
+    TRY(decode_iii(&p, &gi.gw_id));	/* will need to unshare_id_content */
 
     if (!cr->sgw_specified)
     {
@@ -386,6 +387,8 @@ process_txt_rr_body(u_char *str
 	     */
 	    u_char kb[RSA_MAX_ENCODING_BYTES];	/* plenty of space for binary form of public key */
 	    chunk_t kbc;
+	    struct RSA_public_key r;
+
 	    err_t ugh = ttodatav(p, 0, 64, kb, sizeof(kb), &kbc.len
 		, diag_space, sizeof(diag_space), TTODATAV_SPACECOUNTS);
 
@@ -397,17 +400,24 @@ process_txt_rr_body(u_char *str
 		    , (unsigned long) sizeof(kb));
 
 	    kbc.ptr = kb;
-	    ugh = unpack_RSA_public_key(&gi.gw_key, &kbc);
+	    ugh = unpack_RSA_public_key(&r, &kbc);
 	    if (ugh != NULL)
 		return builddiag("invalid key data: %s", ugh);
+
+	    /* now find a key entry to put it in */
+	    gi.key = public_key_from_rsa(&r);
+
+	    free_RSA_public_content(&r);
+
+	    unreference_key(&cr->last_info);
+	    cr->last_info = reference_key(gi.key);
 	}
 
 	/* we're home free!  Allocate everything and add to gateways list. */
 	gi.refcnt = 1;
 	gi.pref = pref;
-	gi.dns_auth_level = dns_auth_level;
-	gi.created_time = now();
-	gi.last_tried_time = gi.last_worked_time = NO_TIME;
+	gi.key->dns_auth_level = dns_auth_level;
+	gi.key->last_tried_time = gi.key->last_worked_time = NO_TIME;
 
 	/* find insertion point */
 	for (gwip = &cr->gateways_from_dns; *gwip != NULL && (*gwip)->pref < pref; gwip = &(*gwip)->next)
@@ -427,6 +437,7 @@ process_txt_rr_body(u_char *str
 
 	gi.next = *gwip;
 	*gwip = clone_thing(gi, "gateway info");
+	unshare_id_content(&(*gwip)->gw_id);
 	unshare_id_content(&(*gwip)->client_id);
     }
 
@@ -450,6 +461,7 @@ rr_typename(int type)
 
 #ifdef USE_LWRES
 
+# ifdef USE_KEYRR
 static err_t
 process_lwdnsq_key(u_char *str
 , enum dns_auth_level dns_auth_level
@@ -462,7 +474,7 @@ process_lwdnsq_key(u_char *str
 
     char *rest = str
 	, *p
-        , *endofnumber;
+	, *endofnumber;
 
     /* flags */
     p = strsep(&rest, " \t");
@@ -515,9 +527,14 @@ process_lwdnsq_key(u_char *str
 	kbc.ptr = kb;
 	TRY(add_public_key(&cr->id, dns_auth_level, PUBKEY_ALG_RSA, &kbc
 	    , &cr->keys_from_dns));
+
+	/* keep a reference to last one */
+	unreference_key(&cr->last_info);
+	cr->last_info = reference_key(cr->keys_from_dns->key);
     }
     return NULL;
 }
+# endif /* USE_KEYRR */
 
 #else  /* ! USE_LWRES */
 
@@ -872,8 +889,9 @@ static struct_desc sig_rdata_desc = {
     sizeof(struct sig_rdata)
 };
 
-/* handle a key Resource Record. */
+/* handle a KEY Resource Record. */
 
+#ifdef USE_KEYRR
 static err_t
 process_key_rr(u_char *ptr, size_t len
 , bool doit	/* should we capture information? */
@@ -909,6 +927,7 @@ process_key_rr(u_char *ptr, size_t len
     }
     return NULL;
 }
+#endif /* USE_KEYRR */
 
 
 /* unpack TXT rr RDATA into C string.
@@ -991,9 +1010,11 @@ process_answer_section(pb_stream *pbs
 
 	    switch (type)
 	    {
+#ifdef USE_KEYRR
 	    case T_KEY:
 		ugh = process_key_rr(pbs->cur, tail, doit, *dns_auth_level, cr);
 		break;
+#endif /* USE_KEYRR */
 	    case T_TXT:
 		ugh = process_txt_rr(pbs->cur, tail, doit, *dns_auth_level, cr);
 		break;
@@ -1033,7 +1054,11 @@ process_answer_section(pb_stream *pbs
 	in_raw(NULL, tail, pbs, "RR RDATA");
     }
 
-    return doit && cr->gateways_from_dns == NULL && cr->keys_from_dns == NULL
+    return doit
+	&& cr->gateways_from_dns == NULL
+#ifdef USE_KEYRR
+	&& cr->keys_from_dns == NULL
+#endif /* USE_KEYRR */
 	? builddiag("no suitable %s record found in DNS", rr_typename(type))
 	: NULL;
 }
@@ -1202,9 +1227,11 @@ static err_t
 build_dns_name(u_char name_buf[NS_MAXDNAME + 2]
 , unsigned long serial USED_BY_DEBUG
 , const struct id *id
-, const char *typename USED_BY_DEBUG)
+, const char *typename USED_BY_DEBUG
+, const char *gwname   USED_BY_DEBUG)
 {
     /* note: all end in "." to suppress relative searches */
+    id = resolve_myid(id);
     switch (id->kind)
     {
     case ID_IPV4_ADDR:
@@ -1240,19 +1267,26 @@ build_dns_name(u_char name_buf[NS_MAXDNAME + 2]
     }
 
     case ID_FQDN:
-	if (id->name.len > NS_MAXDNAME)
-	    return "FQDN too long for domain name";
+	/* strip trailing "." characters, then add one */
+	{
+	    size_t il = id->name.len;
 
-	memcpy(name_buf, id->name.ptr, id->name.len);
-	strcpy(name_buf + id->name.len, ".");
+	    while (il > 0 && id->name.ptr[il - 1] == '.')
+		il--;
+	    if (il > NS_MAXDNAME)
+		return "FQDN too long for domain name";
+
+	    memcpy(name_buf, id->name.ptr, il);
+	    strcpy(name_buf + il, ".");
+	}
 	break;
 
     default:
 	return "can only query DNS for key for ID that is a FQDN, IPV4_ADDR, or IPV6_ADDR";
     }
 
-    DBG(DBG_CONTROL | DBG_DNS, DBG_log("DNS query %lu for %s for %s"
-	, serial, typename, name_buf));
+    DBG(DBG_CONTROL | DBG_DNS, DBG_log("DNS query %lu for %s for %s (gw: %s)"
+	, serial, typename, name_buf, gwname));
     return NULL;
 }
 
@@ -1260,7 +1294,10 @@ void
 gw_addref(struct gw_info *gw)
 {
     if (gw != NULL)
+    {
+	DBG(DBG_DNS, DBG_log("gw_addref: %p refcnt: %d++", gw, gw->refcnt))
 	gw->refcnt++;
+    }
 }
 
 void
@@ -1270,6 +1307,8 @@ gw_delref(struct gw_info **gwp)
 
     if (gw != NULL)
     {
+	DBG(DBG_DNS, DBG_log("gw_delref: %p refcnt: %d--", gw, gw->refcnt));
+
 	passert(gw->refcnt != 0);
 	gw->refcnt--;
 	if (gw->refcnt == 0)
@@ -1277,7 +1316,7 @@ gw_delref(struct gw_info **gwp)
 	    free_id_content(&gw->client_id);
 	    free_id_content(&gw->gw_id);
 	    if (gw->gw_key_present)
-		free_RSA_public_content(&gw->gw_key);
+		unreference_key(&gw->key);
 	    gw_delref(&gw->next);
 	    pfree(gw);	/* trickery could make this a tail-call */
 	}
@@ -1343,7 +1382,9 @@ release_adns_continuation(struct adns_continuation *cr)
 {
     passert(cr != next_query);
     gw_delref(&cr->gateways_from_dns);
+#ifdef USE_KEYRR
     free_public_keys(&cr->keys_from_dns);
+#endif /* USE_KEYRR */
     unshare_id_content(&cr->id);
     unshare_id_content(&cr->sgw_id);
 
@@ -1377,6 +1418,7 @@ start_adns_query(const struct id *id	/* domain to query */
 {
     static unsigned long qtid = 1;	/* query transaction id; NOTE: static */
     const char *typename = rr_typename(type);
+    char gwidb[IDTOA_BUF];
 
     /* Splice this in at head of doubly-linked list of continuations.
      * Note: this must be done before any release_adns_continuation().
@@ -1391,15 +1433,17 @@ start_adns_query(const struct id *id	/* domain to query */
     continuations = cr;
 
     cr->qtid = qtid++;
+    cr->type = type;
     cr->cont_fn = cont_fn;
     cr->id = *id;
-    cr->type = type;
     unshare_id_content(&cr->id);
     cr->sgw_specified = sgw_id != NULL;
     cr->sgw_id = cr->sgw_specified? *sgw_id : empty_id;
     unshare_id_content(&cr->sgw_id);
     cr->gateways_from_dns = NULL;
+#ifdef USE_KEYRR
     cr->keys_from_dns = NULL;
+#endif /* USE_KEYRR */
 
 #ifdef DEBUG
     cr->debugging = cur_debugging;
@@ -1407,11 +1451,13 @@ start_adns_query(const struct id *id	/* domain to query */
     cr->debugging = LEMPTY;
 #endif
 
+    idtoa(&cr->sgw_id, gwidb, sizeof(gwidb));
+
     zero(&cr->query);
 
     {
 	err_t ugh = build_dns_name(cr->query.name_buf, cr->qtid
-	    , id, typename);
+	    , id, typename, gwidb);
 
 	if (ugh != NULL)
 	{
@@ -1494,7 +1540,7 @@ send_unsent_ADNS_queries(void)
 		    , rr_typename(next_query->type)
 		    , next_query->qtid
 		    , next_query->query.name_buf);
-		DBG(DBG_DNS, DBG_log("lwdnsq query: %.*s", strlen(qbuf) - 1, qbuf));
+		DBG(DBG_DNS, DBG_log("lwdnsq query: %.*s", (int)(strlen(qbuf) - 1), qbuf));
 		buf_cur = qbuf;
 		buf_end = qbuf + strlen(qbuf);
 	    }
@@ -1596,7 +1642,10 @@ process_lwdnsq_answer(char *ts)
 	{
 	    /* "no results returned by lwdnsq" should not happen */
 	    cr->cont_fn(cr
-		, cr->keys_from_dns == NULL && cr->gateways_from_dns == NULL
+		, cr->gateways_from_dns == NULL
+#ifdef USE_KEYRR
+		  && cr->keys_from_dns == NULL
+#endif /* USE_KEYRR */
 		    ? "no results returned by lwdnsq" : NULL);
 	    cr->used = TRUE;
 	}
@@ -1647,13 +1696,21 @@ process_lwdnsq_answer(char *ts)
 
 	if (txt_ugh != NULL)
 	{
+	    DBG(DBG_DNS,
+		DBG_log("error processing TXT resource record (%s) while processing: %s"
+			, txt_ugh, rest));
 	    cr->cont_fn(cr, txt_ugh);
 	    cr->used = TRUE;
 	}
     }
     else if (strcaseeq(atype, "SIG"))
     {
-	/* ignore */
+	/* record the SIG records for posterity */
+	if (cr->last_info != NULL)
+	{
+	    pfreeany(cr->last_info->dns_sig);
+	    cr->last_info->dns_sig = clone_str(rest, "sigrecord");
+	}
     }
     else if (strcaseeq(atype, "A"))
     {
@@ -1671,18 +1728,23 @@ process_lwdnsq_answer(char *ts)
     {
 	/* ignore */
     }
+#ifdef USE_KEYRR
     else if (strcaseeq(atype, "KEY"))
     {
-	err_t txt_ugh = process_lwdnsq_key(rest
+	err_t key_ugh = process_lwdnsq_key(rest
 	    , AuthenticatedData? DAL_SIGNED : DAL_NOTSEC
 	    , cr);
 
-	if (txt_ugh != NULL)
+	if (key_ugh != NULL)
 	{
-	    cr->cont_fn(cr, txt_ugh);
+	    DBG(DBG_DNS,
+		DBG_log("error processing KEY resource record (%s) while processing: %s"
+			, key_ugh, rest));
+	    cr->cont_fn(cr, key_ugh);
 	    cr->used = TRUE;
 	}
     }
+#endif /* USE_KEYRR */
     else
     {
 	ugh = "lwdnsq: unrecognized type";
@@ -1747,7 +1809,10 @@ handle_adns_answer(void)
 	const char *typename = rr_typename(cr->query.type);
 	const char *name_buf = cr->query.name_buf;
 
-	passert(cr->gateways_from_dns == NULL && cr->keys_from_dns == NULL);
+#ifdef USE_KEYRR
+	passert(cr->keys_from_dns == NULL);
+#endif /* USE_KEYRR */
+	passert(cr->gateways_from_dns == NULL);
 	adns_in_flight--;
 	if (buf.result == -1)
 	{
@@ -1818,7 +1883,7 @@ handle_adns_answer(void)
 
 	ugh = process_lwdnsq_answer(buf);
 	if (ugh != NULL)
-	    log("failure processing lwdnsq output: %s; record: %s", ugh, buf);
+	    plog("failure processing lwdnsq output: %s; record: %s", ugh, buf);
 
 
 	passert(GLOBALS_ARE_RESET());
