@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: keys.c,v 1.10 2003/07/21 02:34:32 ken Exp $
+ * RCSID $Id: keys.c,v 1.3 2003/10/02 19:46:30 ken Exp $
  */
 
 #include <stddef.h>
@@ -33,6 +33,7 @@
 #endif
 
 #include <freeswan.h>
+#include <freeswan/ipsec_policy.h>
 
 #include "constants.h"
 #include "defs.h"
@@ -49,17 +50,13 @@
 #include "dnskey.h"	/* needs keys.h and adns.h */
 #include "log.h"
 #include "whack.h"	/* for RC_LOG_SERIOUS */
+#include "timer.h"
+
 #include "pkcs.h"
 
 /* Maximum length of filename and passphrase buffer */
-
 #define BUF_LEN		256
 
-#ifdef NAT_TRAVERSAL
-#define PB_STREAM_UNDEFINED
-#include "nat_traversal.h"
-#endif
-
 struct fld {
     const char *name;
     size_t offset;
@@ -234,10 +231,10 @@ form_keyid(chunk_t e, chunk_t n, char* keyid, unsigned *keysize)
 }
 
 
-static struct pubkeyrec*
+static struct pubkey*
 allocate_RSA_public_key(const cert_t cert)
 {
-    struct pubkeyrec *p = alloc_thing(struct pubkeyrec, "pubkeyrec");
+    struct pubkey *pk = alloc_thing(struct pubkey, "pubkey");
     chunk_t e, n;
 
     switch (cert.type)
@@ -254,20 +251,41 @@ allocate_RSA_public_key(const cert_t cert)
 	plog("RSA public key allocation error");
     }
 
-    n_to_mpz(&p->u.rsa.e, e.ptr, e.len);
-    n_to_mpz(&p->u.rsa.n, n.ptr, n.len);
+    n_to_mpz(&pk->u.rsa.e, e.ptr, e.len);
+    n_to_mpz(&pk->u.rsa.n, n.ptr, n.len);
 
-    form_keyid(e, n, p->u.rsa.keyid, &p->u.rsa.k);
+    form_keyid(e, n, pk->u.rsa.keyid, &pk->u.rsa.k);
 
 #ifdef DEBUG
-    DBG(DBG_PRIVATE, RSA_show_public_key(&p->u.rsa));
+    DBG(DBG_PRIVATE, RSA_show_public_key(&pk->u.rsa));
 #endif
 
-    p->alg = PUBKEY_ALG_RSA;
-    p->id  = empty_id;
-    p->issuer = empty_chunk;
+    pk->alg = PUBKEY_ALG_RSA;
+    pk->id  = empty_id;
+    pk->issuer = empty_chunk;
 
-    return p;
+    return pk;
+}
+
+/*
+ * free a public key struct
+ */
+static void
+free_public_key(struct pubkey *pk)
+{
+    free_id_content(&pk->id);
+    freeanychunk(pk->issuer);
+
+    /* algorithm-specific freeing */
+    switch (pk->alg)
+    {
+    case PUBKEY_ALG_RSA:
+	free_RSA_public_content(&pk->u.rsa);
+	break;
+    default:
+	bad_case(pk->alg);
+    }
+    pfree(pk);
 }
 
 struct secret *secrets = NULL;
@@ -277,26 +295,25 @@ struct secret *secrets = NULL;
  * Failure is indicated by a NULL.
  */
 static const struct secret *
-get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
+get_secret(const struct connection *c, enum PrivateKeyKind kind, bool asym)
 {
-    enum {
-	match_default = 0x01,
-	match_him = 	0x02,
-	match_me = 	0x04,
+    enum {	/* bits */
+	match_default = 01,
+	match_him = 02,
+	match_me = 04
     };
-
     unsigned int best_match = 0;
     struct secret *best = NULL;
     struct secret *s;
-    struct id *my_id = &c->this.id
-	, rw_id
-	, *his_id = &c->that.id;
+    const struct id *my_id = &c->spd.this.id
+	, *his_id = &c->spd.that.id;
+    struct id rw_id;
 
 
     /* is there a certificate assigned to this connection? */
-    if (kind == PPK_RSA && c->this.cert.type != CERT_NONE)
+    if (kind == PPK_RSA && c->spd.this.cert.type != CERT_NONE)
     {
-	struct pubkeyrec *my_public_key = allocate_RSA_public_key(c->this.cert);
+	struct pubkey *my_public_key = allocate_RSA_public_key(c->spd.this.cert);
 
 	for (s = secrets; s != NULL; s = s->next)
 	{
@@ -314,22 +331,10 @@ get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
     if (his_id_was_instantiated(c))
     {
 	/* roadwarrior: replace him with 0.0.0.0 */
-	rw_id.kind = c->that.id.kind;
-	happy(anyaddr(addrtypeof(&c->that.host_addr), &rw_id.ip_addr));
+	rw_id.kind = c->spd.that.id.kind;
+	happy(anyaddr(addrtypeof(&c->spd.that.host_addr), &rw_id.ip_addr));
 	his_id = &rw_id;
     }
-#ifdef NAT_TRAVERSAL
-    else if ((nat_traversal_enabled) && (c->policy & POLICY_PSK) &&
-	(kind == PPK_PSK) && (
-	    ((c->kind == CK_TEMPLATE) && (c->that.id.kind == ID_NONE)) ||
-	    ((c->kind == CK_INSTANCE) && (id_is_ipaddr(&c->that.id)))))
-    {
-	/* roadwarrior: replace him with 0.0.0.0 */
-	rw_id.kind = ID_IPV4_ADDR;
-	happy(anyaddr(addrtypeof(&c->that.host_addr), &rw_id.ip_addr));
-	his_id = &rw_id;
-    }
-#endif
 
     for (s = secrets; s != NULL; s = s->next)
     {
@@ -374,7 +379,7 @@ get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
 		 * allow this-side-only match to count, even if
 		 * there are other ids in the list.
 		 */
-		if(!asym)
+		if (!asym)
 		    break;
 		/* FALLTHROUGH */
 	    case match_default:	/* default all */
@@ -429,7 +434,7 @@ get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
  * Note: the result is not to be freed by the caller.
  */
 const chunk_t *
-get_preshared_secret(struct connection *c)
+get_preshared_secret(const struct connection *c)
 {
     const struct secret *s = get_secret(c, PPK_PSK, FALSE);
 
@@ -452,7 +457,7 @@ has_private_key(cert_t cert)
 {
     struct secret *s;
     bool has_key = FALSE;
-    struct pubkeyrec *pubkey = allocate_RSA_public_key(cert);
+    struct pubkey *pubkey = allocate_RSA_public_key(cert);
 
     for (s = secrets; s != NULL; s = s->next)
     {
@@ -471,7 +476,7 @@ has_private_key(cert_t cert)
  * Failure is indicated by a NULL pointer.
  */
 const struct RSA_private_key *
-get_RSA_private_key(struct connection *c)
+get_RSA_private_key(const struct connection *c)
 {
     const struct secret *s = get_secret(c, PPK_RSA, TRUE);
 
@@ -647,6 +652,101 @@ process_rsa_secret(struct RSA_private_key *rsak)
     }
 }
 
+/* process rsa key file protected with optional passphrase which can either be
+ * read from ipsec.secrets or prompted for by using whack
+ */
+static err_t
+process_rsa_keyfile(struct RSA_private_key *rsak, int whackfd)
+{
+    char filename[BUF_LEN];
+    err_t ugh = NULL;
+    rsa_privkey_t *key = NULL;
+    prompt_pass_t pass;
+
+    memset(filename,'\0', BUF_LEN);
+    memset(pass.secret,'\0', sizeof(pass.secret));
+    pass.prompt = FALSE;
+    pass.fd = whackfd;
+
+    /* we expect the filename of a PKCS#1 private key file */
+
+    if (*tok == '"' || *tok == '\'')  /* quoted filename */
+	memcpy(filename, tok+1, flp->cur - tok - 2);
+    else
+    	memcpy(filename, tok, flp->cur - tok);
+
+    if (shift())
+    {
+	/* we expect an appended passphrase or passphrase prompt*/
+	if (tokeqword("%prompt"))
+	{
+	    if (pass.fd == NULL_FD)
+		return "enter a passphrase using ipsec auto --rereadsecrets";
+	    pass.prompt = TRUE;
+	}
+	else if (*tok == '"' || *tok == '\'') /* quoted passphrase */
+	    memcpy(pass.secret, tok+1, flp->cur - tok - 2);
+	else
+	    memcpy(pass.secret, tok, flp->cur - tok);
+
+	if (shift())
+	    ugh = "RSA private key file -- unexpected token after passphrase";
+    }
+
+    key = load_rsa_private_key(filename, &pass);
+
+    if (key == NULL)
+	ugh = "error loading RSA private key file";
+    else
+    {
+	mpz_t u;
+	u_int i;
+
+	for (i = 0; ugh == NULL && i < elemsof(RSA_private_field); i++)
+	{
+	    MP_INT *n = (MP_INT *) ((char *)rsak + RSA_private_field[i].offset);
+
+	    if (key->field[i].len > 0)
+	    {
+		/* PKCS#1 RSA private key format - complete */
+		n_to_mpz(n, key->field[i].ptr, key->field[i].len);
+	    }
+	    else
+	    {
+		/* PGP RSA private key format - missing fields */
+		switch (i)
+		{
+		case 5:		/* dP = d mod (p-1) */
+		    mpz_init(u);
+		    mpz_sub_ui(u, &rsak->p, 1);
+		    mpz_mod(n, &rsak->d, u);
+		    mpz_clear(u);
+		    break;
+		case 6:		/* dQ = d mod (q-1) */
+		    mpz_init(u);
+		    mpz_sub_ui(u, &rsak->q, 1);
+		    mpz_mod(n, &rsak->d, u);
+		    mpz_clear(u);
+		    break;
+		case 7:		/* qInv = (q^-1) mod p */
+		    mpz_invert(n, &rsak->q, &rsak->p);
+		    if (mpz_cmp_ui(n, 0) < 0)
+			mpz_add(n, n, &rsak->p);
+		    passert(mpz_cmp(n, &rsak->p) < 0);
+		    break;
+		default:
+		}
+	    }
+	}
+	form_keyid(key->field[1], key->field[0], rsak->pub.keyid,
+		   &rsak->pub.k);
+	ugh = RSA_private_key_sanity(rsak);
+	pfree(key->keyobject.ptr);
+	pfree(key);
+    }
+    return ugh;
+}
+
 /*
  * process pin read from ipsec.secrets or prompted for it using whack
  */
@@ -689,10 +789,11 @@ process_pin(struct secret *s, int whackfd)
 	/* verify the pin */
 	pin_status = scx_verify_pin(sc) ? "valid" : "invalid";
     }
-
-    plog("  %s PIN for reader: %d, id: %d"
-	, pin_status, sc->reader, sc->id);
-
+#ifdef SMARTCARD
+    plog("  %s PIN for reader: %d, id: %s", pin_status, sc->reader, sc->id);
+#else
+    plog("  warning: SMARTCARD support is deactivated in pluto/Makefile!");
+#endif
     return NULL;
 }
 
@@ -729,85 +830,7 @@ process_secret(struct secret *s, int whackfd)
 	}
 	else
 	{
-	    /* we expect the filename of a PKCS#1 private key file */
-	    char filename[BUF_LEN];
-	    char passphrase[BUF_LEN];
-	    rsa_privkey_t *key = NULL;
-
-	    memset(filename,   '\0', BUF_LEN);
-	    memset(passphrase, '\0', BUF_LEN);
-
-	    if (*tok == '"' || *tok == '\'')  /* quoted filename */
-		memcpy(filename, tok+1, flp->cur - tok - 2);
-	    else
-	    	memcpy(filename, tok, flp->cur - tok);
-
-	    if (shift())
-	    {
-		/* we expect an appended passphrase */
-		if (*tok == '"' || *tok == '\'') /* quoted passphrase */
-		   memcpy(passphrase, tok+1, flp->cur - tok - 2);
-		else
-		   memcpy(passphrase, tok, flp->cur - tok);
-
-		if (shift())
-		{
-		    ugh = "RSA private key file -- unexpected token after passphrase";
-		}
-	    }
-
-	    key = load_rsa_private_key(filename, passphrase);
-
-	    if (key == NULL)
-		ugh = "error loading RSA private key file";
-	    else
-	    {
-		mpz_t u;
-		u_int i;
-
-		for (i = 0; ugh == NULL && i < elemsof(RSA_private_field); i++)
-		{
-		    MP_INT *n = (MP_INT *) ((char *)&s->u.RSA_private_key +
-							  RSA_private_field[i].offset);
-
-		    if (key->field[i].len > 0)
-		    {
-			/* PKCS#1 RSA private key format - complete */
-			n_to_mpz(n, key->field[i].ptr, key->field[i].len);
-		    }
-		    else
-		    {
-			/* PGP RSA private key format - missing fields */
-			switch (i)
-			{
-			case 5:		/* dP = d mod (p-1) */
-			    mpz_init(u);
-			    mpz_sub_ui(u, &s->u.RSA_private_key.p, 1);
-			    mpz_mod(n, &s->u.RSA_private_key.d, u);
-			    mpz_clear(u);
-			    break;
-			case 6:		/* dQ = d mod (q-1) */
-			    mpz_init(u);
-			    mpz_sub_ui(u, &s->u.RSA_private_key.q, 1);
-			    mpz_mod(n, &s->u.RSA_private_key.d, u);
-			    mpz_clear(u);
-			    break;
-			case 7:		/* qInv = (q^-1) mod p */
-			    mpz_invert(n, &s->u.RSA_private_key.q, &s->u.RSA_private_key.p);
-			    if (mpz_cmp_ui(n, 0) < 0)
-				mpz_add(n, n, &s->u.RSA_private_key.p);
-			    passert(mpz_cmp(n, &s->u.RSA_private_key.p) < 0);
-			    break;
-			default:
-			}
-		    }
-		}
-		form_keyid(key->field[1], key->field[0], s->u.RSA_private_key.pub.keyid,
-			   &s->u.RSA_private_key.pub.k);
-		ugh = RSA_private_key_sanity(&s->u.RSA_private_key);
-		pfree(key->keyobject.ptr);
-		pfree(key);
-	    }
+	   ugh = process_rsa_keyfile(&s->u.RSA_private_key, whackfd);
 	}
     }
     else if (tokeqword("pin"))
@@ -823,6 +846,7 @@ process_secret(struct secret *s, int whackfd)
     {
 	loglog(RC_LOG_SERIOUS, "\"%s\" line %d: %s"
 	    , flp->filename, flp->lino, ugh);
+	pfree(s);
     }
     else if (flushline("expected record boundary in key"))
     {
@@ -937,7 +961,7 @@ process_secret_records(int whackfd)
 		    }
 		    else
 		    {
-			ugh = atoid(tok, &id);
+			ugh = atoid(tok, &id, FALSE);
 		    }
 
 		    if (ugh != NULL)
@@ -1085,12 +1109,14 @@ load_preshared_secrets(int whackfd)
     (void) process_secrets_file(shared_secrets_file, whackfd);
 }
 
-/* public key machinery */
+/* public key machinery
+ * Note: caller must set dns_auth_level.
+ */
 
-struct pubkeyrec *
+struct pubkey *
 public_key_from_rsa(const struct RSA_public_key *k)
 {
-    struct pubkeyrec *p = alloc_thing(struct pubkeyrec, "pubkeyrec");
+    struct pubkey *p = alloc_thing(struct pubkey, "pubkey");
 
     p->id = empty_id;	/* don't know, doesn't matter */
     p->issuer = empty_chunk;
@@ -1100,7 +1126,11 @@ public_key_from_rsa(const struct RSA_public_key *k)
     mpz_init_set(&p->u.rsa.e, &k->e);
     mpz_init_set(&p->u.rsa.n, &k->n);
 
-    p->next = NULL;
+    /* note that we return a 1 reference count upon creation:
+     * invariant: recount > 0.
+     */
+    p->refcnt = 1;
+    p->installed_time = now();
     return p;
 }
 
@@ -1113,38 +1143,27 @@ void free_RSA_public_content(struct RSA_public_key *rsa)
 /* Free a public key record.
  * As a convenience, this returns a pointer to next.
  */
-struct pubkeyrec *
-free_public_key(struct pubkeyrec *p)
+struct pubkey_list *
+free_public_keyentry(struct pubkey_list *p)
 {
-    struct pubkeyrec *nxt = p->next;
-
-    free_id_content(&p->id);
-    freeanychunk(p->issuer);
-
-    /* algorithm-specific freeing */
-    switch (p->alg)
-    {
-    case PUBKEY_ALG_RSA:
-	free_RSA_public_content(&p->u.rsa);
-	break;
-    default:
-	bad_case(p->alg);
-    }
+    struct pubkey_list *nxt = p->next;
 
+    if (p->key != NULL)
+	unreference_key(&p->key);
     pfree(p);
     return nxt;
 }
 
 void
-free_public_keys(struct pubkeyrec **keys)
+free_public_keys(struct pubkey_list **keys)
 {
     while (*keys != NULL)
-	*keys = free_public_key(*keys);
+	*keys = free_public_keyentry(*keys);
 }
 
 /* root of chained public key list */
 
-struct pubkeyrec *pubkeys = NULL;	/* keys from ipsec.conf */
+struct pubkey_list *pubkeys = NULL;	/* keys from ipsec.conf */
 
 void
 free_remembered_public_keys(void)
@@ -1154,15 +1173,37 @@ free_remembered_public_keys(void)
 
 /* transfer public keys from *keys list to front of pubkeys list */
 void
-remember_public_keys(struct pubkeyrec **keys)
+transfer_to_public_keys(struct gw_info *gateways_from_dns
+#ifdef USE_KEYRR
+, struct pubkey_list **keys
+#endif /* USE_KEYRR */
+)
 {
-    struct pubkeyrec **pp = keys;
+    {
+	struct gw_info *gwp;
 
-    while (*pp != NULL)
-	pp = &(*pp)->next;
-    *pp = pubkeys;
-    pubkeys = *keys;
-    *keys = NULL;
+	for (gwp = gateways_from_dns; gwp != NULL; gwp = gwp->next)
+	{
+	    struct pubkey_list *pl = alloc_thing(struct pubkey_list, "from TXT");
+
+	    pl->key = gwp->key;	/* note: this is a transfer */
+	    gwp->key = NULL;	/* really, it is! */
+	    pl->next = pubkeys;
+	    pubkeys = pl;
+	}
+    }
+
+#ifdef USE_KEYRR
+    {
+	struct pubkey_list **pp = keys;
+
+	while (*pp != NULL)
+	    pp = &(*pp)->next;
+	*pp = pubkeys;
+	pubkeys = *keys;
+	*keys = NULL;
+    }
+#endif /* USE_KEYRR */
 }
 
 /* decode of RSA pubkey chunk
@@ -1174,7 +1215,7 @@ remember_public_keys(struct pubkeyrec **keys)
  * - modulus
  */
 err_t
-unpack_RSA_public_key(struct RSA_public_key *rsa, chunk_t *pubkey)
+unpack_RSA_public_key(struct RSA_public_key *rsa, const chunk_t *pubkey)
 {
     chunk_t exp;
     chunk_t mod;
@@ -1239,17 +1280,21 @@ same_RSA_public_key(const struct RSA_public_key *a
 
 
 static void
-install_public_key(struct pubkeyrec *p, struct pubkeyrec **head)
+install_public_key(struct pubkey *pk, struct pubkey_list **head)
 {
-    unshare_id_content(&p->id);
+    struct pubkey_list *p = alloc_thing(struct pubkey_list, "pubkey entry");
+    
+    unshare_id_content(&pk->id);
 
     /* copy issuer dn */
-    p->issuer.ptr = clone_bytes(p->issuer.ptr, p->issuer.len, "issuer dn");
+    if (pk->issuer.ptr != NULL)
+	pk->issuer.ptr = clone_bytes(pk->issuer.ptr, pk->issuer.len, "issuer dn");
 
     /* store the time the public key was installed */
-    time(&p->installed);
+    time(&pk->installed_time);
 
     /* install new key at front */
+    p->key = reference_key(pk);
     p->next = *head;
     *head = p;
 }
@@ -1258,36 +1303,72 @@ install_public_key(struct pubkeyrec *p, struct pubkeyrec **head)
 void
 delete_public_keys(const struct id *id, enum pubkey_alg alg)
 {
-    struct pubkeyrec **pp, *p;
+    struct pubkey_list **pp, *p;
+    struct pubkey *pk;
 
     for (pp = &pubkeys; (p = *pp) != NULL; )
     {
-	if (same_id(id, &p->id) && p->alg == alg)
-	    *pp = free_public_key(p);
+	pk = p->key;
+	if (same_id(id, &pk->id) && pk->alg == alg)
+	    *pp = free_public_keyentry(p);
 	else
 	    pp = &p->next;
     }
 }
 
+struct pubkey *
+reference_key(struct pubkey *pk)
+{
+    pk->refcnt++;
+    return pk;
+}
+
+void
+unreference_key(struct pubkey **pkp)
+{
+    struct pubkey *pk = *pkp;
+    char b[IDTOA_BUF];
+
+    if (pk == NULL)
+	return;
+
+    /* print stuff */
+    DBG(DBG_CONTROLMORE,
+	idtoa(&pk->id, b, sizeof(b));
+	DBG_log("unreference key: %p %s cnt %d--", pk, b, pk->refcnt)
+    )
+
+    /* cancel out the pointer */
+    *pkp = NULL;
+
+    passert(pk->refcnt != 0);
+    pk->refcnt--;
+	
+    /* we are going to free the key as the refcount will hit zero */
+    if (pk->refcnt == 0)
+ 	free_public_key(pk);
+}
+
+
 err_t
 add_public_key(const struct id *id
 , enum dns_auth_level dns_auth_level
 , enum pubkey_alg alg
-, chunk_t *key
-, struct pubkeyrec **head)
+, const chunk_t *key
+, struct pubkey_list **head)
 {
-    struct pubkeyrec *p = alloc_thing(struct pubkeyrec, "pubkeyrec");
+    struct pubkey *pk = alloc_thing(struct pubkey, "pubkey");
 
     /* first: algorithm-specific decoding of key chunk */
     switch (alg)
     {
     case PUBKEY_ALG_RSA:
 	{
-	    err_t ugh = unpack_RSA_public_key(&p->u.rsa, key);
+	    err_t ugh = unpack_RSA_public_key(&pk->u.rsa, key);
 
 	    if (ugh != NULL)
 	    {
-		pfree(p);
+		pfree(pk);
 		return ugh;
 	    }
 	}
@@ -1296,12 +1377,13 @@ add_public_key(const struct id *id
 	bad_case(alg);
     }
 
-    p->id = *id;
-    p->dns_auth_level = dns_auth_level;
-    p->alg = alg;
-    p->until = UNDEFINED_TIME;
-    p->issuer = empty_chunk;
-    install_public_key(p, head);
+    pk->id = *id;
+    pk->dns_auth_level = dns_auth_level;
+    pk->alg = alg;
+    pk->until_time = UNDEFINED_TIME;
+    pk->issuer = empty_chunk;
+   
+    install_public_key(pk, head);
     return NULL;
 }
 
@@ -1313,7 +1395,7 @@ add_x509_public_key(x509cert_t *cert , time_t until
     , enum dns_auth_level dns_auth_level)
 {
     generalName_t *gn;
-    struct pubkeyrec *p;
+    struct pubkey *pk;
     cert_t c;
 
     c.type = CERT_X509_SIGNATURE;
@@ -1323,14 +1405,14 @@ add_x509_public_key(x509cert_t *cert , time_t until
     if (cert->subjectPublicKeyAlgorithm != PUBKEY_ALG_RSA) return;
 
     /* ID type: ID_DER_ASN1_DN  (X.509 subject field) */
-    p = allocate_RSA_public_key(c);
-    p->id.kind = ID_DER_ASN1_DN;
-    p->id.name = cert->subject;
-    p->dns_auth_level = dns_auth_level;
-    p->until = until;
-    p->issuer = cert->issuer;
-    delete_public_keys(&p->id, p->alg);
-    install_public_key(p, &pubkeys);
+    pk = allocate_RSA_public_key(c);
+    pk->id.kind = ID_DER_ASN1_DN;
+    pk->id.name = cert->subject;
+    pk->dns_auth_level = dns_auth_level;
+    pk->until_time = until;
+    pk->issuer = cert->issuer;
+    delete_public_keys(&pk->id, pk->alg);
+    install_public_key(pk, &pubkeys);
 
     gn = cert->subjectAltName;
 
@@ -1341,13 +1423,13 @@ add_x509_public_key(x509cert_t *cert , time_t until
 	gntoid(&id, gn);
 	if (id.kind != ID_NONE)
 	{
-	    p = allocate_RSA_public_key(c);
-	    p->id = id;
-	    p->dns_auth_level = dns_auth_level;
-	    p->until = until;
-	    p->issuer = cert->issuer;
-	    delete_public_keys(&p->id, p->alg);
-	    install_public_key(p, &pubkeys);
+	    pk = allocate_RSA_public_key(c);
+	    pk->id = id;
+	    pk->dns_auth_level = dns_auth_level;
+	    pk->until_time = until;
+	    pk->issuer = cert->issuer;
+	    delete_public_keys(&pk->id, pk->alg);
+	    install_public_key(pk, &pubkeys);
 	}
 	gn = gn->next;
     }
@@ -1360,7 +1442,7 @@ void
 add_pgp_public_key(pgpcert_t *cert , time_t until
     , enum dns_auth_level dns_auth_level)
 {
-    struct pubkeyrec *p;
+    struct pubkey *pk;
     cert_t c;
 
     c.type = CERT_PGP;
@@ -1373,14 +1455,14 @@ add_pgp_public_key(pgpcert_t *cert , time_t until
 	return;
     }
 
-    p = allocate_RSA_public_key(c);
-    p->id.kind = ID_KEY_ID;
-    p->id.name.ptr = cert->fingerprint;
-    p->id.name.len = PGP_FINGERPRINT_SIZE;
-    p->dns_auth_level = dns_auth_level;
-    p->until = until;
-    delete_public_keys(&p->id, p->alg);
-    install_public_key(p, &pubkeys);
+    pk = allocate_RSA_public_key(c);
+    pk->id.kind = ID_KEY_ID;
+    pk->id.name.ptr = cert->fingerprint;
+    pk->id.name.len = PGP_FINGERPRINT_SIZE;
+    pk->dns_auth_level = dns_auth_level;
+    pk->until_time = until;
+    delete_public_keys(&pk->id, pk->alg);
+    install_public_key(pk, &pubkeys);
 }
 
 /*  when a X.509 certificate gets revoked, all instances of
@@ -1389,21 +1471,20 @@ add_pgp_public_key(pgpcert_t *cert , time_t until
 void
 remove_x509_public_key(const x509cert_t *cert)
 {
-    struct pubkeyrec *p, **pp, *revoked_p;
+    const cert_t c = {CERT_X509_SIGNATURE, {cert}};
+    struct pubkey_list *p, **pp;
+    struct pubkey *revoked_pk;
 
-    cert_t c = { CERT_X509_SIGNATURE, { NULL } };	/* avoid const cast warnings */
-    (const x509cert_t*) c.u.x509=cert;
-
-    revoked_p = allocate_RSA_public_key(c);
-    p         = pubkeys;
-    pp        = &pubkeys;
+    revoked_pk = allocate_RSA_public_key(c);
+    p          = pubkeys;
+    pp         = &pubkeys;
 
     while(p != NULL)
    {
-	if (same_RSA_public_key(&p->u.rsa, &revoked_p->u.rsa))
+	if (same_RSA_public_key(&p->key->u.rsa, &revoked_pk->u.rsa))
 	{
 	    /* remove p from list and free memory */
-	    *pp = free_public_key(p);
+	    *pp = free_public_keyentry(p);
 	    loglog(RC_LOG_SERIOUS,
 		"invalid RSA public key deleted");
 	}
@@ -1413,7 +1494,7 @@ remove_x509_public_key(const x509cert_t *cert)
 	}
 	p =*pp;
     }
-    free_public_key(revoked_p);
+    free_public_key(revoked_pk);
 }
 
 /*
@@ -1421,7 +1502,7 @@ remove_x509_public_key(const x509cert_t *cert)
  */
 void list_public_keys(bool utc)
 {
-    struct pubkeyrec *p = pubkeys;
+    struct pubkey_list *p = pubkeys;
 
     whack_log(RC_COMMENT, " ");
     whack_log(RC_COMMENT, "List of Public Keys:");
@@ -1429,23 +1510,25 @@ void list_public_keys(bool utc)
 
     while (p != NULL)
     {
-	if (p->alg == PUBKEY_ALG_RSA)
+	struct pubkey *key = p->key;
+
+	if (key->alg == PUBKEY_ALG_RSA)
 	{
 	    char id_buf[IDTOA_BUF];
 	    char expires_buf[TIMETOA_BUF];
 
-	    idtoa(&p->id, id_buf, IDTOA_BUF);
-	    strcpy(expires_buf, timetoa(&p->until, utc));
-
+	    idtoa(&key->id, id_buf, IDTOA_BUF);
+	    strcpy(expires_buf, timetoa(&key->until_time, utc));
 	    whack_log(RC_COMMENT, "%s, %4d RSA Key %s, until %s %s",
-		timetoa(&p->installed, utc), 8*p->u.rsa.k, p->u.rsa.keyid,
+
+		timetoa(&key->installed_time, utc), 8*key->u.rsa.k, key->u.rsa.keyid,
 		expires_buf,
-		check_expiry(p->until, PUBKEY_WARNING_INTERVAL, TRUE));
+		check_expiry(key->until_time, PUBKEY_WARNING_INTERVAL, TRUE));
 	    whack_log(RC_COMMENT,"       %s '%s'",
-		enum_show(&ident_names, p->id.kind), id_buf);
-	    if (p->issuer.len > 0)
+		enum_show(&ident_names, key->id.kind), id_buf);
+	    if (key->issuer.len > 0)
 	    {
-		dntoa(id_buf, IDTOA_BUF, p->issuer);
+		dntoa(id_buf, IDTOA_BUF, key->issuer);
 		whack_log(RC_COMMENT,"       Issuer '%s'", id_buf);
 	    }
 	}
