@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: packet.c,v 1.4 2003/06/19 22:09:38 ken Exp $
+ * RCSID $Id: packet.c,v 1.1.1.1 2003/09/12 19:03:26 ken Exp $
  */
 
 #include <stdio.h>
@@ -517,42 +517,6 @@ struct_desc isakmp_delete_desc = { "ISAKMP Delete Payload", isad_fields, sizeof(
  */
 struct_desc isakmp_vendor_id_desc = { "ISAKMP Vendor ID Payload", isag_fields, sizeof(struct isakmp_generic) };
 
-#ifdef NAT_TRAVERSAL
-/* ISAKMP NAT-Traversal NAT-D
- * layout from draft-ietf-ipsec-nat-t-ike-01.txt section 3.2
- *
- *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- * ! Next Payload  !   RESERVED    !         Payload Length        !
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- * !                 HASH of the address and port                  !
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- */
-struct_desc isakmp_nat_d = { "ISAKMP NAT-D Payload", isag_fields, sizeof(struct isakmp_generic) };
-
-/* ISAKMP NAT-Traversal NAT-OA
- * layout from draft-ietf-ipsec-nat-t-ike-01.txt section 4.2
- *
- *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- * ! Next Payload  !   RESERVED    !         Payload Length        !
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- * !   ID Type     !   RESERVED    !            RESERVED           !
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- * !         IPv4 (4 octets) or IPv6 address (16 octets)           !
- * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- */
-static field_desc isanat_oa_fields[] = {
-    { ft_enum, 8/BITS_PER_BYTE, "next payload type", &payload_names },
-    { ft_mbz, 8/BITS_PER_BYTE, NULL, NULL },
-    { ft_len, 16/BITS_PER_BYTE, "length", NULL },
-    { ft_enum, 8/BITS_PER_BYTE, "ID type", &ident_names },
-    { ft_mbz, 24/BITS_PER_BYTE, NULL, NULL },
-    { ft_end, 0, NULL, NULL }
-};
-
-struct_desc isakmp_nat_oa = { "ISAKMP NAT-OA Payload", isanat_oa_fields, sizeof(struct isakmp_nat_oa) };
-#endif
 
 /* descriptor for each payload type
  *
@@ -577,11 +541,6 @@ struct_desc *const payload_descs[ISAKMP_NEXT_ROOF] = {
     &isakmp_notification_desc,		/* 11 ISAKMP_NEXT_N (Notification) */
     &isakmp_delete_desc,		/* 12 ISAKMP_NEXT_D (Delete) */
     &isakmp_vendor_id_desc,		/* 13 ISAKMP_NEXT_VID (Vendor ID) */
-#ifdef NAT_TRAVERSAL
-	NULL,
-	&isakmp_nat_d,              /* 15=130 ISAKMP_NEXT_NATD (NAT-D) */
-	&isakmp_nat_oa,             /* 16=131 ISAKMP_NEXT_NATOA (NAT-OA) */
-#endif
 };
 
 void
@@ -614,11 +573,13 @@ DBG_print_struct(const char *label, const void *struct_ptr
 
     DBG_log("%s%s:", label, sd->name);
 
-    for (fp = sd->fields; fp->field_type != ft_end; fp++) {
+    for (fp = sd->fields; fp->field_type != ft_end; fp++)
+    {
 	int i = fp->size;
 	u_int32_t n = 0;
 
-	switch (fp->field_type) {
+	switch (fp->field_type)
+	{
 	case ft_mbz:	/* must be zero */
 	    inp += i;
 	    break;
@@ -629,7 +590,8 @@ DBG_print_struct(const char *label, const void *struct_ptr
 	case ft_loose_enum:	/* value from an enumeration with only some names known */
 	case ft_af_enum:	/* Attribute Format + value from an enumeration */
 	case ft_set:	/* bits representing set */
-	    switch (i) {
+	    switch (i)
+	    {
 	    case 8/BITS_PER_BYTE:
 		n = *(const u_int8_t *)inp;
 		break;
@@ -642,7 +604,8 @@ DBG_print_struct(const char *label, const void *struct_ptr
 	    default:
 		bad_case(i);
 	    }
-	    switch (fp->field_type) {
+	    switch (fp->field_type)
+	    {
 	    case ft_len:	/* length of this struct and any following crud */
 	    case ft_lv:		/* length/value field of attribute */
 		if (!immediate && !len_meaningful)
@@ -742,15 +705,19 @@ in_struct(void *struct_ptr, struct_desc *sd
     err_t ugh = NULL;
     u_int8_t *cur = ins->cur;
 
-    if (ins->roof - cur < (ptrdiff_t)sd->size) {
+    if (ins->roof - cur < (ptrdiff_t)sd->size)
+    {
 	ugh = builddiag("not enough room in input packet for %s", sd->name);
-    } else {
+    }
+    else
+    {
 	u_int8_t *roof = cur + sd->size;    /* may be changed by a length field */
 	u_int8_t *outp = struct_ptr;
 	bool immediate = FALSE;
 	field_desc *fp;
 
-	for (fp = sd->fields; ugh == NULL; fp++) {
+	for (fp = sd->fields; ugh == NULL; fp++)
+	{
 	    size_t i = fp->size;
 
 	    passert(ins->roof - cur >= (ptrdiff_t)i);
@@ -761,10 +728,13 @@ in_struct(void *struct_ptr, struct_desc *sd
 	    DBG(DBG_PARSING, DBG_log("%d %s"
 		, (int) (cur - ins->cur), fp->name == NULL? "" : fp->name));
 #endif
-	    switch (fp->field_type) {
+	    switch (fp->field_type)
+	    {
 	    case ft_mbz:	/* must be zero */
-		for (; i != 0; i--) {
-		    if (*cur++ != 0) {
+		for (; i != 0; i--)
+		{
+		    if (*cur++ != 0)
+		    {
 			ugh = builddiag("byte %d of %s must be zero, but is not"
 			    , (int) (cur - ins->cur), sd->name);
 			break;
@@ -786,20 +756,26 @@ in_struct(void *struct_ptr, struct_desc *sd
 		for (; i != 0; i--)
 		    n = (n << BITS_PER_BYTE) | *cur++;
 
-		switch (fp->field_type) {
+		switch (fp->field_type)
+		{
 		case ft_len:	/* length of this struct and any following crud */
 		case ft_lv:	/* length/value field of attribute */
 		{
 		    u_int32_t len = fp->field_type == ft_len? n
 			: immediate? sd->size : n + sd->size;
 
-		    if (len < sd->size) {
+		    if (len < sd->size)
+		    {
 			ugh = builddiag("%s of %s is smaller than minimum"
 			    , fp->name, sd->name);
-		    } else if (pbs_left(ins) < len) {
+		    }
+		    else if (pbs_left(ins) < len)
+		    {
 			ugh = builddiag("%s of %s is larger than can fit"
 			    , fp->name, sd->name);
-		    } else {
+		    }
+		    else
+		    {
 			roof = ins->cur + len;
 		    }
 		    break;
@@ -809,7 +785,8 @@ in_struct(void *struct_ptr, struct_desc *sd
 			immediate = TRUE;
 		    /* FALL THROUGH */
 		case ft_enum:	/* value from an enumeration */
-		    if (enum_name(fp->desc, n) == NULL) {
+		    if (enum_name(fp->desc, n) == NULL)
+		    {
 			ugh = builddiag("%s of %s has an unknown value: %lu"
 			    , fp->name, sd->name, (unsigned long)n);
 		    }
@@ -817,7 +794,8 @@ in_struct(void *struct_ptr, struct_desc *sd
 		case ft_loose_enum:	/* value from an enumeration with only some names known */
 		    break;
 		case ft_set:	/* bits representing set */
-		    if (!testset(fp->desc, n)) {
+		    if (!testset(fp->desc, n))
+		    {
 			ugh = builddiag("bitset %s of %s has unknown member(s): %s"
 			    , fp->name, sd->name, bitnamesof(fp->desc, n));
 		    }
@@ -826,7 +804,8 @@ in_struct(void *struct_ptr, struct_desc *sd
 			break;
 		}
 		i = fp->size;
-		switch (i) {
+		switch (i)
+		{
 		case 8/BITS_PER_BYTE:
 		    *(u_int8_t *)outp = n;
 		    break;
@@ -844,14 +823,16 @@ in_struct(void *struct_ptr, struct_desc *sd
 	    }
 
 	    case ft_raw:	/* bytes to be left in network-order */
-		for (; i != 0; i--) {
+		for (; i != 0; i--)
+		{
 		    *outp++ = *cur++;
 		}
 		break;
 
 	    case ft_end:	/* end of field list */
 		passert(cur == ins->cur + sd->size);
-		if (obj_pbs != NULL) {
+		if (obj_pbs != NULL)
+		{
 		    init_pbs(obj_pbs, ins->cur, roof - ins->cur, sd->name);
 		    obj_pbs->container = ins;
 		    obj_pbs->desc = sd;
@@ -932,10 +913,13 @@ out_struct(const void *struct_ptr, struct_desc *sd
     DBG(DBG_EMITTING
 	, DBG_prefix_print_struct(outs, "emit ", struct_ptr, sd, obj_pbs==NULL));
 
-    if (outs->roof - cur < (ptrdiff_t)sd->size) {
+    if (outs->roof - cur < (ptrdiff_t)sd->size)
+    {
 	ugh = builddiag("not enough room left in output packet to place %s"
 	    , sd->name);
-    } else {
+    }
+    else
+    {
 	bool immediate = FALSE;
 	pb_stream obj;
 	field_desc *fp;
@@ -943,7 +927,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 	obj.lenfld = NULL;  /* until a length field is discovered */
 	obj.lenfld_desc = NULL;
 
-	for (fp = sd->fields; ugh == NULL; fp++) {
+	for (fp = sd->fields; ugh == NULL; fp++)
+	{
 	    size_t i = fp->size;
 
 	    passert(outs->roof - cur >= (ptrdiff_t)i);
@@ -954,7 +939,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 	    DBG(DBG_EMITTING, DBG_log("%d %s"
 		, (int) (cur - outs->cur), fp->name == NULL? "" : fp->name);
 #endif
-	    switch (fp->field_type) {
+	    switch (fp->field_type)
+	    {
 	    case ft_mbz:	/* must be zero */
 		inp += i;
 		for (; i != 0; i--)
@@ -970,7 +956,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 	    {
 		u_int32_t n = 0;
 
-		switch (i) {
+		switch (i)
+		{
 		case 8/BITS_PER_BYTE:
 		    n = *(const u_int8_t *)inp;
 		    break;
@@ -984,7 +971,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 		    bad_case(i);
 		}
 
-		switch (fp->field_type) {
+		switch (fp->field_type)
+		{
 		case ft_len:	/* length of this struct and any following crud */
 		case ft_lv:	/* length/value field of attribute */
 		    if (immediate)
@@ -1003,7 +991,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 			immediate = TRUE;
 		    /* FALL THROUGH */
 		case ft_enum:	/* value from an enumeration */
-		    if (enum_name(fp->desc, n) == NULL) {
+		    if (enum_name(fp->desc, n) == NULL)
+		    {
 			ugh = builddiag("%s of %s has an unknown value: %lu"
 			    , fp->name, sd->name, (unsigned long)n);
 		    }
@@ -1011,7 +1000,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 		case ft_loose_enum:	/* value from an enumeration with only some names known */
 		    break;
 		case ft_set:	/* bits representing set */
-		    if (!testset(fp->desc, n)) {
+		    if (!testset(fp->desc, n))
+		    {
 			ugh = builddiag("bitset %s of %s has unknown member(s): %s"
 			    , fp->name, sd->name, bitnamesof(fp->desc, n));
 		    }
@@ -1020,7 +1010,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 		    break;
 		}
 
-		while (i-- != 0) {
+		while (i-- != 0)
+		{
 		    cur[i] = (u_int8_t)n;
 		    n >>= BITS_PER_BYTE;
 		}
@@ -1029,9 +1020,8 @@ out_struct(const void *struct_ptr, struct_desc *sd
 		break;
 	    }
 	    case ft_raw:	/* bytes to be left in network-order */
-		for (; i != 0; i--) {
+		for (; i != 0; i--)
 		    *cur++ = *inp++;
-		}
 		break;
 	    case ft_end:	/* end of field list */
 		passert(cur == outs->cur + sd->size);
@@ -1044,9 +1034,12 @@ out_struct(const void *struct_ptr, struct_desc *sd
 		obj.roof = outs->roof;	/* limit of possible */
 		/* obj.lenfld and obj.lenfld_desc already set */
 
-		if (obj_pbs == NULL) {
+		if (obj_pbs == NULL)
+		{
 		    close_output_pbs(&obj); /* fill in length field, if any */
-		} else {
+		}
+		else
+		{
 		    /* We set outs->cur to outs->roof so that
 		     * any attempt to output something into outs
 		     * before obj is closed will trigger an error.
@@ -1081,7 +1074,7 @@ out_generic(u_int8_t np, struct_desc *sd
 
 bool
 out_generic_raw(u_int8_t np, struct_desc *sd
-, pb_stream *outs, const void *bytes, size_t len, const char *name)
+, pb_stream *outs, void *bytes, size_t len, const char *name)
 {
     pb_stream pbs;
 
@@ -1092,38 +1085,6 @@ out_generic_raw(u_int8_t np, struct_desc *sd
     return TRUE;
 }
 
-bool
-out_modify_previous_np(u_int8_t np, pb_stream *outs)
-{
-        size_t len = (outs->cur - outs->start), offset;
-        if (len < sizeof(struct isakmp_hdr)) {
-                return FALSE;
-        }
-        else if (len == sizeof(struct isakmp_hdr)) {
-                struct isakmp_hdr *hdr = (struct isakmp_hdr *)outs->start;
-                hdr->isa_np = np;
-                return TRUE;
-        }
-        else {
-                struct isakmp_generic *hdr;
-                for (offset = sizeof(struct isakmp_hdr); offset < len ;
-                        offset += ntohs(hdr->isag_length)) {
-                        if ((len - offset) < sizeof(struct isakmp_generic))
-                                return FALSE;
-                        hdr = (struct isakmp_generic *)(outs->start+offset);
-                        if ((len - offset) < ntohs(hdr->isag_length))
-                                return FALSE;
-                        if ((len - offset) == ntohs(hdr->isag_length)) {
-                                hdr->isag_np = np;
-                                return TRUE;
-                        }
-                }
-        }
-        return FALSE;
-}
-
-
-
 bool
 out_raw(const void *bytes, size_t len, pb_stream *outs, const char *name)
 {
@@ -1179,7 +1140,8 @@ close_output_pbs(pb_stream *pbs)
 	    len -= sizeof(struct isakmp_attribute);
 	DBG(DBG_EMITTING, DBG_log("emitting length of %s: %lu"
 	    , pbs->name, (unsigned long) len));
-	while (i-- != 0) {
+	while (i-- != 0)
+	{
 	    pbs->lenfld[i] = (u_int8_t)len;
 	    len >>= BITS_PER_BYTE;
 	}
