@@ -1,5 +1,5 @@
 /* Loading of PEM encoded files with optional encryption
- * Copyright (C) 2001-2002 Andreas Steffen, Zuercher Hochschule Winterthur
+ * Copyright (C) 2001-2003 Andreas Steffen, Zuercher Hochschule Winterthur
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,7 +11,8 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pem.h,v 1.1 2003/05/05 11:54:26 ken Exp $
+ * RCSID $Id: pem.h,v 1.1 2003/09/12 19:24:42 ken Exp $
  */
 
-extern err_t pemtobin(chunk_t *blob, const char* passphrase, bool *pgp);
+extern err_t pemtobin(chunk_t *blob, prompt_pass_t *pass, const char* label
+    , bool *pgp);
