@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: server.c,v 1.5 2003/07/21 02:17:07 ken Exp $
+ * RCSID $Id: server.c,v 1.2 2003/09/12 19:23:33 ken Exp $
  */
 
 #include <stdio.h>
@@ -58,6 +58,7 @@
 #include "packet.h"
 #include "demux.h"  /* needs packet.h */
 #include "rcv_whack.h"
+#include "rcv_info.h"
 #include "keys.h"
 #include "adns.h"	/* needs <resolv.h> */
 #include "dnskey.h"	/* needs keys.h and adns.h */
@@ -66,10 +67,6 @@
 #include <pfkeyv2.h>
 #include <pfkey.h>
 
-#ifdef NAT_TRAVERSAL
-#include "nat_traversal.h"
-#endif
-
 /*
  *  Server main loop and socket initialization routines.
  */
@@ -80,6 +77,10 @@ static const int on = TRUE;	/* by-reference parameter; constant, we hope */
 int ctl_fd = NULL_FD;	/* file descriptor of control (whack) socket */
 struct sockaddr_un ctl_addr = { AF_UNIX, DEFAULT_CTLBASE CTL_SUFFIX };
 
+/* info (showpolicy) socket */
+int policy_fd = NULL_FD;
+struct sockaddr_un info_addr= { AF_UNIX, DEFAULT_CTLBASE INFO_SUFFIX };
+
 /* Initialize the control socket.
  * Note: this is called very early, so little infrastructure is available.
  * It is important that the socket is created before the original
@@ -126,6 +127,50 @@ delete_ctl_socket(void)
     unlink(ctl_addr.sun_path);
 }
 
+#ifdef IPSECPOLICY
+/* Initialize the info socket.
+ */
+err_t
+init_info_socket(void)
+{
+    err_t failed = NULL;
+
+    delete_info_socket();	/* preventative medicine */
+    info_fd = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (info_fd == -1)
+	failed = "create";
+    else if (fcntl(info_fd, F_SETFD, FD_CLOEXEC) == -1)
+	failed = "fcntl FD+CLOEXEC";
+    else if (setsockopt(info_fd, SOL_SOCKET, SO_REUSEADDR, (const void *)&on, sizeof(on)) < 0)
+	failed = "setsockopt";
+    else
+    {
+	/* this socket should be openable by all proceses */
+	mode_t ou = umask(0);
+
+	if (bind(info_fd, (struct sockaddr *)&info_addr
+	, offsetof(struct sockaddr_un, sun_path) + strlen(info_addr.sun_path)) < 0)
+	    failed = "bind";
+	umask(ou);
+    }
+
+    /* 64 might be big enough, and the system may limit us anyway.
+     */
+    if (failed == NULL && listen(info_fd, 64) < 0)
+	failed = "listen() on";
+
+    return failed == NULL? NULL : builddiag("could not %s info socket: %d %s"
+	    , failed, errno, strerror(errno));
+}
+
+void
+delete_info_socket(void)
+{
+    unlink(info_addr.sun_path);
+}
+#endif /* IPSECPOLICY */
+
+
 bool listening = FALSE;	/* should we pay attention to IKE messages? */
 
 struct iface *interfaces = NULL;	/* public interfaces */
@@ -404,86 +449,6 @@ find_raw_ifaces6(void)
     return rifaces;
 }
 
-#if 1
-static int
-create_socket(struct raw_iface *ifp, const char *v_name, int port)
-{
-    int fd = socket(addrtypeof(&ifp->addr), SOCK_DGRAM, IPPROTO_UDP);
-    int fcntl_flags;
-
-    if (fd < 0)
-    {
-	log_errno((e, "socket() in process_raw_ifaces()"));
-	return -1;
-    }
-
-#if 1
-    /* Set socket Nonblocking */
-    if ((fcntl_flags=fcntl(fd, F_GETFL)) >= 0) {
-	if (!(fcntl_flags & O_NONBLOCK)) {
-	    fcntl_flags |= O_NONBLOCK;
-	    fcntl(fd, F_SETFL, fcntl_flags);
-	}
-    }
-#endif
-
-    if (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)
-    {
-	log_errno((e, "fcntl(,, FD_CLOEXEC) in process_raw_ifaces()"));
-	close(fd);
-	return -1;
-    }
-
-    if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR
-    , (const void *)&on, sizeof(on)) < 0)
-    {
-	log_errno((e, "setsockopt SO_REUSEADDR in process_raw_ifaces()"));
-	close(fd);
-	return -1;
-    }
-
-    /* To improve error reporting.  See ip(7). */
-#if defined(IP_RECVERR) && defined(MSG_ERRQUEUE)
-    if (setsockopt(fd, SOL_IP, IP_RECVERR
-    , (const void *)&on, sizeof(on)) < 0)
-    {
-	log_errno((e, "setsockopt IP_RECVERR in process_raw_ifaces()"));
-	close(fd);
-	return -1;
-    }
-#endif
-
-    /* With IPv6, there is no fragmentation after
-     * it leaves our interface.  PMTU discovery
-     * is mandatory but doesn't work well with IKE (why?).
-     * So we must set the IPV6_USE_MIN_MTU option.
-     * See draft-ietf-ipngwg-rfc2292bis-01.txt 11.1
-     */
-#ifdef IPV6_USE_MIN_MTU	/* YUCK: not always defined */
-    if (addrtypeof(&ifp->addr) == AF_INET6
-    && setsockopt(fd, SOL_SOCKET, IPV6_USE_MIN_MTU
-      , (const void *)&on, sizeof(on)) < 0)
-    {
-	log_errno((e, "setsockopt IPV6_USE_MIN_MTU in process_raw_ifaces()"));
-	close(fd);
-	return -1;
-    }
-#endif
-
-    setportof(htons(port), &ifp->addr);
-    if (bind(fd, sockaddrof(&ifp->addr), sockaddrlenof(&ifp->addr)) < 0)
-    {
-	log_errno((e, "bind() for %s/%s %s:%u in process_raw_ifaces()"
-	    , ifp->name, v_name
-	    , ip_str(&ifp->addr), (unsigned) port));
-	close(fd);
-	return -1;
-    }
-    setportof(htons(pluto_port), &ifp->addr);
-    return fd;
-}
-#endif
-
 static void
 process_raw_ifaces(struct raw_iface *rifaces)
 {
@@ -585,17 +550,61 @@ process_raw_ifaces(struct raw_iface *rifaces)
 		if (q == NULL)
 		{
 		    /* matches nothing -- create a new entry */
-		    int fd = create_socket(ifp, v->name, pluto_port);
+		    int fd = socket(addrtypeof(&ifp->addr), SOCK_DGRAM, IPPROTO_UDP);
 
 		    if (fd < 0)
+		    {
+			log_errno((e, "socket() in process_raw_ifaces()"));
 			break;
+		    }
 
-#ifdef NAT_TRAVERSAL
-		    if (nat_traversal_enabled) {
-			nat_traversal_espinudp_socket(fd, ESPINUDP_WITH_NON_IKE);
+		    if (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)
+		    {
+			log_errno((e, "fcntl(,, FD_CLOEXEC) in process_raw_ifaces()"));
+			break;
+		    }
+
+		    if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR
+		    , (const void *)&on, sizeof(on)) < 0)
+		    {
+			log_errno((e, "setsockopt SO_REUSEADDR in process_raw_ifaces()"));
+			break;
+		    }
+
+		    /* To improve error reporting.  See ip(7). */
+#if defined(IP_RECVERR) && defined(MSG_ERRQUEUE)
+		    if (setsockopt(fd, SOL_IP, IP_RECVERR
+		    , (const void *)&on, sizeof(on)) < 0)
+		    {
+			log_errno((e, "setsockopt IP_RECVERR in process_raw_ifaces()"));
+			break;
 		    }
 #endif
 
+		    /* With IPv6, there is no fragmentation after
+		     * it leaves our interface.  PMTU discovery
+		     * is mandatory but doesn't work well with IKE (why?).
+		     * So we must set the IPV6_USE_MIN_MTU option.
+		     * See draft-ietf-ipngwg-rfc2292bis-01.txt 11.1
+		     */
+#ifdef IPV6_USE_MIN_MTU	/* YUCK: not always defined */
+		    if (addrtypeof(&ifp->addr) == AF_INET6
+		    && setsockopt(fd, SOL_SOCKET, IPV6_USE_MIN_MTU
+		      , (const void *)&on, sizeof(on)) < 0)
+		    {
+			log_errno((e, "setsockopt IPV6_USE_MIN_MTU in process_raw_ifaces()"));
+			break;
+		    }
+#endif
+		    setportof(htons(pluto_port), &ifp->addr);
+		    if (bind(fd, sockaddrof(&ifp->addr), sockaddrlenof(&ifp->addr)) < 0)
+		    {
+			log_errno((e, "bind() for %s/%s %s:%u in process_raw_ifaces()"
+			    , ifp->name, v->name
+			    , ip_str(&ifp->addr), (unsigned) pluto_port));
+			break;
+		    }
+
 		    q = alloc_thing(struct iface, "struct iface");
 		    q->rname = clone_str(ifp->name, "real device name");
 		    q->vname = clone_str(v->name, "virtual device name");
@@ -606,28 +615,6 @@ process_raw_ifaces(struct raw_iface *rifaces)
 		    interfaces = q;
 		    plog("adding interface %s/%s %s"
 			, q->vname, q->rname, ip_str(&q->addr));
-
-#ifdef NAT_TRAVERSAL
-		    if (nat_traversal_support_port_floating) {
-			fd = create_socket(ifp, v->name, NAT_T_IKE_FLOAT_PORT);
-			if (fd < 0)
-			    break;
-			nat_traversal_espinudp_socket(fd,
-			    ESPINUDP_WITH_NON_ESP);
-			q = alloc_thing(struct iface, "struct iface");
-			q->rname = clone_str(ifp->name, "real device name");
-			q->vname = clone_str(v->name, "virtual device name");
-			q->addr = ifp->addr;
-			setportof(htons(NAT_T_IKE_FLOAT_PORT), &q->addr);
-			q->fd = fd;
-			q->next = interfaces;
-			q->change = IFN_ADD;
-			q->ike_float = TRUE;
-			interfaces = q;
-			plog("adding interface %s/%s %s:%d",
-			    q->vname, q->rname, ip_str(&q->addr), NAT_T_IKE_FLOAT_PORT);
-		    }
-#endif
 		    break;
 		}
 
@@ -638,16 +625,6 @@ process_raw_ifaces(struct raw_iface *rifaces)
 		{
 		    /* matches -- rejuvinate old entry */
 		    q->change = IFN_KEEP;
-#ifdef NAT_TRAVERSAL
-		    /* look for other interfaces to keep (due to NAT-T) */
-		    for (q = q->next ; q ; q = q->next) {
-			if (streq(q->rname, ifp->name)
-			    && streq(q->vname, v->name)
-			    && sameaddr(&q->addr, &ifp->addr)) {
-				q->change = IFN_KEEP;
-			    }
-		    }
-#endif
 		    break;
 		}
 
@@ -769,6 +746,11 @@ call_server(void)
 	    FD_ZERO(&readfds);
 	    FD_ZERO(&writefds);
 	    FD_SET(ctl_fd, &readfds);
+#ifdef IPSECPOLICY
+	    FD_SET(info_fd, &readfds);
+	    if (maxfd < info_fd)
+		maxfd = info_fd;
+#endif
 
 	    /* the only write file-descriptor of interest */
 	    if (adns_qfd != NULL_FD && unsent_ADNS_queries)
@@ -915,13 +897,26 @@ call_server(void)
 		ndes--;
 	    }
 
+#ifdef IPSECPOLICY
+	    if (FD_ISSET(info_fd, &readfds))
+	    {
+		passert(ndes > 0);
+		DBG(DBG_CONTROL,
+		    DBG_log(BLANK_FORMAT);
+		    DBG_log("*received info message"));
+		info_handle(info_fd);
+		passert(GLOBALS_ARE_RESET());
+		ndes--;
+	    }
+#endif
+
 	    passert(ndes == 0);
 	}
     }
 }
 
 /*
- * Local Variables: 
+ * Local Variables:
  * c-basic-offset: 4
  * End Variables:
  */
