@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: spdb.c,v 1.8 2003/07/21 02:17:07 ken Exp $
+ * RCSID $Id: spdb.c,v 1.3 2003/09/15 12:57:39 jjo Exp $
  */
 
 #include <stdio.h>
@@ -22,6 +22,7 @@
 #include <arpa/inet.h>
 
 #include <freeswan.h>
+#include <freeswan/ipsec_policy.h>
 
 #include "constants.h"
 #include "defs.h"
@@ -34,7 +35,7 @@
 #include "state.h"
 #include "packet.h"
 #include "keys.h"
-#include "kernel.h"
+#include "kernel.h"	/* needs connections.h */
 #include "log.h"
 #include "spdb.h"
 #include "whack.h"	/* for RC_LOG_SERIOUS */
@@ -50,10 +51,6 @@
 #define AD(x) x, elemsof(x)	/* Array Description */
 #define AD_NULL NULL, 0
 
-#ifdef NAT_TRAVERSAL
-#include "nat_traversal.h"
-#endif
-
 /**************** Oakely (main mode) SA database ****************/
 
 /* arrays of attributes for transforms, preshared key */
@@ -645,35 +642,9 @@ out_sa(pb_stream *outs
 		    if (p->protoid != PROTO_IPCOMP
 		    || st->st_policy & POLICY_TUNNEL)
 		    {
-#ifdef NAT_TRAVERSAL
-#ifndef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
-			if ((st->nat_traversal & NAT_T_DETECTED) &&
-				(!(st->st_policy & POLICY_TUNNEL))) {
-				/* Inform user that we will not respect policy and only
-				 * propose Tunnel Mode
-				 */
-				loglog(RC_LOG_SERIOUS, "NAT-Traversal: "
-					"Transport Mode not allowed due to security concerns -- "
-					"using Tunnel mode");
-			}
-#endif
-#endif
 			out_attr(ENCAPSULATION_MODE
-#ifdef NAT_TRAVERSAL
-#ifdef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
-			    , NAT_T_ENCAPSULATION_MODE(st,st->st_policy)
-#else
-				/* If NAT-T is detected, use UDP_TUNNEL as long as Transport
-				 * Mode has security concerns.
-				 *
-				 * User has been informed of that
-				 */
-			    , NAT_T_ENCAPSULATION_MODE(st,POLICY_TUNNEL)
-#endif
-#else /* ! NAT_TRAVERSAL */
 			    , st->st_policy & POLICY_TUNNEL
 			      ? ENCAPSULATION_MODE_TUNNEL : ENCAPSULATION_MODE_TRANSPORT
-#endif
 			    , attr_desc, attr_val_descs
 			    , &trans_pbs);
 		    }
@@ -967,6 +938,9 @@ parse_isakmp_sa_body(
 #endif
 		    switch (val)
 		    {
+#if 0	/* we don't feel DES is safe */
+		    case OAKLEY_DES_CBC:
+#endif
 		    case OAKLEY_3DES_CBC:
 			ta.encrypt = val;
 			ta.encrypter = crypto_get_encrypter(val);
@@ -1021,11 +995,11 @@ parse_isakmp_sa_body(
 				char mid[IDTOA_BUF]
 				    , hid[IDTOA_BUF];
 
-				idtoa(&c->this.id, mid, sizeof(mid));
+				idtoa(&c->spd.this.id, mid, sizeof(mid));
 				if (his_id_was_instantiated(c))
 				    strcpy(hid, "%any");
 				else
-				    idtoa(&c->that.id, hid, sizeof(hid));
+				    idtoa(&c->spd.that.id, hid, sizeof(hid));
 				ugh = builddiag("Can't authenticate: no preshared key found for `%s' and `%s'"
 				    , mid, hid);
 			    }
@@ -1476,89 +1450,7 @@ parse_ipsec_transform(struct isakmp_transform *trans
 		break;
 	    case ENCAPSULATION_MODE | ISAKMP_ATTR_AF_TV:
 		ipcomp_inappropriate = FALSE;
-#ifdef NAT_TRAVERSAL
-		switch (val) {
-			case ENCAPSULATION_MODE_TUNNEL:
-			case ENCAPSULATION_MODE_TRANSPORT:
-				if (st->nat_traversal & NAT_T_DETECTED) {
-					loglog(RC_LOG_SERIOUS,
-						"%s must only be used if "
-						"NAT-Traversal is not detected",
-						enum_name(&enc_mode_names, val));
-					/*
-					 * Accept it anyway because SSH-Sentinel does not
-					 * use UDP_TUNNEL or UDP_TRANSPORT for the diagnostic.
-					 *
-					 * remove when SSH-Sentinel is fixed
-					 */
-#ifdef I_DONT_CARE_OF_SSH_SENTINEL
-					return FALSE;
-#endif
-				}
-				attrs->encapsulation = val;
-				break;
-			case ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS:
-#ifndef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
-				loglog(RC_LOG_SERIOUS,
-					"NAT-Traversal: Transport mode disabled due "
-					"to security concerns");
-				return FALSE;
-				break;
-#endif
-			case ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS:
-				if (st->nat_traversal & NAT_T_WITH_RFC_VALUES) {
-					loglog(RC_LOG_SERIOUS,
-						"%s must only be used with old IETF drafts",
-						enum_name(&enc_mode_names, val));
-					return FALSE;
-				}
-				else if (st->nat_traversal & NAT_T_DETECTED) {
-					attrs->encapsulation = val - ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS + ENCAPSULATION_MODE_TUNNEL;
-				}
-				else {
-					loglog(RC_LOG_SERIOUS,
-						"%s must only be used if "
-						"NAT-Traversal is detected",
-						enum_name(&enc_mode_names, val));
-					return FALSE;
-				}
-				break;
-			case ENCAPSULATION_MODE_UDP_TRANSPORT_RFC:
-#ifndef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
-				loglog(RC_LOG_SERIOUS,
-					"NAT-Traversal: Transport mode disabled due "
-					"to security concerns");
-				return FALSE;
-				break;
-#endif
-			case ENCAPSULATION_MODE_UDP_TUNNEL_RFC:
-				if ((st->nat_traversal & NAT_T_DETECTED) &&
-					(st->nat_traversal & NAT_T_WITH_RFC_VALUES)) {
-					attrs->encapsulation = val - ENCAPSULATION_MODE_UDP_TUNNEL_RFC + ENCAPSULATION_MODE_TUNNEL;
-				}
-				else if (st->nat_traversal & NAT_T_DETECTED) {
-					loglog(RC_LOG_SERIOUS,
-						"%s must only be used with NAT-T RFC",
-						enum_name(&enc_mode_names, val));
-					return FALSE;
-				}
-				else {
-					loglog(RC_LOG_SERIOUS,
-						"%s must only be used if "
-						"NAT-Traversal is detected",
-						enum_name(&enc_mode_names, val));
-					return FALSE;
-				}
-				break;
-			default:
-				loglog(RC_LOG_SERIOUS,
-					"unknown ENCAPSULATION_MODE %d in IPSec SA", val);
-				return FALSE;
-				break;
-		}
-#else
 		attrs->encapsulation = val;
-#endif
 		break;
 	    case AUTH_ALGORITHM | ISAKMP_ATTR_AF_TV:
 		attrs->auth = val;
@@ -2006,7 +1898,7 @@ parse_ipsec_sa_body(
 			    " in %s Transform from %s"
 			    , enum_name(&auth_alg_names, ah_attrs.auth)
 			    , enum_show(&ah_transformid_names, ah_attrs.transid)
-			    , ip_str(&c->that.host_addr)));
+			    , ip_str(&c->spd.that.host_addr)));
 		    continue;   /* try another */
 		}
 		break;	/* we seem to be happy */
@@ -2063,7 +1955,7 @@ parse_ipsec_sa_body(
 			    DBG(DBG_CONTROL | DBG_CRYPT
 				    , DBG_log("ESP_NULL Transform Proposal from %s"
 					" does not satisfy POLICY_ENCRYPT"
-					, ip_str(&c->that.host_addr)));
+					, ip_str(&c->spd.that.host_addr)));
 			    continue;   /* try another */
 			}
 			break;
@@ -2073,7 +1965,7 @@ parse_ipsec_sa_body(
 			DBG(DBG_CONTROL | DBG_CRYPT
 			    , DBG_log("unsupported ESP Transform %s from %s"
 				, enum_show(&esp_transformid_names, esp_attrs.transid)
-				, ip_str(&c->that.host_addr)));
+				, ip_str(&c->spd.that.host_addr)));
 			continue;   /* try another */
 		}
 #ifndef NO_KERNEL_ALG
@@ -2086,7 +1978,7 @@ parse_ipsec_sa_body(
 			{
 			    DBG(DBG_CONTROL | DBG_CRYPT
 				, DBG_log("ESP from %s must either have AUTH or be combined with AH"
-				    , ip_str(&c->that.host_addr)));
+				    , ip_str(&c->spd.that.host_addr)));
 			    continue;   /* try another */
 			}
 			break;
@@ -2099,7 +1991,7 @@ parse_ipsec_sa_body(
 			DBG(DBG_CONTROL | DBG_CRYPT
 			    , DBG_log("unsupported ESP auth alg %s from %s"
 				, enum_show(&auth_alg_names, esp_attrs.auth)
-				, ip_str(&c->that.host_addr)));
+				, ip_str(&c->spd.that.host_addr)));
 			continue;   /* try another */
 		}
 
@@ -2130,7 +2022,7 @@ parse_ipsec_sa_body(
 	{
 	    DBG(DBG_CONTROL | DBG_CRYPT
 		, DBG_log("policy for \"%s\" requires encryption but ESP not in Proposal from %s"
-		    , c->name, ip_str(&c->that.host_addr)));
+		    , c->name, ip_str(&c->spd.that.host_addr)));
 	    continue;	/* we needed encryption, but didn't find ESP */
 	}
 	else if ((st->st_policy & POLICY_AUTHENTICATE) && !ah_seen)
@@ -2138,7 +2030,7 @@ parse_ipsec_sa_body(
 	    DBG(DBG_CONTROL | DBG_CRYPT
 		, DBG_log("policy for \"%s\" requires authentication"
 		    " but none in Proposal from %s"
-		    , c->name, ip_str(&c->that.host_addr)));
+		    , c->name, ip_str(&c->spd.that.host_addr)));
 	    continue;	/* we need authentication, but we found neither ESP nor AH */
 	}
 
@@ -2147,20 +2039,21 @@ parse_ipsec_sa_body(
 	    int previous_transnum = -1;
 	    int tn;
 
+#ifdef NEVER	/* we think IPcomp is working now */
 	    /**** FUDGE TO PREVENT UNREQUESTED IPCOMP:
 	     **** NEEDED BECAUSE OUR IPCOMP IS EXPERIMENTAL (UNSTABLE).
 	     ****/
 	    if (!(st->st_policy & POLICY_COMPRESS))
 	    {
 		plog("compression proposed by %s, but policy for \"%s\" forbids it"
-		    , ip_str(&c->that.host_addr), c->name);
+		    , ip_str(&c->spd.that.host_addr), c->name);
 		continue;	/* unwanted compression proposal */
 	    }
-
+#endif
 	    if (!can_do_IPcomp)
 	    {
 		plog("compression proposed by %s, but KLIPS is not configured with IPCOMP"
-		    , ip_str(&c->that.host_addr));
+		    , ip_str(&c->spd.that.host_addr));
 		continue;
 	    }
 
@@ -2201,7 +2094,7 @@ parse_ipsec_sa_body(
 			DBG(DBG_CONTROL | DBG_CRYPT
 			    , DBG_log("unsupported IPCOMP Transform %s from %s"
 				, enum_show(&ipcomp_transformid_names, ipcomp_attrs.transid)
-				, ip_str(&c->that.host_addr)));
+				, ip_str(&c->spd.that.host_addr)));
 			continue;   /* try another */
 		}
 
