@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: state.c,v 1.7 2003/07/21 02:17:07 ken Exp $
+ * RCSID $Id: state.c,v 1.2 2003/09/12 19:23:33 ken Exp $
  */
 
 #include <stdio.h>
@@ -36,7 +36,7 @@
 #include "smartcard.h"
 #include "connections.h"	/* needs id.h */
 #include "state.h"
-#include "kernel.h"
+#include "kernel.h"	/* needs connections.h */
 #include "log.h"
 #include "packet.h"	/* so we can calculate sizeof(struct isakmp_hdr) */
 #include "keys.h"	/* for free_public_key */
@@ -122,8 +122,8 @@ generate_msgid(struct state *isakmp_sa)
 
 	if (--timeout == 0)
 	{
-	    plog("gave up looking for unique msgid; using 0x%08lx",
-		(unsigned long) msgid);
+	    plog("gave up looking for unique msgid; using 0x%08lx"
+		, (unsigned long) msgid);
 	    break;
 	}
     }
@@ -227,7 +227,7 @@ void
 insert_state(struct state *st)
 {
     struct state **p = state_hash(st->st_icookie, st->st_rcookie
-	, &st->st_connection->that.host_addr);
+	, &st->st_connection->spd.that.host_addr);
 
     passert(st->st_hashchain_prev == NULL && st->st_hashchain_next == NULL);
 
@@ -255,7 +255,8 @@ unhash_state(struct state *st)
 {
     /* unlink from forward chain */
     struct state **p = st->st_hashchain_prev == NULL
-	? state_hash(st->st_icookie, st->st_rcookie, &st->st_connection->that.host_addr)
+	? state_hash(st->st_icookie, st->st_rcookie
+		     , &st->st_connection->spd.that.host_addr)
 	: &st->st_hashchain_prev->st_hashchain_next;
 
     /* unlink from forward chain */
@@ -282,6 +283,9 @@ release_whack(struct state *st)
 }
 
 /* delete a state object */
+/*
+ * XXX-mcr this will need attention in the co-terminal case
+ */
 void
 delete_state(struct state *st)
 {
@@ -289,8 +293,6 @@ delete_state(struct state *st)
     struct state *old_cur_state = cur_state == st? NULL : cur_state;
 
     set_cur_state(st);
-    if(st->st_dpd_event != NULL)
-	    delete_dpd_event(st);
 
     /* if there is a suspended state transition, disconnect us */
     if (st->st_suspended_md != NULL)
@@ -349,7 +351,7 @@ delete_state(struct state *st)
 	}
     }
 
-    free_public_keys(&st->st_peer_pubkey);
+    unreference_key(&st->st_peer_pubkey);
 
     if (st->st_sec_in_use)
 	mpz_clear(&(st->st_sec));
@@ -375,7 +377,11 @@ delete_state(struct state *st)
     pfree(st);
 }
 
-/* Is a connection in use by some state? */
+/*
+ * Is a connection in use by some state?
+ *
+ * XXX-mcr co-terminal will have to look at this.
+ */
 bool
 states_use_connection(struct connection *c)
 {
@@ -397,6 +403,7 @@ delete_states_by_connection(struct connection *c)
     int pass;
     /* this kludge avoids an n^2 algorithm */
     enum connection_kind ck = c->kind;
+    struct spd_route *sr;
 
     if (ck == CK_INSTANCE)
 	c->kind = CK_GOING_AWAY;
@@ -446,9 +453,16 @@ delete_states_by_connection(struct connection *c)
     }
 
     passert(c->newest_ipsec_sa == SOS_NOBODY
-	&& c->newest_isakmp_sa == SOS_NOBODY
-	&& c->eroute_owner == SOS_NOBODY
-	&& c->routing != RT_ROUTED_TUNNEL);
+	    && c->newest_isakmp_sa == SOS_NOBODY);
+
+
+    sr = &c->spd;
+    while (sr != NULL)
+    {
+	passert(sr->eroute_owner == SOS_NOBODY);
+	passert(sr->routing != RT_ROUTED_TUNNEL);
+	sr = sr->next;
+    }
 
     if (ck == CK_INSTANCE)
     {
@@ -462,13 +476,17 @@ delete_states_by_connection(struct connection *c)
  * Caller must insert_state().
  */
 struct state *
-duplicate_state(const struct state *st)
+duplicate_state(struct state *st)
 {
     struct state *nst;
 
     DBG(DBG_CONTROL, DBG_log("duplicating state object #%lu",
 	st->st_serialno));
 
+    /* record use of the Phase 1 state */
+    st->st_outbound_count++;
+    st->st_outbound_time = now();
+
     nst = new_state();
 
     memcpy(nst->st_icookie, st->st_icookie, COOKIE_SIZE);
@@ -493,21 +511,6 @@ duplicate_state(const struct state *st)
     return nst;
 }
 
-#if 1
-void for_each_state(void *(f)(struct state *, void *data), void *data)
-{
-	struct state *st, *ocs = cur_state;
-	int i;
-	for (i=0; i<STATE_TABLE_SIZE; i++) {
-		for (st = statetable[i]; st != NULL; st = st->st_hashchain_next) {
-			set_cur_state(st);
-			f(st, data);
-		}
-	}
-	cur_state = ocs;
-}
-#endif
-
 /*
  * Find a state object.
  */
@@ -520,7 +523,7 @@ find_state(const u_char *icookie
     struct state *st = *state_hash(icookie, rcookie, peer);
 
     while (st != (struct state *) NULL)
-	if (sameaddr(peer, &st->st_connection->that.host_addr)
+	if (sameaddr(peer, &st->st_connection->spd.that.host_addr)
 	&& memcmp(icookie, st->st_icookie, COOKIE_SIZE) == 0
 	&& memcmp(rcookie, st->st_rcookie, COOKIE_SIZE) == 0
 	&& msgid == st->st_msgid)
@@ -532,9 +535,9 @@ find_state(const u_char *icookie
 	if (st == NULL)
 	    DBG_log("state object not found");
 	else
-	    DBG_log("state object #%lu found, in %s",
-		st->st_serialno,
-		enum_show(&state_names, st->st_state)));
+	    DBG_log("state object #%lu found, in %s"
+		, st->st_serialno
+		, enum_show(&state_names, st->st_state)));
 
     return st;
 }
@@ -614,102 +617,225 @@ find_phase1_state(const struct connection *c, lset_t ok_states)
     return best;
 }
 
+void
+state_eroute_usage(ip_subnet *ours, ip_subnet *his
+, unsigned long count, time_t nw)
+{
+    struct state *st;
+    int i;
+
+    for (i = 0; i < STATE_TABLE_SIZE; i++)
+    {
+	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+	{
+	    struct connection *c = st->st_connection;
+
+	    if (IS_IPSEC_SA_ESTABLISHED(st->st_state)
+		&& c->spd.eroute_owner == st->st_serialno
+		&& c->spd.routing == RT_ROUTED_TUNNEL
+		&& samesubnet(&c->spd.this.client, ours)
+		&& samesubnet(&c->spd.that.client, his))
+	    {
+		if (st->st_outbound_count != count)
+		{
+		    st->st_outbound_count = count;
+		    st->st_outbound_time = nw;
+		}
+		return;
+	    }
+	}
+    }
+    DBG(DBG_CONTROL,
+	{
+	    char ourst[SUBNETTOT_BUF];
+	    char hist[SUBNETTOT_BUF];
+
+	    subnettot(ours, 0, ourst, sizeof(ourst));
+	    subnettot(his, 0, hist, sizeof(hist));
+	    DBG_log("unknown tunnel eroute %s -> %s found in scan"
+		, ourst, hist);
+	});
+}
+
+void fmt_state(struct state *st, time_t n
+, char *state_buf, size_t state_buf_len
+, char *state_buf2, size_t state_buf2_len)
+{
+    /* what the heck is interesting about a state? */
+    const struct connection *c = st->st_connection;
+
+    long delta = st->st_event->ev_time >= n
+	? (long)(st->st_event->ev_time - n)
+	: -(long)(n - st->st_event->ev_time);
+
+    char inst[CONN_INST_BUF];
+    const char *np1 = c->newest_isakmp_sa == st->st_serialno
+	? "; newest ISAKMP" : "";
+    const char *np2 = c->newest_ipsec_sa == st->st_serialno
+	? "; newest IPSEC" : "";
+    const char *eo = c->spd.eroute_owner == st->st_serialno
+	? "; eroute owner" : "";
+
+    passert(st->st_event != 0);
+
+    fmt_conn_instance(c, inst);
+
+    snprintf(state_buf, state_buf_len
+	, "#%lu: \"%s\"%s %s (%s); %s in %lds%s%s%s"
+	, st->st_serialno
+	, c->name, inst
+	, enum_name(&state_names, st->st_state)
+	, state_story[st->st_state - STATE_MAIN_R0]
+	, enum_name(&timer_event_names, st->st_event->ev_type)
+	, delta
+	, np1, np2, eo);
+
+    /* print out SPIs if SAs are established */
+    if (state_buf2_len != 0)
+	state_buf2[0] = '\0';	/* default to empty */
+    if (IS_IPSEC_SA_ESTABLISHED(st->st_state))
+    {
+	char lastused[40];	/* should be plenty long enough */
+	char buf[SATOT_BUF*6 + 1];
+	char *p = buf;
+
+#	define add_said(adst, aspi, aproto) { \
+	    ip_said s; \
+	    \
+	    initsaid(adst, aspi, aproto, &s); \
+	    if (p < &buf[sizeof(buf)-1]) \
+	    { \
+		*p++ = ' '; \
+		p += satot(&s, 0, p, &buf[sizeof(buf)] - p) - 1; \
+	    } \
+	}
+
+	/* XXX - mcr last used is really an attribute of the connection */
+	lastused[0] = '\0';
+	if (c->spd.eroute_owner == st->st_serialno
+	&& st->st_outbound_count != 0)
+	{
+	    snprintf(lastused, sizeof(lastused)
+		, " used %lus ago;"
+		, (unsigned long) (now() - st->st_outbound_time));
+	}
+
+	*p = '\0';
+	if (st->st_ah.present)
+	{
+	    add_said(&c->spd.that.host_addr, st->st_ah.attrs.spi, SA_AH);
+	    add_said(&c->spd.this.host_addr, st->st_ah.our_spi, SA_AH);
+	}
+	if (st->st_esp.present)
+	{
+	    add_said(&c->spd.that.host_addr, st->st_esp.attrs.spi, SA_ESP);
+	    add_said(&c->spd.this.host_addr, st->st_esp.our_spi, SA_ESP);
+	}
+	if (st->st_ipcomp.present)
+	{
+	    add_said(&c->spd.that.host_addr, st->st_ipcomp.attrs.spi, SA_COMP);
+	    add_said(&c->spd.this.host_addr, st->st_ipcomp.our_spi, SA_COMP);
+	}
+#ifdef KLIPS
+	if (st->st_ah.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
+	|| st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
+	|| st->st_ipcomp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
+	{
+	    add_said(&c->spd.that.host_addr, st->st_tunnel_out_spi, SA_IPIP);
+	    add_said(&c->spd.this.host_addr, st->st_tunnel_in_spi, SA_IPIP);
+	}
+#endif
+	snprintf(state_buf2, state_buf2_len
+	    , "#%lu: \"%s\"%s%s%s"
+	    , st->st_serialno
+	    , c->name, inst
+	    , lastused
+	    , buf);
+
+#	undef add_said
+    }
+}
+
+/*
+ * sorting logic is:
+ *
+ *  name
+ *  type
+ *  instance#
+ *  isakmp_sa (XXX probably wrong)
+ *
+ */
+static int
+state_compare(const void *a, const void *b)
+{
+    const struct state *sap = *(const struct state *const *)a;
+    struct connection *ca = sap->st_connection;
+    const struct state *sbp = *(const struct state *const *)b;
+    struct connection *cb = sbp->st_connection;
+
+    /* DBG_log("comparing %s to %s", ca->name, cb->name); */
+
+    return connection_compare(ca, cb);
+}
+
 void
 show_states_status(void)
 {
     time_t n = now();
     int i;
+    char state_buf[LOG_WIDTH];
+    char state_buf2[LOG_WIDTH];
+    int count;
+    struct state **array;
 
+    /* make count of states */
+    count = 0;
     for (i = 0; i < STATE_TABLE_SIZE; i++)
     {
 	struct state *st;
 
 	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
 	{
-	    /* what the heck is interesting about a state? */
-	    const struct connection *c = st->st_connection;
-	    long delta = st->st_event->ev_time >= n
-		? (long)(st->st_event->ev_time - n)
-		: -(long)(n - st->st_event->ev_time);
-	    char inst[CONN_INST_BUF];
-	    const char *np1 = c->newest_isakmp_sa == st->st_serialno
-		? "; newest ISAKMP" : "";
-	    const char *np2 = c->newest_ipsec_sa == st->st_serialno
-		? "; newest IPSEC" : "";
-	    const char *eo = c->eroute_owner == st->st_serialno
-		? "; eroute owner" : "";
-
-	    passert(st->st_event != 0);
-
-	    fmt_conn_instance(c, inst);
-
-	    whack_log(RC_COMMENT
-		, "#%lu: \"%s\"%s %s (%s); %s in %lds%s%s%s"
-		, st->st_serialno
-		, c->name, inst
-		, enum_name(&state_names, st->st_state)
-		, state_story[st->st_state - STATE_MAIN_R0]
-		, enum_name(&timer_event_names, st->st_event->ev_type)
-		, delta
-		, np1, np2, eo);
-
-	    /* print out SPIs if SAs are established */
-	    if (IS_IPSEC_SA_ESTABLISHED(st->st_state))
-	    {
-		char buf[SATOT_BUF*6 + 1];
-		char *p = buf;
-
-#		define add_said(adst, aspi, aproto) { \
-		    ip_said s; \
-		    \
-		    initsaid(adst, aspi, aproto, &s); \
-		    if (p < &buf[sizeof(buf)-1]) \
-		    { \
-			*p++ = ' '; \
-			p += satot(&s, 0, p, &buf[sizeof(buf)] - p) - 1; \
-		    } \
-		}
-
-
-		*p = '\0';
-		if (st->st_ah.present)
-		{
-		    add_said(&c->that.host_addr, st->st_ah.attrs.spi, SA_AH);
-		    add_said(&c->this.host_addr, st->st_ah.our_spi, SA_AH);
-		}
-		if (st->st_esp.present)
-		{
-		    add_said(&c->that.host_addr, st->st_esp.attrs.spi, SA_ESP);
-		    add_said(&c->this.host_addr, st->st_esp.our_spi, SA_ESP);
-		}
-		if (st->st_ipcomp.present)
-		{
-		    add_said(&c->that.host_addr, st->st_ipcomp.attrs.spi, SA_COMP);
-		    add_said(&c->this.host_addr, st->st_ipcomp.our_spi, SA_COMP);
-		}
-#ifdef KLIPS
-		if (st->st_ah.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
-		|| st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
-		|| st->st_ipcomp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
-		{
-		    add_said(&c->that.host_addr, st->st_tunnel_out_spi, SA_IPIP);
-		    add_said(&c->this.host_addr, st->st_tunnel_in_spi, SA_IPIP);
-		}
-#endif
-		whack_log(RC_COMMENT
-		    , "#%lu: \"%s\"%s%s"
-		    , st->st_serialno
-		    , c->name, inst
-		    , buf);
-
-#		undef add_said
-	    }
-
-	    /* show any associated pending Phase 2s */
-	    if (IS_PHASE1(st->st_state))
-		show_pending_phase2(st->st_connection->host_pair, st);
+	    count++;
 	}
     }
+
+    /* build the array */
+    array = alloc_bytes(sizeof(struct state *)*count, "state array");
+    count = 0;
+    for (i = 0; i < STATE_TABLE_SIZE; i++)
+    {
+	struct state *st;
+
+	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+	{
+	    array[count++]=st;
+	}
+    }
+
+    /* sort it! */
+    qsort(array, count, sizeof(struct state *), state_compare);
+
+    /* now print sorted results */
+    for (i = 0; i < count; i++)
+    {
+	struct state *st;
+
+	st = array[i];
+
+	fmt_state(st, n, state_buf, sizeof(state_buf)
+		  , state_buf2, sizeof(state_buf2));
+	whack_log(RC_COMMENT, state_buf);
+	if (state_buf2[0] != '\0')
+	    whack_log(RC_COMMENT, state_buf2);
+
+	/* show any associated pending Phase 2s */
+	if (IS_PHASE1(st->st_state))
+	    show_pending_phase2(st->st_connection->host_pair, st);
+    }
+
+    /* free the array */
+    pfree(array);
 }
 
 /* Given that we've used up a range of unused CPI's,
@@ -792,8 +918,8 @@ startover:
 	for (s = statetable[i]; s != NULL; s = s->st_hashchain_next)
 	{
 	    if (s->st_ipcomp.present
-	    && sameaddr(&s->st_connection->that.host_addr
-	      , &st->st_connection->that.host_addr)
+	    && sameaddr(&s->st_connection->spd.that.host_addr
+	      , &st->st_connection->spd.that.host_addr)
 	    && cpi == s->st_ipcomp.attrs.spi)
 	    {
 		if (++tries == 20)
@@ -805,23 +931,8 @@ startover:
     return cpi;
 }
 
-/* Immediately schedule a replace event for all states for a peer.
+/*
+ * Local Variables:
+ * c-basic-offset:4
+ * End:
  */
-void replace_states_by_peer(const ip_address *peer)
-{
-    struct state *st = NULL;
-    int i;
-    /* struct event *ev;     currently unused */
-
-    for (i = 0; st == NULL && i < STATE_TABLE_SIZE; i++)
-	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
-	    /* Only replace if it already has a replace event. */
-	    if (sameaddr(&st->st_connection->that.host_addr, peer)
-		    && (IS_ISAKMP_SA_ESTABLISHED(st->st_state) || IS_IPSEC_SA_ESTABLISHED(st->st_state))
-		    && st->st_event->ev_type == EVENT_SA_REPLACE)
-	    {
-		delete_event(st);
-		delete_dpd_event(st);
-		event_schedule(EVENT_SA_REPLACE, 0, st);
-	    }
-}
