@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: whack.c,v 1.9 2003/07/21 02:17:07 ken Exp $
+ * RCSID $Id: whack.c,v 1.3 2003/09/15 12:57:39 jjo Exp $
  */
 
 #include <stdio.h>
@@ -39,8 +39,8 @@
 static void
 help(void)
 {
-    fprintf(stderr,
-	"Usage:\n\n"
+    fprintf(stderr
+	, "Usage:\n\n"
 	"all forms:"
 	    " [--optionsfrom <filename>]"
 	    " [--ctlbase <path>]"
@@ -53,8 +53,8 @@ help(void)
 	"connection: whack"
 	    " --name <connection_name>"
 	    " \\\n   "
-	    " [--ipv4 | --ipv6 ]"
-	    " [--tunnelipv4 | --tunnelipv6 ]"
+	    " [--ipv4 | --ipv6]"
+	    " [--tunnelipv4 | --tunnelipv6]"
 	    " \\\n   "
 	    " (--host <ip-address> | --id <identity> | --cert <path>)"
 	    " [--ca <distinguished name>]"
@@ -66,7 +66,6 @@ help(void)
 	    " [--clientprotoport <protocol>/<port>]"
 	    " [--dnskeyondemand]"
 	    " \\\n   "
-	    " [--sourceip <ip-address>]"
 	    " [--updown <updown>]"
 	    " --to"
 	    " (--host <ip-address> | --id <identity> | --cert <path>)"
@@ -79,7 +78,6 @@ help(void)
 	    " [--clientprotoport <protocol>/<port>]"
 	    " [--dnskeyondemand]"
 	    " \\\n   "
-	    " [--sourceip <ip-address>]"
 	    " [--updown <updown>]"
 	    " [--psk]"
 	    " [--rsasig]"
@@ -100,10 +98,6 @@ help(void)
 	    " \\\n   "
 	    " [--esp <esp-algos>]"
 	    " \\\n   "
-	    " [--dpddelay <seconds> --dpdtimeout <seconds>]"
-	    " \\\n   "
-	    " [--dpdaction (clear|hold)]"
-	    " \\\n   "
 	    " [--dontrekey]"
 	    " \\\n   "
 	    " [--initiateontraffic|--pass|--drop|--reject]"
@@ -114,16 +108,15 @@ help(void)
 	    " (--route | --unroute)"
 	    " --name <connection_name>"
 	    "\n\n"
-	"initiation: whack"
+	"initiation:"
+	    "\n "
+	    " whack"
 	    " (--initiate | --terminate)"
 	    " --name <connection_name>"
 	    " [--asynchronous]"
 	    "\n\n"
-	"opportunistic initiation:"
-	    "\n"
-	    " whack"
-	    " [--ipv4 | --ipv6 ]"
-	    " [--tunnelipv4 | --tunnelipv6 ]"
+	"opportunistic initiation: whack"
+	    " [--tunnelipv4 | --tunnelipv6]"
 	    " \\\n   "
 	    " --oppohere <ip-address>"
 	    " --oppothere <ip-address>"
@@ -140,6 +133,9 @@ help(void)
 	    " [--addkey]"
 	    " [--pubkeyrsa <key>]"
 	    "\n\n"
+	"myid: whack"
+	    " --myid <id>"
+	    "\n\n"
 #ifdef DEBUG
 	"debug: whack [--name <connection_name>]"
 	    " \\\n   "
@@ -180,8 +176,8 @@ help(void)
 	"shutdown: whack"
 	    " --shutdown"
 	    "\n\n"
-	"FreeS/WAN %s\n",
-	ipsec_version_code());
+	"FreeS/WAN %s\n"
+	, ipsec_version_code());
 }
 
 static const char *label = NULL;	/* --label operand, saved for diagnostics */
@@ -243,6 +239,8 @@ enum {
     OPT_ADDKEY,
     OPT_PUBKEYRSA,
 
+    OPT_MYID,
+
     OPT_ROUTE,
     OPT_UNROUTE,
 
@@ -289,7 +287,6 @@ enum {
     END_CLIENTWITHIN,
     END_CLIENTPROTOPORT,
     END_DNSKEYONDEMAND,
-    END_SOURCEIP,
     END_UPDOWN,
     	
 #define END_LAST  END_UPDOWN	/* last end description*/
@@ -324,9 +321,6 @@ enum {
     CD_RKMARGIN,
     CD_RKFUZZ,
     CD_KTRIES,
-    CD_DPDDELAY,
-    CD_DPDTIMEOUT,
-    CD_DPDACTION,
     CD_IKE,
     CD_PFSGROUP,
     CD_ESP	
@@ -335,6 +329,7 @@ enum {
 #ifdef DEBUG	/* must be last so others are less than 32 to fit in lset_t */
 #   define DBGOPT_FIRST DBGOPT_NONE
     ,
+    /* NOTE: these definitions must match DBG_* and IMPAIR_* in constants.h */
     DBGOPT_NONE,
     DBGOPT_ALL,
 
@@ -347,6 +342,8 @@ enum {
     DBGOPT_KLIPS,	/* same order as DBG_* */
     DBGOPT_DNS,		/* same order as DBG_* */
     DBGOPT_OPPO,	/* same order as DBG_* */
+    DBGOPT_CONTROLMORE,	/* same order as DBG_* */
+
     DBGOPT_PRIVATE,	/* same order as DBG_* */
 
     DBGOPT_IMPAIR_DELAY_ADNS_KEY_ANSWER,	/* same order as IMPAIR_* */
@@ -384,6 +381,8 @@ static const struct option long_opts[] = {
     { "addkey", no_argument, NULL, OPT_ADDKEY + OO },
     { "pubkeyrsa", required_argument, NULL, OPT_PUBKEYRSA + OO },
 
+    { "myid", required_argument, NULL, OPT_MYID + OO },
+
     { "route", no_argument, NULL, OPT_ROUTE + OO },
     { "unroute", no_argument, NULL, OPT_UNROUTE + OO },
 
@@ -425,7 +424,6 @@ static const struct option long_opts[] = {
     { "clientwithin", required_argument, NULL, END_CLIENTWITHIN + OO },
     { "clientprotoport", required_argument, NULL, END_CLIENTPROTOPORT + OO },
     { "dnskeyondemand", no_argument, NULL, END_DNSKEYONDEMAND + OO },
-    { "sourceip", required_argument, NULL, END_SOURCEIP + OO },
     { "updown", required_argument, NULL, END_UPDOWN + OO },
 
 
@@ -470,9 +468,6 @@ static const struct option long_opts[] = {
     { "rekeywindow", required_argument, NULL, CD_RKMARGIN + OO + NUMERIC_ARG },	/* OBSOLETE */
     { "rekeyfuzz", required_argument, NULL, CD_RKFUZZ + OO + NUMERIC_ARG },
     { "keyingtries", required_argument, NULL, CD_KTRIES + OO + NUMERIC_ARG },
-    { "dpddelay", required_argument, NULL, CD_DPDDELAY + OO + NUMERIC_ARG },
-    { "dpdtimeout", required_argument, NULL, CD_DPDTIMEOUT + OO + NUMERIC_ARG },
-    { "dpdaction", required_argument, NULL, CD_DPDACTION + OO },
     { "ike", required_argument, NULL, CD_IKE + OO },
     { "pfsgroup", required_argument, NULL, CD_PFSGROUP + OO },
     { "esp", required_argument, NULL, CD_ESP + OO },
@@ -489,6 +484,7 @@ static const struct option long_opts[] = {
     { "debug-klips", no_argument, NULL, DBGOPT_KLIPS + OO },
     { "debug-dns", no_argument, NULL, DBGOPT_DNS + OO },
     { "debug-oppo", no_argument, NULL, DBGOPT_OPPO + OO },
+    { "debug-controlmore", no_argument, NULL, DBGOPT_CONTROLMORE + OO },
     { "debug-private", no_argument, NULL, DBGOPT_PRIVATE + OO },
 
     { "impair-delay-adns-key-answer", no_argument, NULL, DBGOPT_IMPAIR_DELAY_ADNS_KEY_ANSWER + OO },
@@ -565,42 +561,27 @@ clear_end(struct whack_end *e)
     e->id = NULL;
     e->cert = NULL;
     e->ca = NULL;
-    e->sourceip = NULL;
     e->updown = NULL;
     e->host_port = IKE_UDP_PORT;
 }
 
-static void
-update_address_port(ip_address * a, sa_family_t family, u_int16_t port)
-{
-    switch (family) {
-    case AF_INET:
-        a->u.v4.sin_port = port;
-        break;
-    case AF_INET6:
-        a->u.v6.sin6_port = port;
-        break;
-    }
-}
-
 static void
 update_ports(struct whack_message * m)
 {
-    unsigned short port;
+    int port;
 
     if (m->left.port != 0) {
         port = htons(m->left.port);
-        update_address_port(&m->left.host_addr, m->addr_family, port);
-        update_address_port(&m->left.client.addr, m->tunnel_addr_family, port);
+        setportof(port, &m->left.host_addr);
+        setportof(port, &m->left.client.addr);
     }
     if (m->right.port != 0) {
         port = htons(m->right.port);
-        update_address_port(&m->right.host_addr, m->addr_family, port);
-        update_address_port(&m->right.client.addr, m->tunnel_addr_family, port);
+        setportof(port, &m->right.host_addr);
+        setportof(port, &m->right.client.addr);
     }
 }
 
-
 static void
 check_end(struct whack_end *this, struct whack_end *that
 , bool default_nexthop, sa_family_t caf, sa_family_t taf)
@@ -676,18 +657,29 @@ main(int argc, char **argv)
 	*tunnel_af_used_by = NULL;
 
     /* check division of numbering space */
+#ifdef DEBUG
     assert(OPTION_OFFSET + DBGOPT_LAST < NUMERIC_ARG);
+#else
+    assert(OPTION_OFFSET + CD_LAST < NUMERIC_ARG);
+#endif
+    /*
+    assert(OPT_LAST - OPT_FIRST < 32);
+    assert(END_LAST - END_FIRST < 32);
+    assert(CD_LAST - CD_FIRST < 32);
+    */
     assert(OPT_LAST - OPT_FIRST < (sizeof cd_seen * BITS_PER_BYTE));
     assert(END_LAST - END_FIRST < (sizeof cd_seen * BITS_PER_BYTE));
     assert(CD_LAST - CD_FIRST < (sizeof cd_seen * BITS_PER_BYTE));
 #ifdef DEBUG	/* must be last so others are less than 32 to fit in lset_t */
+    /*
+    assert(DBGOPT_LAST - DBGOPT_FIRST < 32);
+    */
     assert(DBGOPT_LAST - DBGOPT_FIRST < (sizeof cd_seen * BITS_PER_BYTE));
 #endif
     /* check that POLICY bit assignment matches with CD_ */
     assert(LELEM(CD_DONT_REKEY - CD_POLICY_FIRST) == POLICY_DONT_REKEY);
 
     zero(&msg);
-    msg.magic = WHACK_MAGIC;
 
     clear_end(&msg.right);	/* left set from this after --to */
 
@@ -842,6 +834,11 @@ main(int argc, char **argv)
 	    msg.keyid = optarg;	/* decoded by Pluto */
 	    continue;
 
+	case OPT_MYID:	/* --myid <identity> */
+	    msg.whack_myid = TRUE;
+	    msg.myid = optarg;	/* decoded by Pluto */
+	    continue;
+
 	case OPT_ADDKEY:	/* --addkey */
 	    msg.whack_addkey = TRUE;
 	    continue;
@@ -936,15 +933,15 @@ main(int argc, char **argv)
 	    continue;
 
 	case OPT_OPPO_HERE:	/* --oppohere <ip-address> */
-	    af_used_by = long_opts[long_index].name;
-	    diagq(ttoaddr(optarg, 0, msg.addr_family, &msg.oppo_my_client), optarg);
+	    tunnel_af_used_by = long_opts[long_index].name;
+	    diagq(ttoaddr(optarg, 0, msg.tunnel_addr_family, &msg.oppo_my_client), optarg);
 	    if (isanyaddr(&msg.oppo_my_client))
 		diagq("0.0.0.0 or 0::0 isn't a valid client address", optarg);
 	    continue;
 
 	case OPT_OPPO_THERE:	/* --oppohere <ip-address> */
-	    af_used_by = long_opts[long_index].name;
-	    diagq(ttoaddr(optarg, 0, msg.addr_family, &msg.oppo_peer_client), optarg);
+	    tunnel_af_used_by = long_opts[long_index].name;
+	    diagq(ttoaddr(optarg, 0, msg.tunnel_addr_family, &msg.oppo_peer_client), optarg);
 	    if (isanyaddr(&msg.oppo_peer_client))
 		diagq("0.0.0.0 or 0::0 isn't a valid client address", optarg);
 	    continue;
@@ -1049,19 +1046,8 @@ main(int argc, char **argv)
 	    if (end_seen & LELEM(END_CLIENTWITHIN - END_FIRST))
 		diag("--client conflicts with --clientwithin");
 	    tunnel_af_used_by = long_opts[long_index].name;
-#ifdef VIRTUAL_IP
-	    if ( ((strlen(optarg)>=6) && (strncmp(optarg,"vhost:",6)==0)) ||
-		((strlen(optarg)>=5) && (strncmp(optarg,"vnet:",5)==0)) ) {
-		msg.right.virt = optarg;
-	    }
-	    else {
-		diagq(ttosubnet(optarg, 0, msg.tunnel_addr_family, &msg.right.client), optarg);
-		msg.right.has_client = TRUE;
-	    }
-#else
 	    diagq(ttosubnet(optarg, 0, msg.tunnel_addr_family, &msg.right.client), optarg);
 	    msg.right.has_client = TRUE;
-#endif
 	    msg.policy |= POLICY_TUNNEL;	/* client => tunnel */
 	    continue;
 
@@ -1084,10 +1070,6 @@ main(int argc, char **argv)
 	    msg.right.key_from_DNS_on_demand = TRUE;
 	    continue;
 
-	case END_SOURCEIP:	/* --sourceip <ip-address> */
-	    msg.right.sourceip = optarg;
-	    continue;
-
 	case END_UPDOWN:	/* --updown <updown> */
 	    msg.right.updown = optarg;
 	    continue;
@@ -1154,24 +1136,6 @@ main(int argc, char **argv)
 	    msg.sa_keying_tries = opt_whole;
 	    continue;
 
-	case CD_DPDDELAY:
-	    msg.dpd_delay = opt_whole;
-	    continue;
-
-	case CD_DPDTIMEOUT:
-	    msg.dpd_timeout = opt_whole;
-	    continue;
-
-	case CD_DPDACTION:
-	    msg.dpd_action = 255;
-	    if( strcmp(optarg, "clear") == 0) {
-		    msg.dpd_action = DPD_ACTION_CLEAR;
-	    }
-	    if( strcmp(optarg, "hold") == 0) {
-		    msg.dpd_action = DPD_ACTION_HOLD;
-	    }
-	    continue;
-
 	case CD_IKE:	/* --ike <ike_alg1,ike_alg2,...> */
 	    msg.ike = optarg;
 	    continue;
@@ -1251,6 +1215,7 @@ main(int argc, char **argv)
 	case DBGOPT_KLIPS:	/* --debug-klips */
 	case DBGOPT_DNS:	/* --debug-dns */
 	case DBGOPT_OPPO:	/* --debug-oppo */
+	case DBGOPT_CONTROLMORE: /* --debug-controlmore */
 	case DBGOPT_PRIVATE:	/* --debug-private */
 	case DBGOPT_IMPAIR_DELAY_ADNS_KEY_ANSWER:	/* --impair-delay-adns-key-answer */
 	case DBGOPT_IMPAIR_DELAY_ADNS_TXT_ANSWER:	/* --impair-delay-adns-txt-answer */
@@ -1288,6 +1253,8 @@ main(int argc, char **argv)
 	/*NOTREACHED*/
     case LELEM(OPT_OPPO_HERE) | LELEM(OPT_OPPO_THERE):
 	msg.whack_oppo_initiate = TRUE;
+	if (LIN(cd_seen, LELEM(CD_TUNNELIPV4 - CD_FIRST) | LELEM(CD_TUNNELIPV6 - CD_FIRST)))
+	    opts_seen &= ~LELEM(OPT_CD);
 	break;
     }
 
@@ -1367,7 +1334,7 @@ main(int argc, char **argv)
 	    diag("--addkey and --pubkeyrsa require --keyid");
     }
 
-    if (!(msg.whack_connection || msg.whack_key
+    if (!(msg.whack_connection || msg.whack_key || msg.whack_myid
     || msg.whack_delete || msg.whack_deletestate
     || msg.whack_initiate || msg.whack_oppo_initiate || msg.whack_terminate
     || msg.whack_route || msg.whack_unroute || msg.whack_listen
@@ -1395,15 +1362,6 @@ main(int argc, char **argv)
     next_str = msg.string;
     str_roof = &msg.string[sizeof(msg.string)];
 
-    if(msg.dpd_delay && !msg.dpd_timeout)
-	    diag("dpddelay specified, but dpdtimeout is zero, both should be specified");
-    if(!msg.dpd_delay && msg.dpd_timeout)
-	    diag("dpdtimeout specified, but dpddelay is zero, both should be specified");
-    if(msg.dpd_action != DPD_ACTION_CLEAR && msg.dpd_action != DPD_ACTION_HOLD) {
-	    diag("dpdaction can only be \"clear\" or \"hold\", defaulting to \"hold\"");
-	    msg.dpd_action = DPD_ACTION_HOLD;
-    }
-
     /* build esp message as esp="<esp>;<pfsgroup>" */
     if (msg.pfsgroup) {
 	    snprintf(esp_buf, sizeof (esp_buf), "%s;%s", 
@@ -1415,22 +1373,15 @@ main(int argc, char **argv)
     || !pack_str(&msg.left.id)		/* string  2 */
     || !pack_str(&msg.left.cert)	/* string  3 */
     || !pack_str(&msg.left.ca)		/* string  4 */
-    || !pack_str(&msg.left.sourceip)	/* string  5 */
-    || !pack_str(&msg.left.updown)      /* string  6 */
-#ifdef VIRTUAL_IP
-    || !pack_str(&msg.left.virt)
-#endif
-    || !pack_str(&msg.right.id)		/* string  7 */
-    || !pack_str(&msg.right.cert)	/* string  8 */
+    || !pack_str(&msg.left.updown)	/* string  5 */
+    || !pack_str(&msg.right.id)		/* string  6 */
+    || !pack_str(&msg.right.cert)	/* string  7 */
     || !pack_str(&msg.right.ca)		/* string  8 */
-    || !pack_str(&msg.right.sourceip)	/* string 10 */
-    || !pack_str(&msg.right.updown)	/* string 11 */
-#ifdef VIRTUAL_IP
-    || !pack_str(&msg.right.virt)
-#endif
-    || !pack_str(&msg.keyid)		/* string 12 */
-    || !pack_str(&msg.ike)		/* string 13 */
-    || !pack_str(&msg.esp)		/* string 14 */
+    || !pack_str(&msg.right.updown)	/* string  9 */
+    || !pack_str(&msg.keyid)		/* string 10 */
+    || !pack_str(&msg.myid)		/* string 11 */
+    || !pack_str(&msg.ike)		/* string 12 */
+    || !pack_str(&msg.esp)		/* string 13 */
     || str_roof - next_str < (ptrdiff_t)msg.keyval.len)    /* chunk (sort of string 5) */
 	diag("too many bytes of strings to fit in message to pluto");
 
@@ -1438,12 +1389,17 @@ main(int argc, char **argv)
     msg.keyval.ptr = NULL;
     next_str += msg.keyval.len;
 
+    msg.magic = ((opts_seen & ~(LELEM(OPT_SHUTDOWN) | LELEM(OPT_STATUS)))
+		| cd_seen) != LEMPTY
+	    || msg.whack_options
+	? WHACK_MAGIC : WHACK_BASIC_MAGIC;
+
     /* send message to Pluto */
     if (access(ctl_addr.sun_path, R_OK | W_OK) < 0)
     {
 	int e = errno;
 
-	switch(e)
+	switch (e)
 	{
 	case EACCES:
 	    fprintf(stderr, "whack: no right to communicate with pluto (access(\"%s\"))\n"
@@ -1552,7 +1508,7 @@ main(int argc, char **argv)
 			    /* be happy */
 			    exit_status = 0;
 			    break;
-			case RC_ENTERPIN:
+			case RC_ENTERSECRET:
 			    get_secret(sock);
 			    break;
 			/* case RC_LOG_SERIOUS: */
