@@ -2,7 +2,7 @@
  * Copyright (C) 2000 Andreas Hess, Patric Lichtsteiner, Roger Wegmann
  * Copyright (C) 2001 Marco Bertossa, Andreas Schleiss
  * Copyright (C) 2002 Mario Strasser
- * Copyright (C) 2000-2002 Andreas Steffen, Zuercher Hochschule Winterthur
+ * Copyright (C) 2000-2003 Andreas Steffen, Zuercher Hochschule Winterthur
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +14,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: x509.c,v 1.4 2003/07/21 02:17:07 ken Exp $
+ * RCSID $Id: x509.c,v 1.1 2003/09/12 19:24:42 ken Exp $
  */
 
 #include <stdlib.h>
@@ -26,6 +26,7 @@
 #include <sys/types.h>
 
 #include <freeswan.h>
+#include <freeswan/ipsec_policy.h>
 
 #include "constants.h"
 #include "defs.h"
@@ -52,17 +53,43 @@ static x509crl_t  *x509crls    = NULL;
 /* ASN.1 definition of a basicConstraints extension */
 
 static const asn1Object_t basicConstraintsObjects[] = {
-  { 0, "basicConstraints",		ASN1_SEQUENCE,     ASN1_NONE  }, /*  0 */
+  { 0, "basicConstraints",		ASN1_SEQUENCE,     ASN1_NONE }, /*  0 */
   { 1,   "CA",				ASN1_BOOLEAN,      ASN1_DEF |
 							   ASN1_BODY }, /*  1 */
   { 1,   "pathLenConstraint",		ASN1_INTEGER,      ASN1_OPT |
 							   ASN1_BODY }, /*  2 */
-  { 1,   "end opt",			ASN1_EOC,          ASN1_END  }, /*  3 */
+  { 1,   "end opt",			ASN1_EOC,          ASN1_END  }  /*  3 */
 };
 
 #define BASIC_CONSTRAINTS_CA	1
 #define BASIC_CONSTRAINTS_ROOF	4
 
+/* ASN.1 definition of a keyIdentifier */
+
+static const asn1Object_t keyIdentifierObjects[] = {
+  { 0,   "keyIdentifier",		ASN1_OCTET_STRING, ASN1_BODY }  /*  0 */
+};
+
+/* ASN.1 definition of a authorityKeyIdentifier extension */
+
+static const asn1Object_t authorityKeyIdentifierObjects[] = {
+  { 0,   "authorityKeyIdentifier",	ASN1_SEQUENCE,     ASN1_NONE }, /*  0 */
+  { 1,     "keyIdentifier",		ASN1_CONTEXT_S_0,  ASN1_OPT |
+							   ASN1_OBJ  }, /*  1 */
+  { 1,     "end opt",			ASN1_EOC,          ASN1_END  }, /*  2 */
+  { 1,     "authorityCertIssuer",	ASN1_CONTEXT_C_1,  ASN1_OPT |
+							   ASN1_OBJ  }, /*  3 */
+  { 1,     "end opt",			ASN1_EOC,          ASN1_END  }, /*  4 */
+  { 1,     "authorityCertSerialNumber",	ASN1_CONTEXT_S_2,  ASN1_OPT |
+							   ASN1_BODY }, /*  5 */
+  { 1,     "end opt",			ASN1_EOC,          ASN1_END  }  /*  6 */
+};
+
+#define AUTH_KEY_ID_KEY_ID		1
+#define AUTH_KEY_ID_CERT_ISSUER		3
+#define AUTH_KEY_ID_CERT_SERIAL		5
+#define AUTH_KEY_ID_ROOF		7
+
 /* ASN.1 definition of generalNames */
 
 static const asn1Object_t generalNamesObjects[] = {
@@ -94,7 +121,7 @@ static const asn1Object_t generalNamesObjects[] = {
   { 1,   "registeredID",		ASN1_CONTEXT_S_8,  ASN1_OPT |
 							   ASN1_BODY }, /* 17 */
   { 1,   "end choice",			ASN1_EOC,          ASN1_END  }, /* 18 */
-  { 0, "end loop",			ASN1_EOC,          ASN1_END  }, /* 19 */
+  { 0, "end loop",			ASN1_EOC,          ASN1_END  }  /* 19 */
 };
 
 #define GN_OBJ_OTHER_NAME	 1
@@ -146,32 +173,42 @@ static const asn1Object_t certObjects[] = {
   { 3,       "sigAlg",			ASN1_OID,          ASN1_BODY }, /*  6 */
   { 2,     "issuer",			ASN1_SEQUENCE,     ASN1_OBJ  }, /*  7 */
   { 2,     "validity",			ASN1_SEQUENCE,     ASN1_NONE }, /*  8 */
-  { 3,       "notBefore",		ASN1_UTCTIME,      ASN1_BODY }, /*  9 */
-  { 3,       "notAfter",		ASN1_UTCTIME,      ASN1_BODY }, /* 10 */
-  { 2,     "subject",			ASN1_SEQUENCE,     ASN1_OBJ  }, /* 11 */
-  { 2,     "subjectPublicKeyInfo",	ASN1_SEQUENCE,     ASN1_NONE }, /* 12 */
-  { 3,       "algorithm",		ASN1_SEQUENCE,     ASN1_NONE }, /* 13 */
-  { 4,          "algorithm",		ASN1_OID,          ASN1_BODY }, /* 14 */
-  { 3,       "subjectPublicKey",	ASN1_BIT_STRING,   ASN1_NONE }, /* 15 */
-  { 4,         "RSAPublicKey",		ASN1_SEQUENCE,     ASN1_NONE }, /* 16 */
-  { 5,           "modulus",		ASN1_INTEGER,      ASN1_BODY }, /* 17 */
-  { 5,           "publicExponent",	ASN1_INTEGER,      ASN1_BODY }, /* 18 */
-  { 2,     "issuerUniqueID",		ASN1_CONTEXT_C_1,  ASN1_OPT  }, /* 19 */
-  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 20 */
-  { 2,     "subjectUniqueID",		ASN1_CONTEXT_C_2,  ASN1_OPT  }, /* 21 */
-  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 22 */
-  { 2,     "optional extensions",	ASN1_CONTEXT_C_3,  ASN1_OPT  }, /* 23 */
-  { 3,       "extensions",		ASN1_SEQUENCE,     ASN1_LOOP }, /* 24 */
-  { 4,         "extension",		ASN1_SEQUENCE,     ASN1_NONE }, /* 25 */
-  { 5,           "extnID",		ASN1_OID,          ASN1_BODY }, /* 26 */
+  { 3,       "notBefore",		ASN1_UTCTIME,      ASN1_OPT |
+							   ASN1_BODY }, /*  9 */
+  { 3,       "end choice",		ASN1_EOC,          ASN1_END  }, /* 10 */
+  { 3,       "notBefore",		ASN1_GENERALIZEDTIME, ASN1_OPT |
+							   ASN1_BODY }, /* 11 */
+  { 3,       "end choice",		ASN1_EOC,          ASN1_END  }, /* 12 */
+  { 3,       "notAfter",		ASN1_UTCTIME,      ASN1_OPT |
+							   ASN1_BODY }, /* 13 */
+  { 3,       "end choice",		ASN1_EOC,          ASN1_END  }, /* 14 */
+  { 3,       "notAfter",		ASN1_GENERALIZEDTIME, ASN1_OPT |
+							   ASN1_BODY }, /* 15 */
+  { 3,       "end choice",		ASN1_EOC,          ASN1_END  }, /* 16 */
+  { 2,     "subject",			ASN1_SEQUENCE,     ASN1_OBJ  }, /* 17 */
+  { 2,     "subjectPublicKeyInfo",	ASN1_SEQUENCE,     ASN1_NONE }, /* 18 */
+  { 3,       "algorithm",		ASN1_SEQUENCE,     ASN1_NONE }, /* 19 */
+  { 4,          "algorithm",		ASN1_OID,          ASN1_BODY }, /* 20 */
+  { 3,       "subjectPublicKey",	ASN1_BIT_STRING,   ASN1_NONE }, /* 21 */
+  { 4,         "RSAPublicKey",		ASN1_SEQUENCE,     ASN1_NONE }, /* 22 */
+  { 5,           "modulus",		ASN1_INTEGER,      ASN1_BODY }, /* 23 */
+  { 5,           "publicExponent",	ASN1_INTEGER,      ASN1_BODY }, /* 24 */
+  { 2,     "issuerUniqueID",		ASN1_CONTEXT_C_1,  ASN1_OPT  }, /* 25 */
+  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 26 */
+  { 2,     "subjectUniqueID",		ASN1_CONTEXT_C_2,  ASN1_OPT  }, /* 27 */
+  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 28 */
+  { 2,     "optional extensions",	ASN1_CONTEXT_C_3,  ASN1_OPT  }, /* 29 */
+  { 3,       "extensions",		ASN1_SEQUENCE,     ASN1_LOOP }, /* 30 */
+  { 4,         "extension",		ASN1_SEQUENCE,     ASN1_NONE }, /* 31 */
+  { 5,           "extnID",		ASN1_OID,          ASN1_BODY }, /* 32 */
   { 5,           "critical",		ASN1_BOOLEAN,      ASN1_DEF |
-							   ASN1_BODY }, /* 27 */
-  { 5,           "extnValue",		ASN1_OCTET_STRING, ASN1_BODY }, /* 28 */
-  { 3,       "end loop",		ASN1_EOC,          ASN1_END  }, /* 29 */
-  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 30 */
-  { 1,   "signatureAlgorithm",		ASN1_SEQUENCE,     ASN1_NONE }, /* 31 */
-  { 2,     "algorithm",			ASN1_OID,          ASN1_BODY }, /* 32 */
-  { 1,   "signature",			ASN1_BIT_STRING,   ASN1_BODY }  /* 33 */
+							   ASN1_BODY }, /* 33 */
+  { 5,           "extnValue",		ASN1_OCTET_STRING, ASN1_BODY }, /* 34 */
+  { 3,       "end loop",		ASN1_EOC,          ASN1_END  }, /* 35 */
+  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 36 */
+  { 1,   "signatureAlgorithm",		ASN1_SEQUENCE,     ASN1_NONE }, /* 37 */
+  { 2,     "algorithm",			ASN1_OID,          ASN1_BODY }, /* 38 */
+  { 1,   "signature",			ASN1_BIT_STRING,   ASN1_BODY }  /* 39 */
 };
 
 #define X509_OBJ_CERTIFICATE			 0
@@ -180,19 +217,21 @@ static const asn1Object_t certObjects[] = {
 #define X509_OBJ_SERIAL_NUMBER			 4
 #define X509_OBJ_SIG_ALG			 6
 #define X509_OBJ_ISSUER 			 7
-#define X509_OBJ_NOT_BEFORE			 9
-#define X509_OBJ_NOT_AFTER			10
-#define X509_OBJ_SUBJECT			11
-#define X509_OBJ_SUBJECT_PUBLIC_KEY_ALGORITHM	14
-#define X509_OBJ_SUBJECT_PUBLIC_KEY		15
-#define X509_OBJ_MODULUS			17
-#define X509_OBJ_PUBLIC_EXPONENT		18
-#define X509_OBJ_EXTN_ID			26
-#define X509_OBJ_CRITICAL			27
-#define X509_OBJ_EXTN_VALUE			28
-#define X509_OBJ_ALGORITHM			32
-#define X509_OBJ_SIGNATURE			33
-#define X509_OBJ_ROOF				34
+#define X509_OBJ_NOT_BEFORE_UTC			 9
+#define X509_OBJ_NOT_BEFORE_GENERALIZED		11
+#define X509_OBJ_NOT_AFTER_UTC			13
+#define X509_OBJ_NOT_AFTER_GENERALIZED		15
+#define X509_OBJ_SUBJECT			17
+#define X509_OBJ_SUBJECT_PUBLIC_KEY_ALGORITHM	20
+#define X509_OBJ_SUBJECT_PUBLIC_KEY		21
+#define X509_OBJ_MODULUS			23
+#define X509_OBJ_PUBLIC_EXPONENT		24
+#define X509_OBJ_EXTN_ID			32
+#define X509_OBJ_CRITICAL			33
+#define X509_OBJ_EXTN_VALUE			34
+#define X509_OBJ_ALGORITHM			38
+#define X509_OBJ_SIGNATURE			39
+#define X509_OBJ_ROOF				40
 
 
 /* ASN.1 definition of an X.509 certificate list */
@@ -206,35 +245,49 @@ static const asn1Object_t crlObjects[] = {
   { 2,     "signature",			ASN1_SEQUENCE,     ASN1_NONE }, /*  4 */
   { 3,       "sigAlg",			ASN1_OID,          ASN1_BODY }, /*  5 */
   { 2,     "issuer",			ASN1_SEQUENCE,     ASN1_OBJ  }, /*  6 */
-  { 2,     "thisUpdate",		ASN1_UTCTIME,      ASN1_BODY }, /*  7 */
+  { 2,     "thisUpdate",		ASN1_UTCTIME,      ASN1_OPT |
+							   ASN1_BODY }, /*  7 */
+  { 2,     "end choice",		ASN1_EOC,          ASN1_END  }, /*  8 */
+  { 2,     "thisUpdate",		ASN1_GENERALIZEDTIME, ASN1_OPT |
+							   ASN1_BODY }, /*  9 */
+  { 2,     "end choice",		ASN1_EOC,          ASN1_END  }, /* 10 */
   { 2,     "nextUpdate",		ASN1_UTCTIME,      ASN1_OPT |
-							   ASN1_BODY }, /*  8 */
-  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /*  9 */
+							   ASN1_BODY }, /* 11 */
+  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 12 */
+  { 2,     "nextUpdate",		ASN1_GENERALIZEDTIME, ASN1_OPT |
+							   ASN1_BODY }, /* 13 */
+  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 14 */
   { 2,     "revokedCertificates",	ASN1_SEQUENCE,     ASN1_OPT |
-							   ASN1_LOOP }, /* 10 */
-  { 3,       "certList",		ASN1_SEQUENCE,     ASN1_NONE }, /* 11 */
-  { 4,         "userCertificate",	ASN1_INTEGER,      ASN1_BODY }, /* 12 */
-  { 4,         "revocationDate",	ASN1_UTCTIME,      ASN1_BODY }, /* 13 */
+							   ASN1_LOOP }, /* 15 */
+  { 3,       "certList",		ASN1_SEQUENCE,     ASN1_NONE }, /* 16 */
+  { 4,         "userCertificate",	ASN1_INTEGER,      ASN1_BODY }, /* 17 */
+  { 4,         "revocationDate",	ASN1_UTCTIME,      ASN1_OPT |
+							   ASN1_BODY }, /* 18 */
+  { 4,         "end choice",		ASN1_EOC,          ASN1_END  }, /* 19 */
+  { 4,         "revocationDate",	ASN1_GENERALIZEDTIME, ASN1_OPT |
+							   ASN1_BODY }, /* 20 */
+  { 4,         "end choice",		ASN1_EOC,          ASN1_END  }, /* 21 */
   { 4,         "crlEntryExtensions",	ASN1_SEQUENCE,     ASN1_OPT |
-							   ASN1_LOOP }, /* 14 */
-  { 5,           "extension",		ASN1_SEQUENCE,     ASN1_NONE }, /* 15 */
-  { 6,             "extnID",		ASN1_OID,          ASN1_BODY }, /* 16 */
+							   ASN1_LOOP }, /* 22 */
+  { 5,           "extension",		ASN1_SEQUENCE,     ASN1_NONE }, /* 23 */
+  { 6,             "extnID",		ASN1_OID,          ASN1_BODY }, /* 24 */
   { 6,             "critical",		ASN1_BOOLEAN,      ASN1_DEF |
-							   ASN1_BODY }, /* 17 */
-  { 6,             "extnValue",		ASN1_OCTET_STRING, ASN1_BODY }, /* 18 */
-  { 4,         "end opt or loop",	ASN1_EOC,          ASN1_END  }, /* 19 */
-  { 2,     "end opt or loop",		ASN1_EOC,          ASN1_END  }, /* 20 */
-  { 2,     "optional extensions",	ASN1_CONTEXT_C_0,  ASN1_OPT  }, /* 21 */
-  { 3,       "crlExtensions",		ASN1_SEQUENCE,     ASN1_LOOP }, /* 22 */
-  { 4,         "extension",		ASN1_SEQUENCE,     ASN1_NONE }, /* 23 */
-  { 5,           "extnID",		ASN1_OID,          ASN1_BODY }, /* 24 */
-  { 5,           "critical",		ASN1_BOOLEAN,      ASN1_DEF |
 							   ASN1_BODY }, /* 25 */
-  { 3,       "end loop",		ASN1_EOC,          ASN1_END  }, /* 26 */
-  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 27 */
-  { 1,   "signatureAlgorithm",		ASN1_SEQUENCE,     ASN1_NONE }, /* 28 */
-  { 2,     "algorithm",			ASN1_OID,          ASN1_BODY }, /* 29 */
-  { 1,   "signature",			ASN1_BIT_STRING,   ASN1_BODY }  /* 30 */
+  { 6,             "extnValue",		ASN1_OCTET_STRING, ASN1_BODY }, /* 26 */
+  { 4,         "end opt or loop",	ASN1_EOC,          ASN1_END  }, /* 27 */
+  { 2,     "end opt or loop",		ASN1_EOC,          ASN1_END  }, /* 28 */
+  { 2,     "optional extensions",	ASN1_CONTEXT_C_0,  ASN1_OPT  }, /* 29 */
+  { 3,       "crlExtensions",		ASN1_SEQUENCE,     ASN1_LOOP }, /* 30 */
+  { 4,         "extension",		ASN1_SEQUENCE,     ASN1_NONE }, /* 31 */
+  { 5,           "extnID",		ASN1_OID,          ASN1_BODY }, /* 32 */
+  { 5,           "critical",		ASN1_BOOLEAN,      ASN1_DEF |
+							   ASN1_BODY }, /* 33 */
+  { 5,           "extnValue",		ASN1_OCTET_STRING, ASN1_BODY }, /* 34 */
+  { 3,       "end loop",		ASN1_EOC,          ASN1_END  }, /* 35 */
+  { 2,     "end opt",			ASN1_EOC,          ASN1_END  }, /* 36 */
+  { 1,   "signatureAlgorithm",		ASN1_SEQUENCE,     ASN1_NONE }, /* 37 */
+  { 2,     "algorithm",			ASN1_OID,          ASN1_BODY }, /* 38 */
+  { 1,   "signature",			ASN1_BIT_STRING,   ASN1_BODY }  /* 39 */
  };
 
 #define CRL_OBJ_CERTIFICATE_LIST		 0
@@ -242,14 +295,20 @@ static const asn1Object_t crlObjects[] = {
 #define CRL_OBJ_VERSION				 2
 #define CRL_OBJ_SIG_ALG				 5
 #define CRL_OBJ_ISSUER				 6
-#define CRL_OBJ_THIS_UPDATE			 7
-#define CRL_OBJ_NEXT_UPDATE			 8
-#define CRL_OBJ_USER_CERTIFICATE		12
-#define CRL_OBJ_REVOCATION_DATE			13
-#define CRL_OBJ_CRITICAL			17
-#define CRL_OBJ_ALGORITHM			29
-#define CRL_OBJ_SIGNATURE			30
-#define CRL_OBJ_ROOF				31
+#define CRL_OBJ_THIS_UPDATE_UTC			 7
+#define CRL_OBJ_THIS_UPDATE_GENERALIZED		 9
+#define CRL_OBJ_NEXT_UPDATE_UTC			11
+#define CRL_OBJ_NEXT_UPDATE_GENERALIZED		13
+#define CRL_OBJ_USER_CERTIFICATE		17
+#define CRL_OBJ_REVOCATION_DATE_UTC		18
+#define CRL_OBJ_REVOCATION_DATE_GENERALIZED	20
+#define CRL_OBJ_CRL_ENTRY_CRITICAL		25
+#define CRL_OBJ_EXTN_ID				32
+#define CRL_OBJ_CRITICAL			33
+#define CRL_OBJ_EXTN_VALUE			34
+#define CRL_OBJ_ALGORITHM			38
+#define CRL_OBJ_SIGNATURE			39
+#define CRL_OBJ_ROOF				40
 
 
 const x509cert_t empty_x509cert = {
@@ -273,14 +332,17 @@ const x509cert_t empty_x509cert = {
                  /*     subjectPublicKey */
     { NULL, 0 }, /*       modulus */
     { NULL, 0 }, /*       publicExponent */
-    { NULL, 0 }, /*     issuerUniqueID */
-    { NULL, 0 }, /*     subjectUniqueID */
+                 /*     issuerUniqueID */
+                 /*     subjectUniqueID */
                  /*     extensions */
                  /*       extension */
                  /*         extnID */
                  /*         critical */
                  /*         extnValue */
       FALSE    , /*           isCA */
+    { NULL, 0 }, /*           subjectKeyID */
+    { NULL, 0 }, /*           authKeyID */
+    { NULL, 0 }, /*           authKeySerialNumber */
       NULL     , /*           subjectAltName */
       NULL     , /*           crlDistributionPoints */
                  /*   signatureAlgorithm */
@@ -300,7 +362,13 @@ const x509crl_t empty_x509crl = {
             0  , /*     thisUpdate */
             0  , /*     nextUpdate */
       NULL     , /*     revokedCertificates */
-		 /*     crlExtensions*/
+                 /*     crlExtensions */
+                 /*       extension */
+                 /*         extnID */
+                 /*         critical */
+                 /*         extnValue */
+    { NULL, 0 }, /*           authKeyID */
+    { NULL, 0 }, /*           authKeySerialNumber */
    		 /*   signatureAlgorithm*/
     { NULL, 0 }, /*     algorithm*/
     { NULL, 0 }  /*   signature*/
@@ -315,35 +383,36 @@ typedef struct {
     u_char type;
 } x501rdn_t;
 
-  /* X.501 acronyms for well known object identifiers (OIDs) */
 
+/* X.501 acronyms for well known object identifiers (OIDs) */
+
+static u_char oid_ND[]  = {0x02, 0x82, 0x06, 0x01,
+			   0x0A, 0x07, 0x14};
 static u_char oid_UID[] = {0x09, 0x92, 0x26, 0x89, 0x93,
-                         0xF2, 0x2C, 0x64, 0x01, 0x01};
+			   0xF2, 0x2C, 0x64, 0x01, 0x01};
 static u_char oid_DC[]  = {0x09, 0x92, 0x26, 0x89, 0x93,
-                         0xF2, 0x2C, 0x64, 0x01, 0x19};
-static u_char oid_CN[] = {0x55, 0x04, 0x03};
-static u_char oid_S[]  = {0x55, 0x04, 0x04};
-static u_char oid_SN[] = {0x55, 0x04, 0x05};
-static u_char oid_C[]  = {0x55, 0x04, 0x06};
-static u_char oid_L[]  = {0x55, 0x04, 0x07};
-static u_char oid_ST[] = {0x55, 0x04, 0x08};
-static u_char oid_O[]  = {0x55, 0x04, 0x0A};
-static u_char oid_OU[] = {0x55, 0x04, 0x0B};
-static u_char oid_T[]  = {0x55, 0x04, 0x0C};
-static u_char oid_D[]  = {0x55, 0x04, 0x0D};
-static u_char oid_N[]  = {0x55, 0x04, 0x29};
-static u_char oid_G[]  = {0x55, 0x04, 0x2A};
-static u_char oid_I[]  = {0x55, 0x04, 0x2B};
-static u_char oid_ID[] = {0x55, 0x04, 0x2D};
-static u_char oid_E[]  = {0x2A, 0x86, 0x48, 0x86, 0xF7,
-                        0x0D, 0x01, 0x09, 0x01};
+			   0xF2, 0x2C, 0x64, 0x01, 0x19};
+static u_char oid_CN[]  = {0x55, 0x04, 0x03};
+static u_char oid_S[]   = {0x55, 0x04, 0x04};
+static u_char oid_SN[]  = {0x55, 0x04, 0x05};
+static u_char oid_C[]   = {0x55, 0x04, 0x06};
+static u_char oid_L[]   = {0x55, 0x04, 0x07};
+static u_char oid_ST[]  = {0x55, 0x04, 0x08};
+static u_char oid_O[]   = {0x55, 0x04, 0x0A};
+static u_char oid_OU[]  = {0x55, 0x04, 0x0B};
+static u_char oid_T[]   = {0x55, 0x04, 0x0C};
+static u_char oid_D[]   = {0x55, 0x04, 0x0D};
+static u_char oid_N[]   = {0x55, 0x04, 0x29};
+static u_char oid_G[]   = {0x55, 0x04, 0x2A};
+static u_char oid_I[]   = {0x55, 0x04, 0x2B};
+static u_char oid_ID[]  = {0x55, 0x04, 0x2D};
+static u_char oid_E[]   = {0x2A, 0x86, 0x48, 0x86, 0xF7,
+			   0x0D, 0x01, 0x09, 0x01};
 static u_char oid_TCGID[] = {0x2B, 0x06, 0x01, 0x04, 0x01, 0x89,
-                           0x31, 0x01, 0x01, 0x02, 0x02, 0x4B};
-
-
-
+			     0x31, 0x01, 0x01, 0x02, 0x02, 0x4B};
 
 static const x501rdn_t x501rdns[] = {
+  {"ND"           , {oid_ND,     7}, ASN1_PRINTABLESTRING},
   {"UID"          , {oid_UID,   10}, ASN1_PRINTABLESTRING},
   {"DC"           , {oid_DC,    10}, ASN1_PRINTABLESTRING},
   {"CN"           , {oid_CN,     3}, ASN1_PRINTABLESTRING},
@@ -367,7 +436,7 @@ static const x501rdn_t x501rdns[] = {
   {"TCGID"        , {oid_TCGID, 12}, ASN1_PRINTABLESTRING}
 };
 
-#define X501_RDN_ROOF   21
+#define X501_RDN_ROOF   22
 
 /* Maximum length of ASN.1 distinquished name */
 
@@ -409,9 +478,10 @@ update_chunk(chunk_t *ch, int n)
  *  Pointer is set to the first RDN in a DN
  */
 static err_t
-init_rdn(chunk_t dn, chunk_t *rdn)
+init_rdn(chunk_t dn, chunk_t *rdn, chunk_t *attribute, bool *next)
 {
     *rdn = empty_chunk;
+    *attribute = empty_chunk;
 
     /* a DN is a SEQUENCE OF RDNs */
 
@@ -423,6 +493,9 @@ init_rdn(chunk_t dn, chunk_t *rdn)
     rdn->len = asn1_length(&dn);
     rdn->ptr = dn.ptr;
 
+    /* are there any RDNs ? */
+    *next = rdn->len > 0;
+
     return NULL;
 }
 
@@ -430,50 +503,63 @@ init_rdn(chunk_t dn, chunk_t *rdn)
  *  Fetches the next RDN in a DN
  */
 static err_t
-get_next_rdn(chunk_t *rdn, chunk_t *oid, chunk_t *value, asn1_t *type)
+get_next_rdn(chunk_t *rdn, chunk_t * attribute, chunk_t *oid, chunk_t *value
+, asn1_t *type, bool *next)
 {
-    chunk_t attribute;
+    chunk_t body;
 
     /* initialize return values */
     *oid   = empty_chunk;
     *value = empty_chunk;
 
-    /* an RDN is a SET OF attributeTypeAndValue */
-    if (*rdn->ptr != ASN1_SET)
-	return "RDN is not a SET";
+    /* if all attributes have been parsed, get next rdn */
+    if (attribute->len <= 0)
+    {
+	/* an RDN is a SET OF attributeTypeAndValue */
+	if (*rdn->ptr != ASN1_SET)
+	    return "RDN is not a SET";
 
-    attribute.len = asn1_length(rdn);
-    attribute.ptr = rdn->ptr;
+	attribute->len = asn1_length(rdn);
+	attribute->ptr = rdn->ptr;
 
-    /* advance to start of next RDN */
-    rdn->ptr += attribute.len;
-    rdn->len -= attribute.len;
+	/* advance to start of next RDN */
+	rdn->ptr += attribute->len;
+	rdn->len -= attribute->len;
+    }
 
     /* an attributeTypeAndValue is a SEQUENCE */
-    if (*attribute.ptr != ASN1_SEQUENCE)
+    if (*attribute->ptr != ASN1_SEQUENCE)
  	return "attributeTypeAndValue is not a SEQUENCE";
 
     /* extract the attribute body */
-    attribute.len = asn1_length(&attribute);
+    body.len = asn1_length(attribute);
+    body.ptr = attribute->ptr;
+    
+    /* advance to start of next attribute */
+    attribute->ptr += body.len;
+    attribute->len -= body.len;
 
     /* attribute type is an OID */
-    if (*attribute.ptr != ASN1_OID)
+    if (*body.ptr != ASN1_OID)
 	return "attributeType is not an OID";
 
     /* extract OID */
-    oid->len = asn1_length(&attribute);
-    oid->ptr = attribute.ptr;
+    oid->len = asn1_length(&body);
+    oid->ptr = body.ptr;
 
     /* advance to the attribute value */
-    attribute.ptr += oid->len;
-    attribute.len -= oid->len;
+    body.ptr += oid->len;
+    body.len -= oid->len;
 
     /* extract string type */
-    *type = *attribute.ptr;
+    *type = *body.ptr;
 
     /* extract string value */
-    value->len = asn1_length(&attribute);
-    value->ptr = attribute.ptr;
+    value->len = asn1_length(&body);
+    value->ptr = body.ptr;
+
+    /* are there any RDNs left? */
+    *next = rdn->len > 0 || attribute->len > 0;
 
     return NULL;
 }
@@ -484,19 +570,20 @@ get_next_rdn(chunk_t *rdn, chunk_t *oid, chunk_t *value, asn1_t *type)
 static err_t
 dn_parse(chunk_t dn, chunk_t *str)
 {
-    chunk_t rdn, oid, value;
+    chunk_t rdn, oid, attribute, value;
     asn1_t type;
     int oid_code;
-    int first = TRUE;
+    bool next;
+    bool first = TRUE;
 
-    err_t ugh = init_rdn(dn, &rdn);
+    err_t ugh = init_rdn(dn, &rdn, &attribute, &next);
 
     if (ugh != NULL) /* a parsing error has occured */
         return ugh;
 
-    while (rdn.len > 0)
+    while (next)
     {
-	ugh = get_next_rdn(&rdn, &oid, &value, &type);
+	ugh = get_next_rdn(&rdn, &attribute, &oid, &value, &type, &next);
 
 	if (ugh != NULL) /* a parsing error has occured */
 	    return ugh;
@@ -527,18 +614,19 @@ dn_parse(chunk_t dn, chunk_t *str)
 int
 dn_count_wildcards(chunk_t dn)
 {
-    chunk_t rdn, oid, value;
+    chunk_t rdn, attribute, oid, value;
     asn1_t type;
+    bool next;
     int wildcards = 0;
 
-    err_t ugh = init_rdn(dn, &rdn);
+    err_t ugh = init_rdn(dn, &rdn, &attribute, &next);
 
     if (ugh != NULL) /* a parsing error has occured */
         return -1;
 
-    while (rdn.len > 0)
+    while (next)
     {
-	ugh = get_next_rdn(&rdn, &oid, &value, &type);
+	ugh = get_next_rdn(&rdn, &attribute, &oid, &value, &type, &next);
 	
 	if (ugh != NULL) /* a parsing error has occured */
 	    return -1;
@@ -750,25 +838,26 @@ atodn(char *src, chunk_t *dn)
 bool
 same_dn(chunk_t a, chunk_t b)
 {
-    chunk_t rdn_a,   rdn_b;
-    chunk_t oid_a,   oid_b;
-    chunk_t value_a, value_b;
-    asn1_t  type_a,  type_b;
+    chunk_t rdn_a, rdn_b, attribute_a, attribute_b;
+    chunk_t oid_a, oid_b, value_a, value_b;
+    asn1_t type_a, type_b;
+    bool next_a, next_b;
 
     /* same lengths for the DNs */
     if (a.len != b.len)
 	return FALSE;
 
     /* initialize DN parsing */
-    if (init_rdn(a, &rdn_a) != NULL || init_rdn(b, &rdn_b) != NULL)
-    	return FALSE;
+    if (init_rdn(a, &rdn_a, &attribute_a, &next_a) != NULL
+    ||  init_rdn(b, &rdn_b, &attribute_b, &next_b) != NULL)
+	return FALSE;
 
     /* fetch next RDN pair */
-    while (rdn_a.len > 0 && rdn_b.len > 0)
+    while (next_a && next_b)
     {
 	/* parse next RDNs and check for errors */
-	if (get_next_rdn(&rdn_a, &oid_a, &value_a, &type_a) != NULL ||
-	    get_next_rdn(&rdn_b, &oid_b, &value_b, &type_b) != NULL)
+	if (get_next_rdn(&rdn_a, &attribute_a, &oid_a, &value_a, &type_a, &next_a) != NULL
+	||  get_next_rdn(&rdn_b, &attribute_b, &oid_b, &value_b, &type_b, &next_b) != NULL)
 	{
 	    return FALSE;
 	}
@@ -795,7 +884,7 @@ same_dn(chunk_t a, chunk_t b)
 	}
     }
     /* both DNs must have same number of RDNs */
-    if (rdn_a.len != 0 || rdn_b.len != 0)
+    if (next_a || next_b)
 	return FALSE;
 
     /* the two DNs are equal! */
@@ -809,24 +898,25 @@ same_dn(chunk_t a, chunk_t b)
 bool
 match_dn(chunk_t a, chunk_t b, int *wildcards)
 {
-    chunk_t rdn_a,   rdn_b;
-    chunk_t oid_a,   oid_b;
-    chunk_t value_a, value_b;
-    asn1_t  type_a,  type_b;
+    chunk_t rdn_a, rdn_b, attribute_a, attribute_b;
+    chunk_t oid_a, oid_b, value_a, value_b;
+    asn1_t type_a,  type_b;
+    bool next_a, next_b;
 
     /* initialize wildcard counter */
     *wildcards = 0;
 
     /* initialize DN parsing */
-    if (init_rdn(a, &rdn_a) != NULL || init_rdn(b, &rdn_b) != NULL)
+    if (init_rdn(a, &rdn_a, &attribute_a, &next_a) != NULL
+    ||  init_rdn(b, &rdn_b, &attribute_b, &next_b) != NULL)
     	return FALSE;
 
     /* fetch next RDN pair */
-    while (rdn_a.len > 0 && rdn_b.len > 0)
+    while (next_a && next_b)
     {
 	/* parse next RDNs and check for errors */
-	if (get_next_rdn(&rdn_a, &oid_a, &value_a, &type_a) != NULL ||
-	    get_next_rdn(&rdn_b, &oid_b, &value_b, &type_b) != NULL)
+	if (get_next_rdn(&rdn_a, &attribute_a, &oid_a, &value_a, &type_a, &next_a) != NULL
+	||  get_next_rdn(&rdn_b, &attribute_b, &oid_b, &value_b, &type_b, &next_b) != NULL)
 	{
 	    return FALSE;
 	}
@@ -860,7 +950,7 @@ match_dn(chunk_t a, chunk_t b, int *wildcards)
 	}
     }
     /* both DNs must have same number of RDNs */
-    if (rdn_a.len != 0 || rdn_b.len != 0)
+    if (next_a || next_b)
 	return FALSE;
 
     /* the two DNs match! */
@@ -953,18 +1043,50 @@ select_x509cert_id(x509cert_t *cert, struct id *end_id)
     }
 }
 
+/*
+ * check for equality between two key identifiers
+ */
+static bool
+same_keyid(chunk_t a, chunk_t b)
+{
+    if (a.ptr == NULL || b.ptr == NULL)
+	return FALSE;
+
+    /* both length and content must be equal */
+    if (a.len != b.len)
+ 	return FALSE;
+    return memcmp(a.ptr, b.ptr, a.len) == 0;
+}
+
+/*
+ * check for equality between two serial numbers
+ */
+static bool
+same_serial(chunk_t a, chunk_t b)
+{
+    /* do not compare serial numbers if one of them is not defined */
+    if (a.ptr == NULL || b.ptr == NULL)
+	return TRUE;
+
+    /* both length and content must be equal */
+    if (a.len != b.len)
+ 	return FALSE;
+    return memcmp(a.ptr, b.ptr, a.len) == 0;
+}
+
 /*
  *  get the X.509 CA certificate with a given subject
  */
 static x509cert_t*
-get_x509cacert(chunk_t subject)
+get_x509cacert(chunk_t subject, chunk_t serial, chunk_t keyid)
 {
     x509cert_t *cert = x509cacerts;
     x509cert_t *prev_cert = NULL;
 
     while(cert != NULL)
    {
-	if (same_dn(cert->subject, subject))
+	if ((keyid.ptr != NULL) ? same_keyid(keyid, cert->subjectKeyID)
+	: (same_dn(cert->subject, subject) && same_serial(serial, cert->serialNumber)))
 	{
 	    if (cert != x509cacerts)
 	    {
@@ -981,8 +1103,6 @@ get_x509cacert(chunk_t subject)
     return NULL;
 }
 
-#define MAX_CA_PATH_LEN		10
-
 /*
  * Checks if CA a is trusted by CA b
  */
@@ -1011,7 +1131,7 @@ trusted_ca(chunk_t a, chunk_t b, int *pathlen)
 
     while ((*pathlen)++ < MAX_CA_PATH_LEN)
     {
-	x509cert_t *cacert = get_x509cacert(a);
+	x509cert_t *cacert = get_x509cacert(a, empty_chunk, empty_chunk);
 
 	/* cacert not found or self-signed root cacert-> exit */
 	if (cacert == NULL || same_dn(cacert->issuer, a))
@@ -1032,18 +1152,49 @@ trusted_ca(chunk_t a, chunk_t b, int *pathlen)
     return match;
 }
 
+/* 
+ * does our CA match one of the requested CAs?
+ */
+bool
+match_requested_ca(generalName_t *requested_ca, chunk_t our_ca, int *our_pathlen)
+{
+    /* if no ca is requested than any ca will match */
+    if (requested_ca == NULL)
+    {
+	*our_pathlen = 0;
+	return TRUE;
+    }
+
+    *our_pathlen = MAX_CA_PATH_LEN + 1;
+
+    while (requested_ca != NULL)
+    {
+	int pathlen;
+
+	if (trusted_ca(our_ca, requested_ca->name, &pathlen)
+	&& pathlen < *our_pathlen)
+	    *our_pathlen = pathlen;
+	requested_ca = requested_ca->next;
+    }
+
+    return *our_pathlen <= MAX_CA_PATH_LEN;
+}
+
+
 /*
  *  get the X.509 CRL with a given issuer
  */
 static x509crl_t*
-get_x509crl(chunk_t issuer)
+get_x509crl(chunk_t issuer, chunk_t serial, chunk_t keyid)
 {
     x509crl_t *crl = x509crls;
     x509crl_t *prev_crl = NULL;
 
     while(crl != NULL)
     {
-	if (same_dn(crl->issuer, issuer))
+	if ((keyid.ptr != NULL && crl->authKeyID.ptr != NULL)
+	? same_keyid(keyid, crl->authKeyID)
+	: (same_dn(crl->issuer, issuer) && same_serial(serial, crl->authKeySerialNumber)))
 	{
 	    if (crl != x509crls)
 	    {
@@ -1209,7 +1360,8 @@ store_x509certs(x509cert_t **firstcert, bool strict)
 	    {
 		lock_cacert_list("store_x509certs");
 
-		if (get_x509cacert(cert->subject))
+		if (get_x509cacert(cert->subject, cert->serialNumber
+		,cert->subjectKeyID))
 		{
 		    free_first_cacert();
 		    DBG(DBG_PARSING,
@@ -1282,7 +1434,7 @@ load_cacerts(void)
 	n = scandir(CA_CERT_PATH, &filelist, file_select, alphasort);
 
 	if (n <= 0)
-	   plog("  Warning: empty directory");
+	    plog("  Warning: empty directory");
 	else
 	{
 	    while (n--)
@@ -1295,7 +1447,8 @@ load_cacerts(void)
 
 		    lock_cacert_list("load_cacerts");
 
-		    if (get_x509cacert(cacert->subject))
+		    if (get_x509cacert(cacert->subject, cacert->serialNumber
+		    , cacert->subjectKeyID))
 		    {
 			free_first_cacert();
 			DBG(DBG_PARSING,
@@ -1494,7 +1647,8 @@ insert_crl(chunk_t blob, chunk_t crl_uri)
 
 	lock_cacert_list("insert_crl");
 	/* get the issuer cacert */
-	issuer_cert = get_x509cacert(crl->issuer);
+	issuer_cert = get_x509cacert(crl->issuer, crl->authKeySerialNumber,
+	    crl->authKeyID);
 	if (issuer_cert == NULL)
 	{
 	    plog("crl issuer cacert not found");
@@ -1521,7 +1675,8 @@ insert_crl(chunk_t blob, chunk_t crl_uri)
 	)
 
 	lock_crl_list("insert_crl");
-	oldcrl = get_x509crl(crl->issuer);
+	oldcrl = get_x509crl(crl->issuer, crl->authKeySerialNumber
+	    , crl->authKeyID);
 
 	if (oldcrl != NULL)
 	{
@@ -1597,7 +1752,7 @@ load_crls(void)
 		chunk_t blob = empty_chunk;
 		char *filename = filelist[n]->d_name;
 
-		if (load_coded_file(filename, "", "crl", &blob, &pgp))
+		if (load_coded_file(filename, NULL, "crl", &blob, &pgp))
 		{
 		    chunk_t crl_uri;
 		    crl_uri.len = 7 + sizeof(CRL_PATH) + strlen(filename);
@@ -1694,7 +1849,8 @@ parse_generalNames(chunk_t blob, int level0, bool implicit)
 
     asn1_init(&ctx, blob, level0, implicit, DBG_RAW);
 
-    while (objectID < GN_OBJ_ROOF) {
+    while (objectID < GN_OBJ_ROOF)
+    {
 	bool valid_gn = FALSE;
 
 	if (!extract_object(generalNamesObjects, &objectID, &object, &ctx))
@@ -1760,6 +1916,65 @@ chunk_t get_directoryName(chunk_t blob, int level, bool implicit)
     return name;
 }
 
+/*
+ * extracts a keyIdentifier
+ */
+static chunk_t
+parse_keyIdentifier(chunk_t blob, int level0, bool implicit)
+{
+    asn1_ctx_t ctx;
+    chunk_t object;
+    int objectID = 0;
+
+    asn1_init(&ctx, blob, level0, implicit, DBG_RAW);
+
+    extract_object(keyIdentifierObjects, &objectID, &object, &ctx);
+    return object;
+}
+
+/*
+ * extracts an authoritykeyIdentifier
+ */
+void
+parse_authorityKeyIdentifier(chunk_t blob, int level0
+    , chunk_t *authKeyID, chunk_t *authKeySerialNumber)
+{
+    asn1_ctx_t ctx;
+    chunk_t object;
+    int objectID = 0;
+
+    asn1_init(&ctx, blob, level0, FALSE, DBG_RAW);
+
+    while (objectID < AUTH_KEY_ID_ROOF)
+    {
+	if (!extract_object(authorityKeyIdentifierObjects, &objectID, &object, &ctx))
+	     return;
+
+	switch (objectID) {
+	case AUTH_KEY_ID_KEY_ID:
+	    {
+		u_int level = level0 + authorityKeyIdentifierObjects[objectID].level + 1;
+
+	        *authKeyID = parse_keyIdentifier(object, level, TRUE);
+	    }
+	    break;
+	case AUTH_KEY_ID_CERT_ISSUER:
+	    {
+		u_int level = level0 + authorityKeyIdentifierObjects[objectID].level + 1;
+		generalName_t * gn = parse_generalNames(object, level, TRUE);
+
+		free_generalNames(gn, FALSE);
+	    }
+	    break;
+	case AUTH_KEY_ID_CERT_SERIAL:
+	    *authKeySerialNumber = object;
+	    break;
+	default:
+	}
+	objectID++;
+    }
+}
+
 /*  extracts one or several crlDistributionPoints and puts them into
  *  a chained list
  */
@@ -1775,8 +1990,8 @@ parse_crlDistributionPoints(chunk_t blob, int level0)
 
     asn1_init(&ctx, blob, level0, FALSE, DBG_RAW);
 
-    while (objectID < CRL_DIST_POINTS_ROOF) {
-
+    while (objectID < CRL_DIST_POINTS_ROOF)
+    {
 	if (!extract_object(crlDistributionPointsObjects, &objectID,
 			    &object, &ctx))
 	     return NULL;
@@ -1814,8 +2029,8 @@ parse_x509cert(chunk_t blob, u_int level0, x509cert_t *cert)
 
     asn1_init(&ctx, blob, level0, FALSE, DBG_RAW);
 
-    while (objectID < X509_OBJ_ROOF) {
-
+    while (objectID < X509_OBJ_ROOF)
+    {
 	if (!extract_object(certObjects, &objectID, &object, &ctx))
 	     return FALSE;
 
@@ -1845,12 +2060,18 @@ parse_x509cert(chunk_t blob, u_int level0, x509cert_t *cert)
 		DBG_log("  '%s'",buf);
 	    )
 	    break;
-	case X509_OBJ_NOT_BEFORE:
+	case X509_OBJ_NOT_BEFORE_UTC:
 	    cert->notBefore = asn1totime(&object, ASN1_UTCTIME);
 	    break;
-	case X509_OBJ_NOT_AFTER:
+	case X509_OBJ_NOT_BEFORE_GENERALIZED:
+	    cert->notBefore = asn1totime(&object, ASN1_GENERALIZEDTIME);
+	    break;
+	case X509_OBJ_NOT_AFTER_UTC:
 	    cert->notAfter = asn1totime(&object, ASN1_UTCTIME);
 	    break;
+	case X509_OBJ_NOT_AFTER_GENERALIZED:
+	    cert->notAfter = asn1totime(&object, ASN1_GENERALIZEDTIME);
+	    break;
 	case X509_OBJ_SUBJECT:
 	    cert->subject = object;
 	    dntoa(buf, BUF_LEN, object);
@@ -1890,15 +2111,28 @@ parse_x509cert(chunk_t blob, u_int level0, x509cert_t *cert)
 		u_int extn_oid = known_oid(extnID);
 		u_int level = level0 + certObjects[objectID].level + 1;
 
-		if (extn_oid == OID_BASIC_CONSTRAINTS)
-		    cert->isCA =
-			parse_basicConstraints(object, level);
-		else if (extn_oid == OID_SUBJECT_ALT_NAME)
+		switch (extn_oid) {
+		case OID_SUBJECT_KEY_ID:
+		    cert->subjectKeyID =
+			parse_keyIdentifier(object, level, FALSE);
+		    break;
+		case OID_SUBJECT_ALT_NAME:
 		    cert->subjectAltName =
 			parse_generalNames(object, level, FALSE);
-		else if (extn_oid == OID_CRL_DISTRIBUTION_POINTS)
+		    break;
+		case OID_BASIC_CONSTRAINTS:
+		    cert->isCA =
+			parse_basicConstraints(object, level);
+		    break;
+		case OID_CRL_DISTRIBUTION_POINTS:
 		    cert->crlDistributionPoints =
 			parse_crlDistributionPoints(object, level);
+		    break;
+		 case OID_AUTHORITY_KEY_ID:
+		    parse_authorityKeyIdentifier(object, level
+			, &cert->authKeyID, &cert->authKeySerialNumber);
+		    break;
+		}
 	    }
 	    break;
 	case X509_OBJ_ALGORITHM:
@@ -1926,14 +2160,15 @@ parse_x509crl(chunk_t blob, u_int level0, x509crl_t *crl)
     u_char buf[BUF_LEN];
     asn1_ctx_t ctx;
     bool critical;
+    chunk_t extnID;
     chunk_t userCertificate;
     chunk_t object;
     int objectID = 0;
 
     asn1_init(&ctx, blob, level0, FALSE, DBG_RAW);
- 
-    while (objectID < CRL_OBJ_ROOF) {
 
+    while (objectID < CRL_OBJ_ROOF)
+    {
 	if (!extract_object(crlObjects, &objectID, &object, &ctx))
 	     return FALSE;
 
@@ -1960,33 +2195,58 @@ parse_x509crl(chunk_t blob, u_int level0, x509crl_t *crl)
 		DBG_log("  '%s'",buf);
 	    )
 	    break;
-	case CRL_OBJ_THIS_UPDATE:
+	case CRL_OBJ_THIS_UPDATE_UTC:
 	    crl->thisUpdate = asn1totime(&object, ASN1_UTCTIME);
 	    break;
-	case CRL_OBJ_NEXT_UPDATE:
+	case CRL_OBJ_THIS_UPDATE_GENERALIZED:
+	    crl->thisUpdate = asn1totime(&object, ASN1_GENERALIZEDTIME);
+	    break;
+	case CRL_OBJ_NEXT_UPDATE_UTC:
 	    crl->nextUpdate = asn1totime(&object, ASN1_UTCTIME);
 	    break;
+	case CRL_OBJ_NEXT_UPDATE_GENERALIZED:
+	    crl->nextUpdate = asn1totime(&object, ASN1_GENERALIZEDTIME);
+	    break;
 	case CRL_OBJ_USER_CERTIFICATE:
 	    userCertificate = object;
 	    break;
-	case CRL_OBJ_REVOCATION_DATE:
+	case CRL_OBJ_REVOCATION_DATE_UTC:
+	case CRL_OBJ_REVOCATION_DATE_GENERALIZED:
 	    {
 		/* put all the serial numbers and the revocation date in a chained list
 		   with revocedCertificates pointing to the first revoked certificate */
 
 		revokedCert_t *revokedCert = alloc_thing(revokedCert_t, "revokedCert");
 		revokedCert->userCertificate = userCertificate;
-		revokedCert->revocationDate = asn1totime(&object, ASN1_UTCTIME);
+		revokedCert->revocationDate = asn1totime(&object
+		    , (objectID == CRL_OBJ_REVOCATION_DATE_UTC)? ASN1_UTCTIME
+							       : ASN1_GENERALIZEDTIME);
 		revokedCert->next = crl->revokedCertificates;
 		crl->revokedCertificates = revokedCert;
 	    }
 	    break;
+	case CRL_OBJ_EXTN_ID:
+	    extnID = object;
+	    break;
+	case CRL_OBJ_CRL_ENTRY_CRITICAL:
 	case CRL_OBJ_CRITICAL:
 	    critical = object.len && *object.ptr;
 	    DBG(DBG_PARSING,
 		DBG_log("  %s",(critical)?"TRUE":"FALSE");
 	    )
 	    break;
+	case CRL_OBJ_EXTN_VALUE:
+	    {
+		u_int extn_oid = known_oid(extnID);
+		u_int level = level0 + crlObjects[objectID].level + 1;
+
+		if (extn_oid == OID_AUTHORITY_KEY_ID)
+		{
+		    parse_authorityKeyIdentifier(object, level
+			, &crl->authKeyID, &crl->authKeySerialNumber);
+		}
+	    }
+	    break;
 	case CRL_OBJ_ALGORITHM:
 	    crl->algorithm = object;
 	    break;
@@ -2124,7 +2384,8 @@ verify_x509cert(const x509cert_t *cert, bool strict, time_t *until)
 	)
 
 	lock_cacert_list("verify_x509cert");
-	issuer_cert = get_x509cacert(cert->issuer);
+	issuer_cert = get_x509cacert(cert->issuer, cert->authKeySerialNumber
+	    , cert->authKeyID);
 	unlock_cacert_list("verify_x509cert");
 
 	if (issuer_cert == NULL)
@@ -2147,7 +2408,8 @@ verify_x509cert(const x509cert_t *cert, bool strict, time_t *until)
 	)
 
 	lock_crl_list("verify_x509cert");
-	crl = get_x509crl(cert->issuer);
+	crl = get_x509crl(cert->issuer, cert->authKeySerialNumber
+	    , cert->authKeyID);
 
 	if (crl == NULL)
 	{
@@ -2271,6 +2533,9 @@ list_x509cert_chain(const char * caption, x509cert_t* cert, bool utc)
 	whack_log(RC_COMMENT, "       subject: '%s'", buf);
 	dntoa(buf, BUF_LEN, cert->issuer);
 	whack_log(RC_COMMENT, "       issuer:  '%s'", buf);
+	datatot(cert->serialNumber.ptr, cert->serialNumber.len, ':'
+	    , buf, BUF_LEN);
+	whack_log(RC_COMMENT, "       serial:   %s", buf);
 	form_keyid(cert->publicExponent, cert->modulus, keyid, &keysize);
 	whack_log(RC_COMMENT, "       pubkey:   %4d RSA Key %s%s", 8*keysize, keyid,
 		cert->smartcard ? ", on smartcard" :
@@ -2281,6 +2546,24 @@ list_x509cert_chain(const char * caption, x509cert_t* cert, bool utc)
 	whack_log(RC_COMMENT, "                 not after  %s %s",
 		timetoa(&cert->notAfter, utc),
 		check_expiry(cert->notAfter, CA_CERT_WARNING_INTERVAL, TRUE));
+	if (cert->subjectKeyID.ptr != NULL)
+	{
+	    datatot(cert->subjectKeyID.ptr, cert->subjectKeyID.len, ':'
+	        , buf, BUF_LEN);
+	    whack_log(RC_COMMENT, "       subjkey:  %s", buf);
+	}
+	if (cert->authKeyID.ptr != NULL)
+	{
+	    datatot(cert->authKeyID.ptr, cert->authKeyID.len, ':'
+		, buf, BUF_LEN);
+	    whack_log(RC_COMMENT, "       authkey:  %s", buf);
+	}
+	if (cert->authKeySerialNumber.ptr != NULL)
+	{
+	    datatot(cert->authKeySerialNumber.ptr, cert->authKeySerialNumber.len, ':'
+		, buf, BUF_LEN);
+	    whack_log(RC_COMMENT, "       aserial:  %s", buf);
+	}
 	cert = cert->next;
     }
 }
@@ -2347,6 +2630,19 @@ list_crls(bool utc, bool strict)
 	whack_log(RC_COMMENT, "                 next %s %s",
 		timetoa(&crl->nextUpdate, utc),
 		check_expiry(crl->nextUpdate, CRL_WARNING_INTERVAL, strict));
+	if (crl->authKeyID.ptr != NULL)
+	{
+	    datatot(crl->authKeyID.ptr, crl->authKeyID.len, ':'
+		, buf, BUF_LEN);
+	    whack_log(RC_COMMENT, "       authkey:  %s", buf);
+	}
+	if (crl->authKeySerialNumber.ptr != NULL)
+	{
+	    datatot(crl->authKeySerialNumber.ptr, crl->authKeySerialNumber.len, ':'
+		, buf, BUF_LEN);
+	    whack_log(RC_COMMENT, "       aserial:  %s", buf);
+	}
+
 	crl = crl->next;
     }
     unlock_crl_list("list_crls");
