@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char spi_c_version[] = "RCSID $Id: spi.c,v 1.4 2003/06/07 01:31:59 ken Exp $";
+char spi_c_version[] = "RCSID $Id: spi.c,v 1.2 2003/09/15 12:57:41 jjo Exp $";
 
 #include <asm/types.h>
 #include <sys/types.h>
@@ -984,7 +984,7 @@ main(int argc, char *argv[])
 					       "required %d by encrypt algo string=\"%s\"\n",
 				       program_name, 
 				       keylen,
-				       (int) esp_info->esp_ealg_keylen,
+				       esp_info->esp_ealg_keylen,
 				       alg_string);
 			       exit(1);
 
@@ -1696,17 +1696,15 @@ main(int argc, char *argv[])
 
 /*
  * $Log: spi.c,v $
- * Revision 1.4  2003/06/07 01:31:59  ken
- * GCC 3.3 (SuSE 8.2) compile fix from dhr
+ * Revision 1.2  2003/09/15 12:57:41  jjo
+ * . alg-0.8.1_rc10 merged into sfs-2.02+x509-1.4.6
  *
- * Revision 1.3  2003/05/25 21:32:25  ken
- * cast to (int) for 64bit complainace
+ * Revision 1.1.1.1  2003/09/12 19:03:34  ken
+ * Stock 2.02
  *
- * Revision 1.2  2003/05/09 17:41:54  jjo
- * . Merged alg-0.8.1rc7
+ * Revision 1.99  2003/06/07 16:42:10  dhr
  *
- * Revision 1.1.1.1  2003/05/05 11:46:15  ken
- * Initial Import of v2.00
+ * adjust spi.c to conform to stronger type checking of GCC 3.3
  *
  * Revision 1.98  2003/01/30 02:33:07  rgb
  *
