@@ -1,11 +1,11 @@
 #
-# $Id: net.road.sh,v 1.1.1.1 2003/05/05 11:46:15 ken Exp $
+# $Id: net.road.sh,v 1.1.1.1 2003/09/12 19:03:36 ken Exp $
 #
 if [ -n "$UML_public_CTL" ]
 then
     net_eth0="eth0=daemon,10:00:00:ab:cd:02,unix,$UML_public_CTL,$UML_public_DATA";
 else
-    net_eth0="eth0=mcast,10:00:00:64:64:23,239.192.1.2,31200"
+    net_eth0="eth0=mcast,10:00:00:96:96:A1,239.192.1.2,31200"
 fi
 
 net="$net_eth0"
