@@ -1,5 +1,9 @@
 #!/bin/sh
 
+#
+# $Id: functions.sh,v 1.1.1.1 2003/09/12 19:05:30 ken Exp $
+#
+
 preptest() {
     local testdir="$1"
     local testtype="$2"
@@ -25,8 +29,50 @@ preptest() {
         echo "Error: TEST_TYPE differs.  Check agreement of TESTLIST and testparams.sh"
 	exit 1
     fi
+
+    # get rid of any pluto core files.
+    if [ -z "$XHOST_LIST" ]
+    then
+	XHOST_LIST="EAST WEST JAPAN"
+    fi
+
+    export XHOST_LIST
+
+    # Xhost script takes things from the environment.
+    for host in $XHOST_LIST
+    do
+	ROOT=$POOLSPACE/$host/root
+	rm -f $ROOT/var/tmp/core
+    done
 }
 
+lookforcore() {
+    local testdir="$1"
+
+    cd $testdir
+    source testparams.sh
+
+    # get rid of any pluto core files.
+    if [ -z "$XHOST_LIST" ]
+    then
+	XHOST_LIST="EAST WEST JAPAN"
+    fi
+
+    export XHOST_LIST
+
+    # Xhost script takes things from the environment.
+    for host in $XHOST_LIST
+    do
+	ROOT=$POOLSPACE/$host/root
+	if [ -f $ROOT/var/tmp/core ]
+	then
+	    mv $ROOT/var/tmp/core OUTPUT/pluto.$host.core
+	    echo "pluto.$host.core "
+        fi
+    done
+}
+    
+
 verboseecho() {
     if [ -n "${NETJIGVERBOSE}" ]
     then
@@ -219,6 +265,38 @@ foreach_ref_console() {
     done
 }
 
+roguekill() {
+    REPORT_NAME="$1"
+
+    if [ -n "$REGRESSRESULTS" ]
+    then
+	rm -f $REGRESSRESULTS/$REPORT_NAME/roguelist.txt
+	mkdir -p $REGRESSRESULTS/$REPORT_NAME
+    fi
+
+    # search for rogue UML
+    for i in `grep -s -l linux /proc/[1-9]*/cmdline`
+    do
+	local pdir=`dirname "$i"`
+	if [ -r $pdir/environ ]
+	then
+  	  if strings $pdir/environ | grep -l "^UML_BRAND=$UML_BRAND"'$'
+	  then
+		local badpid=`basename $pdir`
+		echo "KILLING ROGUE UML: $badpid `tr '\000' ' ' <$pdir/cmdline`"
+		if [ -n "$REGRESSRESULTS" ]
+		then
+       		   echo "UML pid $pdir went ROGUE" >>$REGRESSRESULTS/$REPORT_NAME/roguelist.txt
+		fi
+
+		# the cwd is a good indication of what test was being executed.
+		ls -l $pdir/cwd
+		kill -9 $badpid
+	  fi
+	fi
+    done
+}
+
 #
 # record results records the status of each test in 
 #   $REGRESSRESULTS/$REPORT_NAME/status
@@ -244,37 +322,10 @@ recordresults() {
     success="$3"
     REPORT_NAME="$4"
 
-    if [ -d "$REGRESSRESULTS" ]
-    then
-	rm -f $REGRESSSRESULTS/$REPORT_NAME/roguelist.txt
-	mkdir -p $REGRESSRESULTS/$REPORT_NAME
-    fi
+    export REGRESSRESULTS
+    roguekill $REPORT_NAME
 
-    # search for rogue UML
-    for i in `grep -s -l linux /proc/[1-9]*/cmdline`
-    do
-	local pdir=`dirname "$i"`
-	if [ ! -r $pdir/environ ]
-	then
-	    continue;
-	fi
-
-	if strings $pdir/environ | grep -l "^UML_BRAND=$UML_BRAND"'$'
-	then
-		local badpid=`basename $pdir`
-		echo "KILLING ROGUE UML: $badpid `tr '\000' ' ' <$pdir/cmdline`"
-		if [ -n "$REGRESSRESULTS" ]
-		then
-       		   echo "UML pid $pdir went ROGUE" >>$REGRESSSRESULTS/$REPORT_NAME/roguelist.txt
-		fi
-
-		# the cwd is a good indication of what test was being executed.
-		ls -l $pdir/cwd
-		kill -9 $badpid
-	fi
-    done
-
-    if [ -d "$REGRESSRESULTS" ]
+    if [ -n "$REGRESSRESULTS" ]
     then
 	rm -rf $REGRESSRESULTS/$REPORT_NAME
 	mkdir -p $REGRESSRESULTS/$REPORT_NAME
@@ -298,6 +349,13 @@ recordresults() {
 	    return;;
          esac
 
+	 # fail if there was a core file!
+	 cores=`( lookforcore $testname )`
+	 if [ ! -z "$cores" ]
+	 then
+	    success="core";
+	 fi
+
 	 echo Recording $success to $REGRESSRESULTS/$REPORT_NAME/status
 	 echo $success >$REGRESSRESULTS/$REPORT_NAME/status
 	 echo console=$console >>$REGRESSRESULTS/$REPORT_NAME/status
@@ -1067,7 +1125,13 @@ do_module_compile_test() {
 
     # make this name absolute.
     MODULE_DEF_INCLUDE=`pwd`/$MODULE_DEF_INCLUDE
-    MODULE_DEFCONFIG=`pwd`/$MODULE_DEFCONFIG
+
+    if [ -z "${MODULE_DEFCONFIG}" ]
+    then
+	MODULE_DEFCONFIG=/dev/null
+    else
+	MODULE_DEFCONFIG=`pwd`/$MODULE_DEFCONFIG
+    fi
 
     rm -f OUTPUT/module/ipsec.o
 
@@ -1200,6 +1264,9 @@ do_umlX_test() {
        export ${host}_INIT_SCRIPT
        export ${host}_RUN_SCRIPT
        export ${host}_RUN2_SCRIPT
+       export ${host}_RUN3_SCRIPT
+       export ${host}_RUN4_SCRIPT
+       export ${host}_RUN5_SCRIPT
        export ${host}_FINAL_SCRIPT
     done
 
@@ -1285,3 +1352,27 @@ umlXhost() {
     fi
 }
 
+#
+# $Id: functions.sh,v 1.1.1.1 2003/09/12 19:05:30 ken Exp $
+#
+# $Log: functions.sh,v $
+# Revision 1.1.1.1  2003/09/12 19:05:30  ken
+# Stock 2.02
+#
+# Revision 1.88  2003/08/21 22:38:06  mcr
+# 	sense of test was wrong for MODULE_DEFCONFIG
+#
+# Revision 1.87  2003/08/21 22:35:11  mcr
+# 	if we didn't specify a MODULE_DEFCONFIG, then use /dev/null
+#
+# Revision 1.86  2003/08/18 16:32:48  mcr
+# 	export 3,4,5 RUN script variables.
+#
+# Revision 1.85  2003/05/21 14:10:31  mcr
+# 	look for core files produced by pluto, and save them to the
+# 	OUTPUT directory, changing failure state to "core".
+#
+# Revision 1.84  2003/05/03 23:26:02  mcr
+# 	added RCS Ids.
+#
+# 
\ No newline at end of file
