@@ -1,7 +1,7 @@
 #!/usr/bin/expect --
 
 #
-# $Id: localswitches.tcl,v 1.1.1.1 2003/05/05 11:46:19 ken Exp $
+# $Id: localswitches.tcl,v 1.1.1.1 2003/09/12 19:05:30 ken Exp $
 #
 
 source $env(FREESWANSRCDIR)/testing/utils/GetOpts.tcl
@@ -11,7 +11,7 @@ set netjig_debug_opt ""
 
 set netjig_prog $env(FREESWANSRCDIR)/testing/utils/uml_netjig/uml_netjig
 
-spawn -noecho -open [open "|$netjig_prog --cmdproto -t $netjig_debug_opt 2>@stderr" w+]
+spawn $netjig_prog --cmdproto -t $netjig_debug_opt 
 set netjig1 $spawn_id
 
 newswitch $netjig1 public
@@ -22,8 +22,11 @@ foreach host $argv {
     system "$host single &"
 }
 
-puts "Waiting for netjig to die"
+puts "\nExit the netjig when you are done\n"
 
 set timeout -1
-expect -i $netjig1 eof
+interact -reset -i $netjig1 
+
+
+
 
